<?php


namespace Ron\DB;


class PDODatabase extends Database
{

    /**
     * @var \PDO
     */
    private $pdo;
    /**
     * @var string
     */
    private $dbType;

    private $connectionString;
    private $attributes;

    /**
     * @param string $connectionString
     * @param string $dbType    Lze pouzit pro zjisteni dialektu SQL (napr. mysql, interbase)
     */
    public function __construct($connectionString, $dbType = 'pdo')
    {
        $this ->connectionString = $connectionString;
        $this ->dbType = $dbType;
    }


    /**
     * @return mixed
     */
    public function getConnectionString()
    {
        return $this->connectionString;
    }

    /**
     * @return mixed
     */
    public function getAttributes()
    {
        return $this->attributes;
    }

    /**
     * @param mixed $attributes
     */
    public function setAttributes($attributes)
    {
        $this->attributes = $attributes;
    }



    /**
     * @return string
     */
    public function getDbType()
    {
        return $this ->dbType;
    }

    /**
     * @param mixed $dbType
     */
    public function setDbType($dbType)
    {
        $this->dbType = $dbType;
    }


    /**
     * @param string $user
     * @param string $password
     * @throws DBException
     */
    public function connect($user, $password)
    {
        try {
            $this->pdo = new \PDO($this->connectionString, $user, $password);
        } catch (\PDOException $e) {
            throw new DBException($this, '', $e ->getMessage(), $e, $e ->errorInfo);
        }

        if ($this ->attributes) {
            foreach ($this ->attributes as $attribute => $value)
                $this ->pdo->setAttribute($attribute, $value);
        }

        //$count = $this ->pdo ->query("SET NAMES '{$this ->codePage}'");
//        $count = $this ->pdo ->query("SET NAMES utf8");
//        if (!isset($count)) {
//            throw new \Exception('chybaaa!');
//        };
    }

    /**
     * @return bool
     */
    public function close()
    {
        return true;
    }


    public function createParameters()
    {
        return new PlaceholderParameters($this);
    }


    /**
     * @param string $sqlText
     * @param Parameters $params
     * @return Cursor
     * @throws DBException
     */
    public function executeSelect($sqlText, $params = null)
    {
        if ($params) {
            $data = $this ->pdo ->prepare($sqlText);
            if (!$data) {
                throw new DBException($this, $sqlText);
            }

            if (!$data ->execute($params ->getSqlParameters())) {
                throw new DBException($this, $sqlText);
            }
        } else {
            $data = $this->pdo->query($sqlText);
            if (!$data) {
                throw new DBException($this, $sqlText);
            }
        }

        return new PdoCursor($this, $data);
    }


    /**
     * @param string $sqlText
     * @param Parameters $params
     * @throws DBException
     */
    public function executeSql($sqlText, $params = null)
    {
        if ($params) {
            $data = $this ->pdo ->prepare($sqlText);

            if (!$data) {
                throw new DBException($this, $sqlText);
            }

            if (!$data ->execute($params ->getSqlParameters())) {
                throw new DBException($this, $sqlText);
            }

        } else {
            $count = $this->pdo->exec($sqlText);

            // count obsahuje pocet zmenenych zaznamu nebo NULL pri chybe
            if ($count === NULL ||  $count === false) {
                throw new DBException($this, $sqlText);
            }
        }
    }


    function lastAutoincrementValue()
    {
        return $this ->pdo ->lastInsertId();
    }

    /**
     * @return string
     */
    public function lastErrorText()
    {
        $result = '';

        if ($this ->pdo) {
            $errorInfo = $this->pdo->errorInfo();

            if ($errorInfo) {
                $result = 'SQLSTATE: ' . $errorInfo[0] . '<br/>';
                $result .= 'Code: ' . $errorInfo[1] . '<br/>';
                $result .= 'Message: ' . $errorInfo[2] . '<br/>';
            }
        }

        return $result;
    }

    /**
     * @throws DBException
     */
    public function beginTransaction()
    {
        $this ->executeSql('BEGIN TRAN');
    }

    /**
     * @throws DBException
     */
    public function commitTransaction()
    {
        $this ->executeSql('COMMIT TRAN');
    }

    /**
     * @throws DBException
     */
    public function rollbackTransaction()
    {
        $this ->executeSql('ROLLBACK TRAN');
    }

    /**
     * @param string $str
     * @return string
     */
    public function addSlashes($str)
    {
        return str_replace("'", "''", $str);
    }

    /**
     * @param string $text
     * @return string;
     */
    public function castWideMemo($text)
    {
        return $text;
    }

    /**
     * @param string $text
     * @return string;
     */
    public function extractYear($text)
    {
        return "EXTRACT(YEAR FROM $text)";
    }


    /**
     * @param string $str
     * @return int|null
     */
    public function stringToTimestamp($str)
    {
        if ( !isset($str) ) return null;
        if ( $str === '' ) return null;

        //echo "prevadim: $str <BR> ";
        // Maximalni hodnota pro cas je 19.1.2038 - vetsi hodnoty PHP nezvlada
        if (preg_match ("/2(\d\d\d)/", $str, $m) && ($m[1] >= 38))
        {
            return 0x7fffffff;
        }

        // Pokusi se prevest datum primo. Pokud se to nepovede tak funkce
        // vraci -1 a musi se to prechrchlat
        $result = strtotime($str);
        return $result;
    }

    /**
     * @param int $timestamp
     * @return string
     */
    public function timestampToString($timestamp)
    {
        if (!isset($timestamp)) return 'null';

        return "'" . date("Y-m-d H:i:s", $timestamp) . "'";
    }

    /**
     * Prevede timestamp na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function dateToString($timestamp)
    {
        if (!isset($timestamp)) return 'null';

        return "'" . date("Y-m-d", $timestamp) . "'";
    }

    /**
     * prevede cas na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function timeToString($timestamp)
    {
        throw new \Exception('Function PDODatabase::timeToString is not implemented!');
    }


    public function timestampToParam($timestamp)
    {
        if (!isset($timestamp)) return null;

        return date('Y-m-d H:i:s', $timestamp);
    }

    public function dateToParam($timestamp)
    {
        if (!isset($timestamp)) return null;

        return date('Y-m-d', $timestamp);
    }

    public function timeToParam($timestamp)
    {
        throw new \Exception('Function PDODatabase::timeToParam is not implemented!');
    }


    /**
     * prevede retezec z databaze na denni cas - vraci pocet hodin (realne cislo)\
     * napr "... 7:30" => 7.5
     * @param string $str
     * @return integer
     */
    public function stringToTimeOfDay($str)
    {
        if ( !isset($str) ) return null;

        $timestr = strrchr($str, ' '); // vraci vse za posledni mezerou

        $poma = explode (':', $timestr);

        return $poma[0] + $poma[1] / 60.0 + $poma[2] / 3600.0 ;
    }


}



class PdoCursor extends Cursor
{
    /**
     * @var \PDOStatement
     */
    protected $data;


    /**
     * @param PdoDatabase $database
     * @param \PDOStatement $data
     */
    function __construct($database, $data)
    {
        parent::__construct($database);
        $this->data = $data;
    }


    /**
     * @return \PDO
     */
    public function getData()
    {
        return $this->data;
    }

    /**
     * @return array|bool
     */
    public function fetch()
    {
        return $this ->data ->fetch(\PDO::FETCH_NUM);
//        $result = $this ->data ->fetch(\PDO::FETCH_NUM);
//        var_dump($result);
//        return $result;
    }

    /**
     * Vraci pocet nactenych zaznamu
     * @return int
     */
    public function numRows()
    {
        return $this ->data ->rowCount(); // pozor! u nekterych databazi nemusi fungovat!
    }

}



require('_appdbset.php');

function createDefaultDatabase() {
    global $db_connectionstring;
    global $pdo_dbtype;

    $dbType = $pdo_dbtype ?: 'pdo';

    $database = new PDODatabase($db_connectionstring, $dbType);

    global $pdo_attributes;
    $database ->setAttributes($pdo_attributes);

    return $database;
}

