<?php

/**
 * UTF-8
 */

namespace Ron\Personalistika;

use Exception;
use Ron\Browser;
use Ron\BrowserField;
use Ron\DB\Cursor;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\GuiUtils;
use Ron\HttpUtils;
use Ron\MVC\DataOutputObject;
use Ron\MVC\OutputObject;
use Ron\RonPortal\Prava;
use Ron\DB\Database;
use Ron\StringUtils;
use function Ron\ronText;
use Ron\TextManager;

require_once('ron/browser.php');

class SmazaniDovednostiField extends BrowserField
{
    private $osobaId;
    private $osobaRC;
    private $dovednostIndex;
    private $dovednostPopisIndex;
    private $urovenIndex;
    private $datumZiskaniIndex;

    /**
     * SmazaniDovednostiField constructor.
     * @param null|Database $database
     * @param int $osobaId
     * @param string $osobaRC
     * @param int $dovednostIndex
     * @param int $dovednostPopisIndex
     * @param int $urovenIndex
     * @param int $datumZiskaniIndex
     */
    public function __construct($database, $osobaId, $osobaRC, $dovednostIndex, $dovednostPopisIndex, $urovenIndex, $datumZiskaniIndex)
    {
        parent::__construct($database);

        $this->osobaId = $osobaId;
        $this->osobaRC = $osobaRC;
        $this->dovednostIndex = $dovednostIndex;
        $this->dovednostPopisIndex = $dovednostPopisIndex;
        $this->urovenIndex = $urovenIndex;
        $this->datumZiskaniIndex = $datumZiskaniIndex;
    }

    /** Zobrazi odkaz pro smazani ziskane dovednosti. Smazani je mozne jen pokud neni pripojen dokument.
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        require_once('ron/httputils.php');

        if (!DovednostiController::jePripojenDokumentDovednosti($this->database, $this->osobaRC, $rowData[$this->dovednostIndex], $rowData[$this->urovenIndex],
            $this->database->stringToTimestamp($rowData[$this->datumZiskaniIndex]))) {

            return GuiUtils::changeLocationQuest('<i class="inline-glyph glyph-button fa glyph-delete" title="' . ronText('Zrusit_zaznam') . '"></i>',
                sprintf(ronText('Dovednosti_smazat'), $rowData[$this->dovednostPopisIndex]),
                '?menuid=maticedovednosti&action=zrusitdovednost&osoba=' . $this->osobaId . '&dovednost=' . HttpUtils::urlEncode($rowData[$this->dovednostIndex]) .
                '&uroven=' . $rowData[$this->urovenIndex] . '&datumziskani=' . $this->database->stringToTimestamp($rowData[$this->datumZiskaniIndex])
            );
        }

        return '';
    }
}

/**
 * Seznam platnych dovednosti osoby
 */
class DovednostiBrowser extends Browser implements OutputObject
{
    private $pravaEditace;
    private $pravaZobrazeni;
    private $pravaSmazani;
    
    private $idOsoby;

    /**
     * @param Database $database
     * @param string $url
     * @param $idOsoby
     * @throws DBException
     * @throws Exception
     */
    function __construct($database, $url, $idOsoby)
    {
        parent::__construct($database);
        
        $this ->pravaEditace = Prava::vratPrava('doved_edit');
        $this ->pravaZobrazeni = Prava::vratPrava('doved_ziskane');
        $this ->pravaSmazani = Prava::vratPrava('doved_smazani');
        
        Prava::nastavObdobiProPrava(time(), time());
        
        $this ->idOsoby = $idOsoby;
        
        $this->title = ronText('Dovednosti');
        
        $this->browserUrl = $url;
        
        $this->sqlSelectPart =
            "DovednostOsoby.KodDovednosti, DovednostOsoby.KodUrovne, DovednostOsoby.DatumZiskani, DovednostOsoby.PlatnostDo, ". //0 - 3
            "Dovednost.Popis, UrovenDovednosti.Popis, DovednostOsoby.Insert_UserName, DovednostOsoby.Insert_Timestamp, ".  //4 - 7
            "DovednostOsoby.Update_UserName, DovednostOsoby.Update_Timestamp, " . $database->castWideMemo('DovednostOsoby.Poznamka') .  //8 - 10
            ", (SELECT Popis FROM S_SpecUrovneDovednosti WHERE Kod = DovednostOsoby.SpecifikaceUrovne) as SpecifikaceUrovne, " . //11
            $database->castWideMemo("UrovenDovednosti.RozsirenyPopis"); //12

        $this->sqlFromPart =
            "DovednostOsoby, UrovenDovednosti, Dovednost ";

        $this->sqlWherePart =
            " (UrovenDovednosti.KodDovednosti = DovednostOsoby.KodDovednosti) " .
            " AND (UrovenDovednosti.KodUrovne = DovednostOsoby.KodUrovne) " .
            " AND (UrovenDovednosti.KodDovednosti = Dovednost.Kod) "; 

        $this->addFields([
            [BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_SQL_NAME => 'Dovednost.Popis', BROWSER_FIELD_HEADER_TEXT => 'Dovednost',
                BROWSER_FIELD_ALIGN => 'left', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_SQL_INDEX => 5, BROWSER_FIELD_SQL_NAME => 'UrovenDovednosti.Popis', BROWSER_FIELD_HEADER_TEXT => 'Dovednosti_uroven_dovednosti',
                BROWSER_FIELD_ALIGN => 'left', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_SQL_INDEX => 1, BROWSER_FIELD_SQL_NAME => 'DovednostOsoby.KodUrovne', BROWSER_FIELD_HEADER_TEXT => 'Kod',
                BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_SQL_INDEX => 11, BROWSER_FIELD_SQL_NAME => 'SpecifikaceUrovne', BROWSER_FIELD_HEADER_TEXT => 'Dovednosti_specifikace_urovne',
                BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_SQL_INDEX => 2, BROWSER_FIELD_SQL_NAME => 'DovednostOsoby.DatumZiskani', BROWSER_FIELD_HEADER_TEXT => 'Date',
                BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_FORMAT => 'finedate', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_SQL_INDEX => 3, BROWSER_FIELD_SQL_NAME => 'DovednostOsoby.PlatnostDo', BROWSER_FIELD_HEADER_TEXT => 'Platnost_do',
                BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_FORMAT => 'finedate', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_SQL_INDEX => 6, BROWSER_FIELD_SQL_NAME => 'DovednostOsoby.Insert_UserName', BROWSER_FIELD_HEADER_TEXT => 'Dovednosti_vlozil_uzivatel',
                BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_SQL_INDEX => 7, BROWSER_FIELD_SQL_NAME => 'DovednostOsoby.Insert_Timestamp', BROWSER_FIELD_HEADER_TEXT => 'Date',
                BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_FORMAT => 'finedatetime', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_SQL_INDEX => 8, BROWSER_FIELD_SQL_NAME => 'DovednostOsoby.Update_UserName', BROWSER_FIELD_HEADER_TEXT => 'Dovednosti_aktualizoval_uzivatel',
                BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_ORDERING => 1],
            [BROWSER_FIELD_SQL_INDEX => 9, BROWSER_FIELD_SQL_NAME => 'DovednostOsoby.Update_Timestamp', BROWSER_FIELD_HEADER_TEXT => 'Date',
                BROWSER_FIELD_ALIGN => 'right', BROWSER_FIELD_FORMAT => 'finedatetime', BROWSER_FIELD_ORDERING => 1],
            
        ]);

        if ($this->pravaSmazani->jsouNejakePrava()) {
            $params = $this->database->createParameters();
            $osobaRC = DBUtils::getSqlValue($this->database, 'SELECT RC FROM PracovniPomer WHERE ID = ' . $params->addParameter($this->idOsoby), $params);
            if ($this->pravaSmazani->lzeZobrazitRC($osobaRC)) {
                $this->addField(new SmazaniDovednostiField($this->database, $this->idOsoby, $osobaRC, 0, 4, 1, 2));
            }
        }

        $this->defaultSort = 5;
        $this->primaryField = 0;
    }


    function getSQLFilter()
    {
        return
            " EXISTS(SELECT PracovniPomer.RC FROM PracovniPomer WHERE PracovniPomer.RC = DovednostOsoby.RC AND PracovniPomer.Id = " . (int)$this->idOsoby .
            " AND ((" . $this->pravaEditace->sqlFiltr('PracovniPomer', time(), time()) . ") OR " .
            "      (" . $this->pravaZobrazeni->sqlFiltr('PracovniPomer', time(), time()) . "))" .
            ") ";
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);
        
        if ($values[10] or $values[12]) {
            // poznamka
            echo "<tr class='$className'>";
            echo '<td colspan="' . ($this->columnCount) . '">';
            echo '<span class="poznamka browserpoznamka do_poznamka">' . StringUtils::htmlTextConvert($values[10]) . '</span>';
            echo '<span class="poznamka browserpoznamka ud_poznamka"><strong>' . ronText('Dovednosti_rozsireny_popis_urovne') . '</strong>: ' . StringUtils::htmlTextConvert($values[12])  . '</span>';
            echo '</td></tr>';
        }
    }

    function renderHeader()
    {
        parent::renderHeader();

        echo "<a href='?menuid=dovednosti&action=downloadcsv&osoba=$this->idOsoby'>" . ronText('Dovednosti_csv') . "</a>";
    }

    public function outputHeader()
    {
        $poms = '';

        foreach ($this ->fields as $i => $field) {
            if ($poms) {
                $poms .= ';';
            }

            $poms .= $field->header;
        }

        return $poms . "\n";
    }

    public function outputLine($values)
    {
        $poms = '';

        foreach ($this ->fields as $i => $field) {
            $value = null;

            if ($field ->sqlFieldIndex >= 0  &&  isset($values[$field ->sqlFieldIndex])) {
                $value = $values[$field ->sqlFieldIndex];
            }

            if ($poms) {
                $poms .= ';';
            }

            $poms .= ($field->format ? DBUtils::getFormatedValue($this ->database, $field->format, $value) : $value);
        }

        return $poms;
    }

    /**
     * @param Cursor $cursor
     * @return string
     * @throws Exception
     */
    public function outputCursor($cursor)
    {
        $data = '';

        if (!$cursor) {
            throw new Exception('Chyba v metode browser::outputCursor - nejsou nactena data!');
        }

        while ($row = $cursor ->fetch()) {
            if ($this ->enableLine($row)) {
                $data .= $this ->outputLine($row) . "\n";
            }
        }

        return $data;
    }

    /**
     * @throws Exception
     */
    public function output()
    {
        $data = $this->outputHeader() . $this->outputCursor($this->cursor);

        $csv = new DataOutputObject($data, 'dovednosti.csv', TextManager::getInstance() ->getCharSet());

        $csv->output();

        return;
    }

}
