<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\Controls;

use ArrayAccess;
use Exception;
use Ron\DB\Cursor;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\Http\App;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;


class BrowserFilter
{
    /** @var string     Jedinecny kod filtru */
    public $code;
    /** @var string */
    public $caption;
    /** @var string */
    public $title;
    /** @var string */
    public $sql;

    /** @var bool */
    public $active = false;
}



class Browser implements ViewObject
{
    /** @var  Database */
    var $database;

    /** @var string     Kod browseru - slouzi pro ukladani nastaveni do SESSION */
    var $code;
    /** @var string     Nadpis browseru */
    var $title;
    /** @var string     Id tabulky v html strance */
    var $tableId;

    /** @var BrowserField[] */
    var $fields;

    /** @var string     Select cast vyrazu (jmena poli) - pokud neni zadana, sestavi se automaticky z poli */
    var $sqlSelectPart;
    /** @var string     From cast sql vyrazu */
    var $sqlFromPart;
    /** @var string|SqlStatement  Where cast sql vyrazu */
    var $sqlWherePart;
    /** @var string     Pokud neni zadano, radi se podle $defaultSort */
    var $sqlOrderPart;
    /** @var string */
    var $sqlGroupByPart;
    /** @var bool       Priznak, jestli se ma do selektu pridavat DISTINCT */
    var $distinct;

    /** @var int        Implicitni razeni v browseru - cislo pole (zaporne pro sestupne razeni) */
    var $defaultSort;
    /** @var int        Maximalni pocet zaznamu na strance - pokud je zadan, pokusi se strankovat (nemusi fungovat na vsech databazich!) */
    var $maxPageRows = null;
    /** @var int        Index zobrazene stranky */
    var $pageIndex = null;
    //var $maxLinesCount; // maximalni pocet radku, ktery se zobrazi. Pokud jich bude vice, tak se strankovat

    /** @var  int       Index primarniho pole v seznamu poli - slouzi pro identifikaci zaznamu pro editaci */
    var $primaryField;
    /** @var  bool      Nastavit na true, pokud se nema zobrazovat header */
    var $noHeader;
    /** @var  string    Odkaz na akci, ktera se provede po kliknuti na radek */
    var $rowClickLink;
    /** @var  bool      Jestli se ma zobrazovat paticka seznamu */
    var $showFooter;
    /** @var bool       Jestli se maji zobrazit zpravy MessageManageru */
    var $showMessages;

    /** @var int        Nastavi se v load pomoci getOrderFieldIndex */
    var $orderFieldIndex;
    /** @var int        Pomocna promenna - pocet sloupcu - urci se pri vykreslovani hlavicky */
    var $columnCount;

    /** @var string     css trida pro zobrazeni, defaultne "browser" */
    var $cssClass;

    /** @var string     Url adresa pro zobrazeni tohoto browseru - pouzije se pro sestaveni odkazu pro razeni atd. */
    var $browserUrl;

    /** @var Cursor  Data nactena pomoci load */
    var $cursor;

    /** @var string     CSS, ktery se vyrenderuje pred seznamem. Lze nastavit napriklad uzivatelskou konfiguraci. */
    var $cssStyle;

    /** @var string[]   Asociativni pole HTML parametru */
    var $tableHtmlParams = [];

    /** @var BrowserFilter[]    Pole filtru (definovanych napr. pomoci XML konfigurace), ktere se maji zobrazit */
    var $sqlFilters = [];


    /**
     * @param Database $database
     */
    public function __construct($database)
    {
        $this ->database = $database;
        $this ->cssClass = 'browser rowOver';
        $this ->showMessages = true;
    }


    /**
     * Prida sloupec. Pokud je zadan primo objekt BrowserField, tak jej vlozi. Jinak vytvori novy objekt zadaneho typu nebo se zadanymi parametry
     * @param BrowserField|array|string $field  Objekt BrowserField nebo asociativni pole podle ktereho se vytvori novy objekt nebo typ pole
     * @return BrowserField
     */
    public function addField($field = null)
    {
        if (is_array($field)) {
            $field = new BrowserField($this ->database, $field);
        } else if (!$field  ||  !($field instanceof BrowserField)) {
            $newField = new BrowserField($this ->database);
            $newField ->setType($field ?: BrowserField::FIELD_TYPE_DEFAULT);
            $field = $newField;
        }

        $this ->fields[] = $field;

        return $field;
    }


    /**
     * Najde pole se zadanym Id
     * @param string $fieldId
     * @return int
     */
    public function findFieldIndex($fieldId)
    {
        foreach ($this ->fields as $index => $field) {
            if ($field ->getId() === $fieldId) {
                return $index;
            }
        }

        return null;
    }

    /**
     * Vraci pole se zadanym kodem
     * @param $code
     * @return null|int
     */
    public function findFieldIndexByCode($code)
    {
        foreach ($this ->fields as $index => $field) {
            if ($field ->code === $code) {
                return $index;
            }
        }

        return null;
    }

    /**
     * Prida BrowserFields ze zadaneho pole $fields - to mohou byt primo objekty BrowserField nebo pole pro init()
     * @param array $fields
     */
    public function addFields($fields)
    {
        foreach($fields as $field) {
            $this ->addField($field);
        }
    }


    /**
     * Presune zadane pole ($fieldIndex) pred pole s kodem $before, nebo za pole s kodem $after
     * @param int $fieldIndex
     * @param string $before
     * @param string $after
     */
    public function moveField($fieldIndex, $before, $after)
    {
        if ($before) {
            $field2Index = $this ->findFieldIndexByCode($before);

            if (isset($field2Index)) {
                if ($field2Index === $fieldIndex + 1) {
                    // uz je na pozadovane pozici
                    return;
                }

                $removed = array_splice($this ->fields, $fieldIndex, 1);

                if ($field2Index > $fieldIndex) {
                    // po odstraneni pole se index zmenil
                    $field2Index --;
                }

                array_splice($this ->fields, $field2Index, 0, $removed);
                return;
            }
        }

        if ($after) {
            $field2Index = $this ->findFieldIndexByCode($after);

            if (isset($field2Index)) {
                if ($field2Index === $fieldIndex - 1) {
                    // uz je na pozadovane pozici
                    return;
                }

                if ($field2Index > $fieldIndex) {
                    // po odstraneni pole se index zmenil
                    $field2Index --;
                }

                $removed = array_splice($this ->fields, $fieldIndex, 1);
                array_splice($this ->fields, $field2Index + 1, 0, $removed);
                return;
            }
        }
    }


    public function createNewField($colSettings)
    {
        if (!isset($colSettings['sqlFieldName'])) {
            return null;
        }

        $field = $this ->addField(isset($colSettings['type']) ? $colSettings['type'] : BrowserField::FIELD_TYPE_DEFAULT);
        $field ->loadConfigurationArray($colSettings);

        // na konec bylo pridano nove pole => posune indexy vsech ostatnich poli, ktera maji zaporny index
        foreach ($this ->fields as $f) {
            if ($f ->sqlFieldIndex < 0) {
                $f ->sqlFieldIndex--;
            }
        }

        if (!isset($field ->sqlFieldIndex)) {
            $field ->sqlFieldIndex = -1; // aktualne je to posledni pole odzadu
        }

        if ($this ->sqlSelectPart) {
            // doplni pole do SELECT
            $this ->sqlSelectPart .= ', ' . $field ->sqlFieldName;
        }

        return count($this->fields) - 1;
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'style' :
                $this ->cssStyle = ($this ->cssStyle ?: '') . $valueText;
                break;

            case 'showFooter' :
            case 'noHeader' :
            case 'distinct' :
            case 'defaultSort' :
            case 'maxPageRows' :
            case 'pageIndex' :
            case 'showMessages' :
                $this ->{$key} = (int)$valueText;
                break;

            case 'tableId' :
            case 'sqlSelectPart' :
            case 'sqlFromPart' :
            case 'sqlWherePart' :
            case 'sqlOrderPart' :
            case 'sqlGroupByPart' :
            case 'rowClickLink' :
            case 'cssClass' :
            case 'browserUrl' :
                $this ->{$key} = $valueText;
                break;

            case 'sqlFilter' :
                if ($valueText) {
                    $this ->sqlWherePart .= ($this ->sqlWherePart ? ' AND ' : '') . $valueText;
                }
                break;

            case 'title' :
                $this ->{$key} = TextManager::_($valueText);
                break;
        }
    }

    /**
     * @param ArrayAccess $colSettings
     */
    public function loadColumnConfiguration($colSettings)
    {
        $fieldIndex = isset($colSettings['code'])
            ? $this ->findFieldIndexByCode($colSettings['code'])
            : null;

        if (isset($fieldIndex)) {
            $this->fields[$fieldIndex]->loadConfigurationArray($colSettings);
        } else {
            // zkusi vytvorit uplne nove pole
            $fieldIndex = $this ->createNewField($colSettings);
        }

        if (isset($fieldIndex)) {
            if ($colSettings['before']  ||  $colSettings['after']) {
                $this ->moveField($fieldIndex, $colSettings['before'], $colSettings['after']);
            }
        }
    }

    /**
     * @param ArrayAccess $columnsConfArray
     */
    public function loadColumnsConfiguration($columnsConfArray)
    {
        if (!$columnsConfArray) {
            return;
        }

        foreach ($columnsConfArray as $colSettings) {
            $this ->loadColumnConfiguration($colSettings);
        }
    }


    /**
     * Nacte konfiguraci ze zadaneho xmlTextu
     * @param string $xmlText
     */
    public function loadConfigurationXml($xmlText)
    {
        if (!$xmlText) {
            return;
        }

        $a = XmlLoader::loadXmlToArray($xmlText,
            [XmlLoader::PARAM_CONVERT_FROM_UTF8 => true, XmlLoader::PARAM_INCLUDE_ATTRIBUTES => true]
        );

        if (isset($a['filter'])) {
            foreach ($a['filter'] as $filterSettings) {
                $filter = new BrowserFilter();
                $filter ->code = $filterSettings['code'];
                $filter ->caption = $filterSettings['caption'];
                $filter ->title = $filterSettings['title'];
                $filter ->sql = $filterSettings['#text'];

                $this ->sqlFilters[] = $filter;
            }
        }

        foreach ($a as $key => $value) {
            if ($value instanceof ArrayAccess) {
                // hodnoty definovane pomoci vnorenych tagu spoji do jednoho textu (napr. style)
                $valueText = '';
                foreach ($value as $valA) {
                    if (isset($valA['#text'])) {
                        $valueText .= ($valueText ? "\n" : '') . $valA['#text'];
                    }
                }
            } else {
                $valueText = $value;
            }

            $this ->loadConfigurationItem($key, $value, $valueText);
        }

        if (isset($a['column'])) {
            $this ->loadColumnsConfiguration($a['column']);
        }
    }


    /**
     * Vraci ORDER BY klauzuli
     * @return string
     */
    public function getOrderSQLPart()
    {
        if ($this ->sqlOrderPart) {
            return $this->sqlOrderPart;
        }

        if (!isset($this ->orderFieldIndex)) {
            $this ->orderFieldIndex = 1;
        } else if (!$this ->orderFieldIndex) {
            $this ->orderFieldIndex = 1;
        } else if ($this ->orderFieldIndex > count($this->fields)) {
            $this ->orderFieldIndex = 1;
        } else if ($this ->orderFieldIndex < -count($this->fields)) {
            $this ->orderFieldIndex = 1;
        }

        $field = $this ->fields[abs($this ->orderFieldIndex) - 1];
        if ($this ->orderFieldIndex < 0) {
            if ($field ->sortSQLDesc) {
                return $field ->sortSQLDesc;
            }
            if ($field ->sortSQL) {
                return $field ->sortSQL . ' DESC';
            }
            if ($field ->sqlFieldName) {
                return $field ->sqlFieldName . ' DESC';
            }
        } else {
            if ($field ->sortSQL) {
                return $field ->sortSQL;
            }

            return $field ->sqlFieldName;
        }

        return '';
    }


    /**
     * Vraci cast retezce, ktera se vlozi do WHERE casti dotazu
     * muze vracet napriklad filtr podle zvolenych filtru (viz renderFilter)
     * @return string|SqlStatement
     */
    public function getSQLFilter()
    {
        return '';
    }


    /**
     * Vraci jmena poli pro SQL dotaz (tj. SELECT cast dotazu) - pokud neni primo zadana, tak ji sestavi z nazvu poli
     * @return string
     */
    public function getSelectSQLPart()
    {
        if ($this ->sqlSelectPart) {
            return $this ->sqlSelectPart;
        }

        $result = '';

        $sqlFieldIndex = 0;

        foreach ($this ->fields as $i => $field) {
            if ($field ->sqlFieldName) {
                $result .= ($result ? ', ' : '') . $field ->sqlFieldName;

                $this ->fields[$i] ->sqlFieldIndex = $sqlFieldIndex;

                $sqlFieldIndex++;
            } else {
                $field ->sqlFieldIndex = null;
            }
        }

        return $result;
    }


    /**
     * @param string|SqlStatement   $sqlFilter   Dodatecny sql filtr
     * @return string|SqlStatement
     */
    public function getSQL($sqlFilter = '')
    {
        $orderBy = $this ->getOrderSQLPart();

        $wherePart = SQLUtils::concatenateWherePartsOfSql(
            [$this ->sqlWherePart, $sqlFilter, $this ->getSQLFilter()]);

        return SQLUtils::createSql(
            $this ->getSelectSQLPart(), $this ->sqlFromPart,
            $wherePart, $orderBy, $this ->distinct, $this ->sqlGroupByPart);
    }



    /**
     * Vraci index pole, podle ktereho se ma radit
     * @return int
     */
    public function getOrderFieldIndex()
    {
        $orderFieldIndex = $this->defaultSort;
        $defaultSortStr = $this ->defaultSort
            ? (($this ->defaultSort < 0 ? '-' : '') . $this ->fields[abs($this ->defaultSort) - 1] ->getId())
            : '';

        if (isset($_GET['sort'])) {
            // v URL nastaveno nove razeni
            $newFieldIndex = $_GET['sort']; // pozor, cislovani od 1 (aby bylo mozne rozlisit sestupne/vzestupne)
            if ($newFieldIndex  &&  abs($newFieldIndex) <= count($this ->fields)  &&  $this ->fields[abs($newFieldIndex) - 1] ->ordering) {
                // OK, pole existuje a je mozne podle nej radit
                $orderFieldIndex = $newFieldIndex;

                if ($this ->code) {
                    // zapise do nastaveni uzivatele
                    App::getInstance() ->storeUserSettings(
                        'browser_' . $this->code . '_sort',
                        ($newFieldIndex < 0 ? '-' : '') . $this ->fields[abs($newFieldIndex) - 1] ->getId(),
                        $defaultSortStr );

                    // a do session
                    $_SESSION['BROWSERSSORT'][$this->code] = $newFieldIndex;
                }
            }
        } else if ($this->code) {
            if (isset($_SESSION['BROWSERSSORT'][$this->code])) {
                // uz ulozeno v session
                $orderFieldIndex = $_SESSION['BROWSERSSORT'][$this->code];
            } else {
                // jeste neni v session - nacte z nastaveni uzivatele
                $sortStr = App::getInstance()->getUserSettings('browser_' . $this->code . '_sort', 'x' . $defaultSortStr);

                $minus = false;
                if ($sortStr && $sortStr[0] === '-') {
                    $minus = true;
                    $sortStr = substr($sortStr, 1);
                }
                $newFieldIndex = $this->findFieldIndex($sortStr); // pozor, cislovani od 0
                if (isset($newFieldIndex)) {
                    $orderFieldIndex = $newFieldIndex + 1;
                    if ($minus) {
                        $orderFieldIndex = -$orderFieldIndex;
                    }
                }
            }
        }


//        // Ulozi nastaveni do session
//        if ($this->code)
//            $_SESSION['BROWSERSSORT'][$this ->code] = $orderFieldIndex;

        return $orderFieldIndex;
    }


    /**
     * Vraci true, pokud ma byt radek zobrazen
     * @param array $values
     * @return bool
     */
    public function enableLine($values)
    {
        return true;
    }


    /**
     * Nacte data
     * @param string $sql
     * @throws Exception
     */
    public function loadSQL($sql) // , $firstRow  * @param int  $firstRow          Prvni radek, ktery se ma zobrazit
    {
        MessageManager::addDebugLog('Browser::loadSQL ' . ($this ->code ? "({$this ->code}): " : ': ') . ($sql instanceof SqlStatement ? $sql ->sqlText : $sql));

        $this ->cursor = $this ->database ->select($sql);
    }



    /**
     * Nacte data
     * @param string|SqlStatement $sqlFilter     Nepovinny filtr, ktery se prida do where pomoci AND
     * @throws Exception
     */
    public function load($sqlFilter = '')
    {
        $this ->orderFieldIndex = $this ->getOrderFieldIndex();
        $sqlText = $this ->getSQL($sqlFilter);

        $this ->loadSQL($sqlText);
    }


    public function renderSqlFilters()
    {
        if (!$this ->sqlFilters) {
            return;
        }

        foreach ($this ->sqlFilters as $filter) {
            // TODO: dodelat konfigurovatelne SQL filtry
        }

    }


    /**
     * Zobrazi nad seznamem filtry (napr vyber typu, stavu, ...)
     */
    public function renderFilter()
    {
        if ($this ->showMessages) {
            MessageManager::getInstance()->showMessages();
        }
    }


    /**
     */
    public function renderHeader()
    {
        if ($this ->cssStyle) {
            echo "<style>{$this ->cssStyle}</style>\n";
        }

        if ($this ->title) {
            echo "<h1>{$this ->title}</h1>";
        }

        $this ->renderFilter();
    }


    /**
     * Zobrazi hlavicku tabulky
     */
    public function renderTableHeader()
    {
        echo '<thead class="floating-table-header">';
        echo '<tr>';

        $this ->columnCount = 0;
        $colSpan = 0;

        foreach ($this ->fields as $i => $field) {
            if ($field ->visible) {
                if ($colSpan > 1) {
                    // tento header nezobrazuje, protoze predchozi byl pres vice sloupcu a zasahuje i do tohoto sloupce
                    $colSpan--;
                } else {
                    $poms = $field->getHeader($i, $this->orderFieldIndex, $this->browserUrl);
                    if ($poms) {
                        echo $poms;

                        $colSpan = (int)$field->headerColSpan;
                        $this->columnCount += ($colSpan > 1) ? $colSpan : 1;
                    }
                }
            }
        }
        echo "</tr>\n";
        echo "</thead>\n";
    }


    /**
     */
    public function renderTableFooter()
    {
        echo '<tr>';

        $colspan = 0;
        foreach ($this ->fields as $i => $field) {
            if ($field ->visible) {
                if ($colspan > 1) {
                    // tento footer nezobrazuje, protoze predchozi byl pres vice sloupcu a zasahuje i do tohoto sloupce
                    $colspan--;
                } else {
                    echo $field->getFooter($i, $this->browserUrl);
                    $colspan = -(int)$field->footerColSpan;
                }
            }
        }

        echo "</tr>\n";
    }


    /**
     * @param int $orderFieldIndex
     */
    public function renderTableStart($orderFieldIndex = 0)
    {
        echo '<table ';

        if ($this ->cssClass) {
            echo " class='{$this ->cssClass}'";
        }
        if ($this ->tableId) {
            echo " id='{$this ->tableId}'";
        }
        if ($this ->code) {
            echo " data-code='{$this ->code}'";

            $dataCols = '';
            foreach ($this ->fields as $field) {
                // doplni seznam skrytych poli - aby se daly v HTML dohledat
                if ($field ->code  &&  !$field ->visible) {
                    $dataCols .= ($dataCols ? ' ' : '') . $field ->code;
                }
            }

            if ($dataCols) {
                echo " data-invisible-fields='$dataCols'";
            }
        }

        if ($this ->tableHtmlParams) {
            foreach ($this ->tableHtmlParams as $name => $value) {
                echo " $name='" . StringUtils::htmlTextConvert($value) . "'";
            }
        }

        echo ">\n";

        if (!$this ->noHeader) {
            $this->renderTableHeader();
        }
    }


    /**
     *
     */
    public function renderTableEnd()
    {
        echo "</table>\n";
    }


    /**
     * Zobrazeni separatoru (radek s vodorovnou linkou)
     */
    public function renderSeparator()
    {
        echo '<tr class="browserseparator"><td colspan="' . $this ->columnCount . '"></td></tr>';
    }


    /**
     * @param int   $index
     * @param BrowserField $field
     * @param mixed $value
     * @param int   $rowNumber
     * @param mixed $recordId
     * @param array $values
     */
    public function renderFieldEdit($index, $field, $value, $rowNumber, $recordId, $values)
    {
        echo $field ->getFieldCellText($value, $rowNumber, $recordId, $values);
    }


    /**
     * @param int   $index
     * @param BrowserField $field
     * @param mixed $value
     * @param int   $rowNumber
     * @param mixed $recordId
     * @param array $values
     */
    public function renderField($index, $field, $value, $rowNumber, $recordId, $values)
    {
        echo $field ->getFieldCellText($value, $rowNumber, $recordId, $values);
    }


    /**
     * @return int   return the index of primary field in sql query
     */
    public function getPrimaryFieldSQLIndex()
    {
        if ($this ->primaryField < 0) {
            return -1;
        }
        if ($this ->sqlSelectPart) {
            // je primo zadana sqlSelectPart => index je primo index pole v SELECT casti sql dotazu
            return $this ->primaryField;
        }

        $result = 0;
        for ($i = 0; $i < $this ->primaryField - 1; $i++) {
            if ($this->fields[$i]->sqlFieldName) {
                $result++;
            }
        }

        return $result;
    }


    /**
     * Zobrazeni jednoho radku
     * @param $values
     * @return int|string
     */
    public function getClickLink($values)
    {
        if ($this ->rowClickLink && $this ->primaryField >= 0) {
            $primaryIndex = $this ->getPrimaryFieldSQLIndex();
            if ($primaryIndex < 0) {
                return '';
            }

            return $this ->rowClickLink . '&record=' . $values[$primaryIndex];
        }

        return 0;
    }


    /**
     * @param array  $values
     * @param string $className
     * @param int    $rowNumber
     * @return string   Vraci atributy pro <tr>
     */
    public function getRowAttributes($values, $className, $rowNumber)
    {
        //$skript = '';
        $data = '';

        $link = $this ->getClickLink($values);
        if ($link) {
            $className .= ' clickHref';
            $data = ' data-ron-href="' . StringUtils::htmlTextConvert($link) . '"';
            //$skript .= ' onclick="window.location=' . "'$link'" . '"';
        }

        return
            ($className ? "class='$className'" : '') .
            $data;
        //$skript;
    }


    /**
     * @param array  $values
     * @param string $className
     * @param int    $rowNumber
     * @param bool   $edit   Zobrazeni jednoho radku
     */
    public function renderLine($values, $className, $rowNumber, $edit = false)
    {
        $rowAttrs = $this ->getRowAttributes($values, $className, $rowNumber);
        echo "<tr $rowAttrs>";

        $primaryIndex = $this ->getPrimaryFieldSQLIndex();
        $recordId = ($values  &&  !is_null($primaryIndex) && isset($values[$primaryIndex]))
            ? $values[$primaryIndex]
            : '';

        foreach ($this ->fields as $i => $field) {
            if ($field ->visible) {
                $value = null;

                if (isset($field ->sqlFieldIndex)) {
                    if ($field->sqlFieldIndex >= 0 && isset($values[$field->sqlFieldIndex])) {
                        $value = $values[$field->sqlFieldIndex];
                    } else if ($field->sqlFieldIndex < 0) { //  && isset($values[count($values) + $field->sqlFieldIndex])) {
                        // zaporny index - pocita se odzadu
                        $value = $values[count($values) + $field->sqlFieldIndex];
                    }
                }

                if ($edit) {
                    $this->renderFieldEdit($i, $field, $value, $rowNumber, $recordId, $values);
                } else {
                    $this->renderField($i, $field, $value, $rowNumber, $recordId, $values);
                }
            }
        }
        echo "</tr>\n";
    }


    /**
     * @param Cursor $cursor
     * @throws Exception
     */
    public function renderCursor($cursor)
    {
        if (!$cursor) {
            throw new Exception('Chyba v metode browser::renderCursor - nejsou nactena data!');
        }

        $i = 1;

        while ($row = $cursor ->fetch()) {
            if ($this ->enableLine($row)) {
                // Pozor! Pokud enableLine vypne nektere radky a bude zapnuto strankovani, tak se mohou nektere radky opakovat!
                $this ->renderLine($row, ($i % 2) ? 'browsercolor1' : 'browsercolor2', $i);

                $i++;
                if ($this ->maxPageRows  &&  isset($this ->pageIndex)) {
                    if ($i > $this->maxPageRows) {
                        break;
                    }
                }
            }
        }

        $this ->cursor ->close();
    }


    /**
     * Zobrazi odkazy na dalsi stranky browseru
     * @param $rowsCount
     * @param $sort
     */
    public function renderPages($rowsCount, $sort)
    {
        if (!$rowsCount  ||  !$this ->maxPageRows) {
            return;
        }

        echo '<div class="pages">';
        echo '<ul class="linkslist wrap">';

        $minIndex = floor(max($this->pageIndex-5,1));
        $maxIndex = ceil(min(($rowsCount / $this ->maxPageRows) + 1, $this->pageIndex+5));

        if ($minIndex > 1) {
            $rozsah = 1 . '..' . $this->maxPageRows;
            $url = $this ->browserUrl . 'page=1' . ($sort ? "&sort=" . $sort : '');
            echo "<li><a href='$url'><span title='$rozsah'>1</span></a> ... </li>";
        }

        for ($i = $minIndex; $i < $maxIndex; $i++) {
            $url = $this ->browserUrl . 'page=' . $i;
            $classStr = ($this ->pageIndex === $i) ? ' class="selectedlink"' : '';
            echo "<li$classStr><a href='$url" . ($sort ? "&sort=" . $sort : '') . "'>";

            $first = ($i - 1) * $this ->maxPageRows + 1;
            $last = $i * $this ->maxPageRows;
            if ($last > $rowsCount) {
                $last = $rowsCount;
            }

            $rozsah = $first . '..' . $last;
            echo "<span title='$rozsah'>" .  $i . "</span>" . '</a></li>';
        }

        $lastPage = floor($rowsCount / $this ->maxPageRows) + 1;

        if ($maxIndex < $rowsCount / $this ->maxPageRows) {
            $rozsah = ($lastPage * $this ->maxPageRows) . '..' . ($rowsCount);
            $url = $this ->browserUrl . 'page=' . $lastPage . ($sort ? "&sort=" . $sort : '');
            echo "<li> ... <a href='$url'><span title='$rozsah'>$lastPage</span></a></li>";
        }


        echo '</ul>';

        $urlStr = $this->browserUrl;

        if (strpos($urlStr, '?') === 0) {
            $urlStr = substr($urlStr, 1);
        }

        parse_str($urlStr, $urlArr);

        echo "<form method='get' class='browserpagenumber'>";

        foreach ($urlArr as $urlKey => $urValue) {
            echo "<input type='hidden' name='$urlKey' value='$urValue' >";
        }

        echo TextManager::_('Go_to_page') . "&nbsp;";
        echo "<input type='number' style='width: 4rem;' name='page' min='1' max='$lastPage' >";
        echo "<input type='submit' class='submitbutton' value='" . TextManager::_('OK') . "'>";

        echo "</form>";

        echo "</div>\n";
    }


    /**
     * Zobrazi data z nactenych dat
     * @throws Exception
     */
    public function renderData() // , $firstRow)
    {
        $this ->renderCursor($this ->cursor);
    }


    /**
     * @throws Exception
     */
    public function renderTable()
    {
        $this ->renderTableStart();
        echo "<tbody>\n";

        $this ->renderData(); // , $firstRow

        if ($this ->showFooter) {
            $this ->renderTableFooter();
        }

        echo '</tbody>';
        $this ->renderTableEnd();
    }


    /**
     * Vraci index stranky, ktera se ma zobrazi (standardne podle hodnoty url parametru "page")
     * @param $rowsCount
     * @return float|int
     */
    public function getPageIndex($rowsCount)
    {
        if (!$rowsCount) {
            return null;
        }

        $pageIndex = isset($_GET['page']) ? (int)$_GET['page'] : 1;

        if ($pageIndex * $this ->maxPageRows > $rowsCount) {
            // je za posledni strankou - presune se na posledni
            $pageIndex = (int)($rowsCount/$this ->maxPageRows)  + 1;
        }

        return $pageIndex;
    }


    /**
     * @throws Exception
     */
    public function renderContent()
    {
        $rowsCount = 0;
        if ($this ->maxPageRows) {
            $rowsCount = $this ->cursor ->numRows();

            if ($rowsCount === false) {
                $this ->pageIndex = null;
                echo '<span class="d-none">Strankovani neni v ovladaci databaze podporovano!</span>';
            } else if ($rowsCount > $this ->maxPageRows) {
                $this ->pageIndex = $this ->getPageIndex($rowsCount);
                $this ->renderPages($rowsCount, null);

                $this ->cursor ->seek(($this->pageIndex - 1) * $this ->maxPageRows + 1);
            } else {
                $this ->pageIndex = 1;
            }
        }

        $this ->renderTable();

        if ($this ->maxPageRows  &&  $rowsCount > $this ->maxPageRows) {
            $this ->renderPages($rowsCount, null);
        }
    }


    /**
     * @throws Exception
     */
    public function render()
    {
        $this ->renderHeader();
        $this ->renderContent();
    }

}
