<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\Controls;

use ArrayAccess;
use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecord;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;


class InputForm implements ViewObject
{
    const NEW_RECORD = 'new';
    const COPY_RECORD = 'copy';
    const INSERT_RECORD = 'insert';
    const EDIT_RECORD = 'edit';
    const SAVE_RECORD = 'save';

    /** @var  Database */
    public $database;

    /** @var string         nadpis formulare */
    public $title;

    /** @var string         kod formulare pro nacteni konfigurace */
    public $code;

    /** @var string         kod formulare - slouzi pro rozliseni vice formularu na jedne strance */
    public $formCode;

    /** @var bool|string    Hlaska, ktera se zobrazi uzivateli pri odeslani formulare. Pokud je true, zobrazi se implicitni hlaska. */
    public $submitWorkingInfo = true;

    /** @var string         Jmeno SQL tabulky */
    public $sqlTableName;
    public $sqlSelectPart;

    /** @var FormField[]    Pole zobrazovanych poli - viz formfield.php */
    public $fields;

    /** @var bool           nastavit na true, pokud se nema kazde pole zobrazovat v novem radku tabulky */
    public $noTableRows;
    /** @var bool           nastavit na true, pokud se nema zobrazovat tlacitko "puvodni" */
    public $noOldValuesButton;
    /** @var string         jmeno css tridy pro formular */
    public $cssClass;

    public $defaultRowClass = 'form-row';
    public $defaultCaptionColClass = 'form-col1 caption';
    public $defaultDataColClass = 'form-col2';

    /** @var array          Html parametry formulare (napr. id, style, data atd.) - asociativni pole nazev=>hodnota */
    public $htmlParams = [];

    /** @var string         edit,save / new,copy,insert */
    public $formAction;
    /** @var string         akce pro formular v HTML */
    public $htmlAction;
    /** @var string         pokud je nastavena, zobrazi se tlacitko Storno */
    public $stornoAction;

    public $readOnly;

    public $data;      // nactena data

    /** @var bool           Vyrenderuje formular jako dialog (skryty atd.) */
    public $dialogMode = false;
    /** @var bool           Priznak, ze ma vstupni data konvertovat z UTF8 (pouziva se pri AJAX) */
    public $utf8 = false;

    /** @var string         HTML obsah, ktery se vygeneruje pred formularem (lze nastavit v konfiguracnim XML) */
    public $htmlContentBefore = '';
    /** @var string         HTML obsah, ktery se vygeneruje za formularem (lze nastavit v konfiguracnim XML) */
    public $htmlContentBehind = '';

    /** @var bool           Priznak, jestli se ma renderovat HTML formular (<form ....>). */
    public $renderHtmlForm = true;


    /**
     * @param Database $database
     * @param string $formAction
     * @param bool $dialogMode
     */
    public function __construct($database, $formAction, $dialogMode = false)
    {
        $this->database = $database;

        $this ->dialogMode = $dialogMode;

        //$this->cssClass = 'inputtable';
        $this->cssClass = 'form large-form form-30-70' . ($dialogMode ? ' ronform' : '');
        $this->fields = [];

        if ($this ->dialogMode) {
            $this ->htmlParams['data-ronform-field-selector'] = 'input, select, textarea';
        }

        $this->formCode = '';

        $this->readOnly = false;

        $post_ok = true;
        if (isset($_POST['formcode'])) {
            $post_ok = $_POST['formcode'] === $this->formCode;
        } else if ($this->formCode) {
            // jedna se o data z jineho formulare - ? ? ? muzu vzit formaction z GET?
            $post_ok = false;
        }

        if ($post_ok && isset($_POST['formaction'])) {
            $this->formAction = StringUtils::secureString($_POST['formaction']);
        }
        else if (isset($_GET['formaction'])) {
            $this->formAction = StringUtils::secureString($_GET['formaction']);
        }
        else {
            $this->formAction = $formAction;
        }
    }


    /**
     * @param  string|FormField $field
     * @return FormField
     */
    public function addField($field = FormField::FIELD_TYPE_EDIT)
    {
        if (!($field instanceof FormField)) {
            // array nebo string
            $field = new FormField($field);
        }

        if ($field) {
            $this->fields[] = $field;
        }

        return $field;
    }


    /**
     * Prida BrowserFields ze zadaneho pole $fields - objekty BrowserField
     * @param BrowserField[] $fields
     */
    public function addFields($fields)
    {
        foreach ($fields as $field) {
            $this->addField($field);
        }
    }


    /**
     * Vraci pole se zadanym kodem
     * @param $code
     * @return null|int
     */
    public function findFieldIndexByCode($code)
    {
        if (!$code) {
            return null;
        }

        foreach ($this ->fields as $index => $field) {
            if ($field ->code === $code) {
                return $index;
            }
        }

        return null;
    }


    /**
     * Presune zadane pole ($fieldIndex) pred pole s kodem $before, nebo za pole s kodem $after
     * @param int $fieldIndex
     * @param string $before
     * @param string $after
     */
    public function moveField($fieldIndex, $before, $after)
    {
        if ($before) {
            $field2Index = $this ->findFieldIndexByCode($before);

            if (isset($field2Index)) {
                if ($field2Index === $fieldIndex - 1) {
                    // uz je na pozadovane pozici
                    return;
                }

                $count = 1;
                if ($this ->fields[$fieldIndex] ->fieldType === FormField::FIELD_TYPE_GROUP) {
                    // presune celou skupinu - spocita, kolik polozek do teto skupiny patri (az po dalsi zacatek skupiny)
                    while ($fieldIndex + $count < count($this ->fields)  &&  $this ->fields[$fieldIndex + $count] ->fieldType !== FormField::FIELD_TYPE_GROUP) {
                        $count++;
                    }
                }

                $removed = array_splice($this ->fields, $fieldIndex, $count);

                if ($field2Index > $fieldIndex) {
                    // po odstraneni pole se index zmenil
                    $field2Index -= $count;
                }

                array_splice($this ->fields, $field2Index, 0, $removed);
                return;
            }
        }

        if ($after) {
            $field2Index = $this ->findFieldIndexByCode($after);

            if (isset($field2Index)) {
                if ($field2Index === $fieldIndex + 1) {
                    // uz je na pozadovane pozici
                    return;
                }

                if ($field2Index > $fieldIndex) {
                    // po odstraneni pole se index zmenil
                    $field2Index --;
                }

                $removed = array_splice($this ->fields, $fieldIndex, 1);
                array_splice($this ->fields, $field2Index + 1, 0, $removed);
                return;
            }
        }
    }


    /**
     * @param string    $key
     * @param ArrayAccess|string $value
     * @param string    $valueText
     */
    public function loadConfigurationItem($key, $value, $valueText)
    {
        if (strpos($key, 'html:') === 0) {
            if (!$this ->htmlParams) {
                $this ->htmlParams = [];
            }

            $this ->htmlParams[substr($key, 5)] = $valueText;
        } else {
            switch ($key) {
                case 'noTableRows' :
                case 'noOldValuesButton' :
                case 'readOnly' :
                    $this ->{$key} = (int)$valueText;
                    break;

                case 'cssClass' :
                case 'stornoAction' :
                case 'defaultRowClass' :
                case 'defaultCaptionColClass' :
                case 'defaultDataColClass' :
                    $this ->{$key} = $valueText;
                    break;

                case 'submitWorkingInfo' :
                    if ($valueText === '0') {
                        $this ->submitWorkingInfo = false;
                    }
                    else if ($valueText === '1') {
                        $this ->submitWorkingInfo = true;
                    }
                    else {
                        $this ->submitWorkingInfo = $valueText;
                    }
                    break;
            }
        }
    }


    /**
     * Vytvori nove pole pro zadanou konfiguraci.
     * @param $fieldSettings
     * @return FormField|null
     */
    protected function createNewFieldFromConfiguration($fieldSettings)
    {
        if (isset($fieldSettings['fieldType'])) {
            return new FormField($fieldSettings['fieldType']);
        }

        return null;
    }

    /**
     * Nacte konfiguraci poli ze zadaneho XML-pole.
     * @param ArrayAccess $fieldsConfArray
     */
    public function loadFieldsConfiguration($fieldsConfArray)
    {
        if (!$fieldsConfArray) {
            return;
        }

        foreach ($fieldsConfArray as $fieldSettings) {
            $field = null;

            $fieldIndex = $this ->findFieldIndexByCode($fieldSettings['code']);

            if (isset($fieldIndex)) {
                $field = $this->fields[$fieldIndex];
            } else {
                // prida nove pole
                $field = $this ->createNewFieldFromConfiguration($fieldSettings);
                if ($field) {
                    $field ->code = $fieldSettings['code'];
                    $fieldIndex = count($this->fields);
                    $this->fields[] = $field;
                }
            }

            if ($field) {
                $field ->loadConfigurationArray($fieldSettings);
                if ($fieldSettings['before'] || $fieldSettings['after']) {
                    $this->moveField($fieldIndex, $fieldSettings['before'], $fieldSettings['after']);
                }
            }
        }
    }


    /**
     * Nacte konfiguraci ze zadaneho pole nacteneho pomoci XmlLoader
     * @param $array
     */
    public function loadConfigurationArray($array)
    {
        if (!$array) {
            return;
        }

        foreach ($array as $key => $value) {
            if ($key === 'html') {
                foreach ($value as $htmlItem) {
                    $content = XmlLoader::rawDataToString($htmlItem);

                    if ($content) {
                        if (isset($htmlItem['#attr']['position'])  &&  $htmlItem['#attr']['position'] === 'behind') {
                            $this ->htmlContentBehind .= $content;
                        } else {
                            $this ->htmlContentBefore .= $content;
                        }
                    };
                }
            } else {

                if ($value instanceof ArrayAccess) {
                    // hodnoty definovane pomoci vnorenych tagu spoji do jednoho textu (napr. style)
                    $valueText = '';
                    foreach ($value as $valA) {
                        if (isset($valA['#text'])) {
                            $valueText .= ($valueText ? "\n" : '') . $valA['#text'];
                        }
                    }
                } else {
                    $valueText = $value;
                }

                $this->loadConfigurationItem($key, $value, $valueText);
            }
        }

        if (isset($array['field'])) {
            $this ->loadFieldsConfiguration($array['field']);
        }
    }

    /**
     * Nacte konfiguraci ze zadaneho xmlTextu
     * @param string $xmlText
     */
    public function loadConfigurationXml($xmlText)
    {
        if (!$xmlText) {
            return;
        }

        $a = XmlLoader::loadXmlToArray($xmlText,
            [
                XmlLoader::PARAM_CONVERT_FROM_UTF8 => true,
                XmlLoader::PARAM_INCLUDE_ATTRIBUTES => true,
                XmlLoader::PARAM_RAW_DATA_ELEMENT => 'html'
            ]
        );

        $this ->loadConfigurationArray($a);
    }


    /**
     * volano pri zobrazeni formulare pro editaci - pokud funkce vrati false
     * zobrazi se jen readonly verze formulare
     * @param array $dbrow pole nactenych dat z databaze
     * @return bool
     */
    public function canEditRecord($dbrow)
    {
        if ($this->readOnly) {
            return false;
        }

        return true;
    }



    /**
     * Zkontroluje, jestli jsou zadana vsechna povinna pole. Chybove hlasky vlozi do MessageManageru.
     * Vraci true, pokud je vse v poradku
     *
     * @param bool $edit
     * @return bool
     */
    public function checkRequiredFields($edit)
    {
        $result = true;

        foreach ($this->fields as $field) {
            if (!$field->checkRequired($edit)) {
                MessageManager::getInstance()->addError(TextManager::_('form.Missing_value') . $field->caption, $field->fieldName);
                $result = false;
            }
        }

        return $result;
    }


    /**
     * @return string|SqlStatement
     */
    public function getSqlFromPart()
    {
        return $this ->sqlTableName;
    }

    /**
     * @return string|SqlStatement
     * @throws Exception
     */
    public function getWhereCondition()
    {
        if (isset($_POST['record'])) {
            $record = $_POST['record'];
        }
        else if (isset($_GET['record'])) {
            $record = $_GET['record'];
        }
        else {
            $record = 0;
        }

        // najde primarni pole a urci podle nej podminku
        foreach ($this->fields as $field) {
            if ($field->primary) {
                return $field->sqlFieldName . ' = ' . FormField::fieldValueToSQLConst($this->database, $field->fieldType, $record);
            }
        }

        return '';
    }


    /**
     * Sestavi dotaz pro nacteni zaznamu z databaze
     * @return string|SqlStatement
     * @throws Exception
     */
    public function buildSelectSQL()
    {
        if ($this->sqlSelectPart) {
            $fieldsStr = $this->sqlSelectPart;
        } else {

            $fieldsStr = '';

            $dbFieldIndex = 0;

            foreach ($this->fields as $field) {
                $field->sqlFieldIndex = null;

                if ($field->sqlFieldName) {
                    $tableName = $field ->tableAlias ?: $field ->tableName;
                    $sqlName = ($tableName ? $tableName . '.' : '') . $field->sqlFieldName;

                    $fieldName = ($field->fieldType === FormField::FIELD_TYPE_MEMO)
                        ? $this->database->castWideMemo($sqlName)
                        : $sqlName;

                    $field->sqlFieldIndex = $dbFieldIndex;
                    //$this -> f_fields[$i] -> sqlFieldIndex = $dbfieldindex; // verze pro PHP4 !
                    $dbFieldIndex++;

                    $fieldsStr .= ($fieldsStr ? ', ' : '') . $fieldName;
                }
            }
        }

        $where = $this->getWhereCondition();
        $fromPart = $this ->getSqlFromPart();

        $params = null;
        if ($fromPart instanceof SqlStatement) {
            if ($fromPart ->parameters) {
                $params = $this ->database ->createParameters();
                $params ->appendParameters($fromPart ->parameters);
            }
            $fromStr = $fromPart ->sqlText;
        } else {
            $fromStr = $fromPart;
        }

        if ($where instanceof SqlStatement) {
            if ($params) {
                $params ->appendParameters($where ->parameters);
            } else {
                $params = $where ->parameters;
            }
            return new SqlStatement(
                "SELECT $fieldsStr FROM $fromStr\n WHERE " . $where ->sqlText,
                $params);
        }

        // $where je jen string
        $str =
            "SELECT $fieldsStr FROM $fromStr\n" .
            ' WHERE ' . $where;

        return $params
            ? new SqlStatement($str, $params)
            : $str;
    }


    /**
     * @return array|bool|null
     * @throws Exception
     */
    public function loadRecord()
    {
        // Nacte aktualni hodnoty poli z databaze
        $sql = $this->buildSelectSQL();

        if (!$sql) {
            throw new Exception('Neni zadan SQL vyraz pro nacteni dat z databaze!');
        }

        MessageManager::addDebugLog($sql);

        //MessageManager::addDebugLog('InputForm::loadRecord: ' . $sql);
        $cursor = $this->database->select($sql);
        $result = $cursor->fetch();
        $cursor ->close();

        return $result;
    }


    /**
     * vraci true, pokud se jedna o vkladani noveho zaznamu
     * @return bool
     */
    public function isInsert()
    {
        return $this->formAction === 'new'
        || $this->formAction === 'copy'
        || $this->formAction === 'insert';
    }


    protected function renderPrimaryFields()
    {
        if (isset($_POST['record'])) {
            $record = $_POST['record'];
        }
        else if (isset($_GET['record'])) {
            $record = $_GET['record'];
        }
        else {
            $record = null;
        }

        if ($record) {
            // id editovaneho zaznamu
            echo '<INPUT type="hidden" name="record" value="' . StringUtils::htmlTextConvert($record) . '">';
        }
    }


    /**
     * Vlozi zakladni pole (formaction). Potomek muze pridat svoje vlastni dodatecna pole.
     */
    public function renderCommonFields()
    {
        $formAction = $this->isInsert() ? 'insert' : 'save';

        echo '<INPUT type="hidden" name="formaction" class="ronform-field" value="' . $formAction . '">';

        if ($this->formCode) {
            echo '<INPUT type="hidden" name="formcode" value="' . $this->formCode . '">';
        }

        $this ->renderPrimaryFields();
    }


//    /**
//     * Zobrazi formular pro vkladani. $action je akce tlacitka pro vlozeni zaznamu.
//     * @param bool $postvalues Urcuje, jestli se maji zobrazit hodnoty z $_POST (true), nebo defaultni hodnoty
//     * @return bool
//     */
//    function showFieldsInsert($postvalues)
//    {
//        foreach ($this->fields as $i => $field) {
//            if ($field ->visible) {
//                $field->showFieldInsert($this->database, $postvalues, !$this->noTableRows);
//            }
//        }
//    }


//    function showFieldsEdit($values, $postvalues, $caneditrecord)
//    {
//        foreach ($this->fields as $i => $field) {
//            if ($field ->visible) {
//                if ($postvalues && $caneditrecord && $field->editable) {
//                    $value = $field->getPostValueText();
//                } else {
//                    // Standardni zobrazeni hodnot z databaze
//                    $value = (isset($field->sqlFieldIndex) && $field->sqlFieldIndex >= 0)
//                        ? $field->getValueFromDBValue($this->database, $values[$field->sqlFieldIndex])
//                        : $field->defaultValue;
//                }
//
//
//                if ($caneditrecord) {
//                    $field->showFieldEdit($this->database, $value, !$this->noTableRows, $values);
//                } else {
//                    $field->showFieldReadOnly($this->database, $value, !$this->noTableRows, $values);
//                }
//            }
//        }
//    }

    protected function loadFieldValues($insert, $values, $usePostValues, $canEditRecord)
    {
        foreach ($this->fields as $field) {
            if ($field->visible) {
                if ($insert) {
                    $field ->loadInsertValue($this ->database, $usePostValues, $values);
                } else {
                    $field ->loadEditValue($this->database, $usePostValues, $values, $canEditRecord);
                }
            }
        }
    }


    /**
     * @throws DBException
     */
    public function renderFields($insert, $values, $postValues, $canEditRecord)
    {
        //$this ->loadFieldValues($insert, $values, $postValues, $canEditRecord);

        $isRow = false;
        $isFieldGroup = false;

        foreach ($this->fields as $field) {
            if ($field ->visible) {
                if ($field ->fieldType === FormField::FIELD_TYPE_GROUP) {
                    // pole zacina novou skupinu
                    if ($isRow) {
                        echo "</div>\n";  // konec predchoziho radku
                        $isRow = false;
                    }

                    if ($isFieldGroup) {
                        echo "</div>\n"; // konec predchozi skupiny
                    }

                    $attrs = GuiUtils::getAttributesHtml($field ->getFieldAttributes());

                    // zacatek nove skupiny
                    echo "\n<div $attrs>\n";

                    $isFieldGroup = true;
                }

                if (!$this->noTableRows  &&  $field ->newRow) {
                    // pole je zacatek radku (nebo ma vlastni radek)

                    if ($isRow) {
                        echo "</div>\n";  // konec predchoziho radku
                    }

                    $rowid = isset($field ->rowId)
                        ? $field ->rowId
                        : ( $field ->fieldName ? 'row_' . str_replace('#', '_', $field ->fieldName) : '');

                    $className = $field ->rowClass ?: $this ->defaultRowClass;

                    // zacatek noveho radku
                    echo "\n<div" . ($rowid ? " id='$rowid'" : '') . ($className ? " class='{$className}'" : '') . '>';

                    $isRow = true;
                }

                if ($field ->showColumns) {
                    // caption
                    //$captionClass = $field ->captionColumnClass ?: $this ->defaultCaptionColClass . ($field->required ? ' required' : '');
                    $attrsStr = GuiUtils::getAttributesHtml($field ->getCaptionAttributes($this ->defaultCaptionColClass));
                    echo "<div $attrsStr>{$field->caption}</div>";

                    $attrsStr = GuiUtils::getAttributesHtml($field ->getDataColAttributes($this ->defaultDataColClass));
                    echo "<div $attrsStr>";
                }

                if ($insert) {
                    //$field ->loadInsertValue($this ->database, $postValues);
                    $field ->showField($this ->database, $field ->value, true, true, null); // canedit = true, insert = true, rowdata = null (insert)
                    //$field->showFieldInsert($this->database, $postValues);
                } else {
                    //$field ->loadEditValue($this ->database, $postValues, $values, $canEditRecord);
//                    if ($postValues && $canEditRecord && !$field->readOnly) {
//                        $value = $field->getPostValueText();
//                    } else {
//                        // Standardni zobrazeni hodnot z databaze
//                        $value = (isset($field->sqlFieldIndex) && $field->sqlFieldIndex >= 0)
//                            ? $field->getValueFromDBValue($this->database, $values[$field->sqlFieldIndex])
//                            : $field->defaultValue;
//                    }


                    if ($canEditRecord) {
                        $canEdit = !$field ->readOnly && $field->editable && !$field->primary;     // $this->editable pro kompatibilitu se starym kodem
                        if (!$field ->changeable  &&  $field ->value) {
                            $canEdit = false;
                        }
                        //$field->showFieldEdit($this->database, $value, $values);
                    } else {
                        $canEdit = false;
                        //$field->showFieldReadOnly($this->database, $value, $values);
                        //$field->showField($this ->database, $field ->value, false, false, $values); // canedit = false, insert = false
                    }

                    $field->showField($this ->database, $field ->value, $canEdit, false, $values); // insert = false
                }

                if ($field ->showColumns) {
                    // konec col-2 (sloupec s inputem)
                    echo '</div>';
                }
            }
        }

        if ($isRow) {
            echo "</div>\n"; // konec posledniho radku
        }

        if ($isFieldGroup) {
            echo "\n</div>\n"; // konec posledni skupiny
        }
    }


    /**
     * Vraci seznam tlacitek, ktera se maji vygenerovat do formulare
     * @param $insert
     * @return array
     */
    public function getButtons($insert)
    {
        $result = [
             'submit' => [
                 'type' => 'submit',
                 'class' => 'submitbutton' . ($this ->dialogMode ? ' ronform-submit' : ''),
                 'value' => TextManager::_($insert ? 'form.Insert' : 'form.Store')
             ]
        ];

        if (!$insert && !$this->noOldValuesButton) {
            $result['original'] = [
                'type' => 'reset',
                'class' => 'button',
                'value' => TextManager::_('form.Original')
            ];
        }

        if ($this->stornoAction) {
            $classStr =
                'button' .
                ($this ->stornoAction === '#'
                    ? ' cancel'
                    : ' clickHref');

            $result['cancel'] = [
                'type' => 'button',
                'class' => $classStr,
                'value' => TextManager::_('Cancel'),
                'data-ron-href' => $this->stornoAction
            ];
        }

        return $result;
    }

    /**
     * Zobrazeni tlacitek
     * @param bool $insert
     */
    public function renderButtons($insert)
    {
        $buttons = $this ->getButtons($insert);
        if ($buttons) {
            echo '<div class="buttons">';
            foreach ($buttons as $button) {
                echo '<span class="margin-right-2">';

                echo '<input';
                foreach ($button as $attrName => $attrValue) {
                    echo " $attrName='$attrValue'";
                }
                echo '/>';

                echo '</span>';
            }
            echo '</div>';
        }
    }


    /**
     * Zobrazi formularova pole
     * @throws Exception
     */
    public function renderContent()
    {
        echo '<div class="form-content">';

        $insert = $this ->isInsert();

        if (!$insert) {
            if (!isset($this->data)) {
                throw new Exception('Chyba! Pozadovany zaznam nebyl nalezen!');
            } else if (!$this->data) {
                // napriklad chyba v SQL dotazu
                throw new Exception('Chyba ve formulari (funkce renderFormFields) - nelze nacist hodnoty z databaze!');
            }
        }
        /*
        if ($this->isInsert()) {
            // insert
            $this ->showFieldsInsert($this->formAction == 'insert'); // pri "insert" pouzije POST
        } else {

            $canEditRecord = $this->canEditRecord($this->data);

            // edit
            $this->showFieldsEdit($this->data,
                $this->formAction == 'save',    // pri "save" pouzije postvalues
                $canEditRecord);
        }
        */

        $this ->renderFields($insert,
            $this ->data,
            $this->formAction === 'insert' || $this->formAction === 'save',
            $insert || $this->canEditRecord($this->data)
            );

        echo '</div>';
    }


    public function renderHeader()
    {
        if ($this->title) {
            echo "<h1>{$this->title}</h1>\n";
        }
    }


    public function getHtmlFormParams()
    {
        return
            ($this ->code
                ? " data-code='{$this ->code}'"
                : '') .
            ($this ->htmlAction
                ? " action='{$this->htmlAction}'"
                : '') .
            ($this ->submitWorkingInfo
                ? " class='submitWorkingInfo'"
                : '') .
            ($this ->submitWorkingInfo  &&  $this ->submitWorkingInfo !== true
                ? " data-ron-working-info='" . StringUtils::htmlTextConvert($this ->submitWorkingInfo) . "'"
                : '');
    }


    public function getCssClass()
    {
        $result = $this ->cssClass ?: '';

        $usePost = $this->formAction === 'insert' || $this->formAction === 'save';

        // prida class podle poli, ktera maji nastaveno "appendFormClass"
        foreach ($this ->fields as $field) {
            if ($field ->appendFormClass) {
                if ($usePost) {
                    if ($field ->fieldName && isset($_POST[$field ->fieldName])  &&  $_POST[$field ->fieldName]) {
                        $result .= ' ' . $field ->appendFormClass;
                    }
                } else {
                    if ($this ->data  &&  $field ->sqlFieldIndex  &&  isset($this ->data[$field ->sqlFieldIndex])  &&  $this ->data[$field ->sqlFieldIndex]) {
                        $result .= ' ' . $field ->appendFormClass;
                    }
                }
            }
        }

        return $result;
    }


    public function renderMessages()
    {
        MessageManager::getInstance()->showMessages();
    }


    /**
     * Zobrazi formular
     * @param string $formAction
     * @throws Exception
     */
    public function renderForm($formAction = '')
    {
        if ($formAction) {
            $this->formAction = $formAction;
        }

        $this->renderHeader(); // header pred <FORM> - muze obsahovat jiny formular!

        if ($this ->htmlContentBefore) {
            echo $this ->htmlContentBefore;
        }

        $this ->renderMessages();

        echo '<div';
        $cssClass = $this ->getCssClass();
        if ($cssClass) {
            echo ' class="' . $cssClass . '"';
        }
        if ($this ->htmlParams) {
            foreach ($this ->htmlParams as $paramName => $paramValue) {
                echo " $paramName='$paramValue'";
            }
        }
        echo '>';

        if ($this ->renderHtmlForm) {
            echo
                "\n<form enctype='multipart/form-data' method='post' " . $this->getHtmlFormParams() . ">\n";
        }

        $this->renderCommonFields();
        $this->renderContent();
        $this->renderButtons($this ->isInsert());

        if ($this ->renderHtmlForm) {
            echo "\n</form>\n";
        }

        echo '</div>';

        if ($this ->htmlContentBehind) {
            echo $this ->htmlContentBehind;
        }
    }


    /**
     * Nacte data pro javascriptove formulare
     * @return array
     * @throws Exception
     */
    public function getJsonData()
    {
        $data = $this ->loadRecord();

        $result = [];

        foreach ($this ->fields as $field) {
            if ($field ->visible) {
                $value = ($data && isset($field->sqlFieldIndex) && $field->sqlFieldIndex >= 0)
                    ? $field->getValueFromDBValue($this->database, $data[$field->sqlFieldIndex])
                    : $field->defaultValue;

                $result[$field ->fieldName] = StringUtils::jsonText($value);
            }
        }

        return $result;
    }


    /**
     * @param string $tableName
     * @return DBRecord
     */
    public function createDBRecord($tableName = null)
    {
        return new DBRecord($this->database, $tableName ?: $this->sqlTableName);
    }


    /**
     * @param DBRecord $rec
     * @return bool
     */
    public function beforeInsertRecord($rec)
    {
        return true;
    }

    /**
     * @param DBRecord $rec
     * @return bool
     */
    public function afterInsertRecord($rec)
    {
        return true;
    }


    /**
     * Vytvori pole pro zaznam zadane tabulky - pridava jen pole, ktera nemaji nastavene jmeno tabulky, nebo patri k zadane tabulce
     * @param DBRecord $rec
     * @param bool $insert
     * @param string $tableName
     * @throws DBException
     */
    public function createDBRecordFieldsForTable($rec, $tableName, $insert)
    {
        foreach ($this->fields as $field) {
            $fieldTableName = $field ->tableAlias ?: $field ->tableName;

            if ($field ->visible  &&  (!$fieldTableName  ||  $fieldTableName === $tableName)) {
                $rec_field = $field->createDBRecordField($this->database, $tableName, $insert, $this ->utf8); // $insert=true
                if ($rec_field) {
                    $rec->addDBRecordField($rec_field);
                }
            }
        }
    }

    /**
     * @param DBRecord $rec
     * @param bool $insert
     * @throws DBException
     */
    public function createDBRecordFields($rec, $insert)
    {
        $this ->createDBRecordFieldsForTable($rec, $this ->sqlTableName, $insert);
    }


    /**
     * @return bool
     * @throws DBException
     */
    public function insertRecord()
    {
        if (!$this->checkRequiredFields(false)) {
            return false;
        }

        $rec = $this->createDBRecord();
        $this ->createDBRecordFields($rec, true);

        if (!$this->beforeInsertRecord($rec)) {
            return false;
        }

        $rec->insertRecord();

        return $this->afterInsertRecord($rec);
    }


//    /**
//     * @return bool
//     */
//    function insert()
//    {
//        if (!$this->checkRequiredFields(false)) {
//            return false;
//        }
//
//        $rec = $this->createDBRecord();
//        $this ->createDBRecordFields($rec, true);
//
//        if (!$this->beforeInsertRecord($rec)) {
//            return false;
//        }
//
//        if (!$rec->insertRecord()) {
//            return false;
//        }
//
//        return $this->afterInsertRecord($rec);
//    }


    /**
     * @param DBRecord $rec
     * @return bool
     */
    public function beforeUpdateRecord($rec)
    {
        return true;
    }

    /**
     * @param DBRecord $rec
     * @return bool
     */
    public function afterUpdateRecord($rec)
    {
        return true;
    }


    /**
     * @param DBRecord $rec
     * @throws DBException
     */
    public function setDBRecord($rec)
    {
        $params = $this ->database ->createParameters();
        $whereStr = '';

        $this ->createDBRecordFields($rec, false);
        foreach ($this->fields as $field) {
            if ($field->primary  &&  $field ->sqlFieldName) {
                $rec_field = $rec ->findFieldByName($field ->sqlFieldName);
                if ($rec_field) {
                    if ($whereStr) {
                        $whereStr .= ' AND ';
                    }
                    $whereStr .= $field->sqlFieldName . ' = ' . $params->addParameter($rec_field->value);
                }
            }
        }

        $rec->setWhere($whereStr, $params);
    }

    /**
     * @return bool|int
     * @throws Exception
     */
    public function editRecord()
    {
        if (!$this->checkRequiredFields(true)) {
            return false;
        }

        $rec = $this->createDBRecord();
        $this ->setDBRecord($rec);

        if (!$rec->loadRecord()) {
            throw new Exception('Chyba pri editaci zaznamu - puvodni zaznam nebyl nalezen!');
        }

        if (!$this->beforeUpdateRecord($rec)) {
            return false;
        }

        if (!$rec->updateRecord()) {
            return false;
        }

        return $this->afterUpdateRecord($rec);
    }


    /**
     * @return bool|int
     * @throws Exception
     */
    public function handleAction()
    {
        $loadRecord = false;
        $usePostValues = false;

        try {
            switch ($this->formAction) {
                case self::COPY_RECORD :
                    $loadRecord = true;
                    $result = false;
                    break;

                case self::EDIT_RECORD :
                    $loadRecord = true;
                    $result = false;    // zobrazit formular
                    break;

                case self::NEW_RECORD :
                    $result = false;    // zobrazit formular
                    break;

                case self::INSERT_RECORD :
                    $usePostValues = true;
                    if ($this->insertRecord()) {
                        return true;
                    }
                    $result = false;
                    break;

                case self::SAVE_RECORD :
                    $loadRecord = true;
                    $usePostValues = true;
                    $result = $this->editRecord();
                    break;

                default :
                    throw new Exception('Chyba v metode Form::handleAction! Neznama akce!');
            }
        } catch (RonException $e) {
            if ($this ->dialogMode) {
                throw $e;
            }
            MessageManager::addErrorMessage($e ->getMessage(), $e ->getFieldName(), null, $e ->getErrorDescription());
            $result = false;
        } catch (Exception $e) {
            if ($this ->dialogMode) {
                throw $e;
            }
            MessageManager::addErrorMessage($e ->getMessage());
            $result = false;
        }

        if (!$result  &&  $loadRecord) {
            $this ->data = $this->loadRecord();
        }

        if (!$result) {
            $insert = $this ->isInsert();
            $this ->loadFieldValues($insert,
                $this ->data, $usePostValues,
                $insert || $this->canEditRecord($this->data));
        }

        return $result;
    }


    /**
     * @throws Exception
     */
    public function render()
    {
        $this ->renderForm();
    }

}
