<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\Controls;
use Ron\MVC\ViewObject;
use Ron\StringUtils;

/**
 * Skupina filtru (prepinacich nebo zapinacich)
 * Class LinksList
 * @package Ron\Controls
 */
class LinksList implements ViewObject
{
    /** @var string */
    public $url;

    /** @var bool   Urcuje, ze se nema do URL pridavat kod skupiny (pouziva se hlavne v renderLinks) */
    public $noGroupInParam;

    /** @var LinksListItem[] */
    public $items = [];

    /** @var string     Obalujici HTML element - vychozi je DIV */
    public $wrapElement;
    /** @var string[]   Asociativni pole parametru obalujiciho tagu, defaultne ['class' => 'text_link'] */
    public $wrapElementParams;

    /** @var string     Nadpis zobrazeny pred odkazy */
    public $caption;

    /** @var string */
    //public $separator;

    public $cssClass = 'linkslist';

    /** @var string     Pokud je zadan, budou mit jednotlive polozky nastavene id (tento prefix + index z $items) */
    public $itemsIdPrefix = null;


    /**
     * LinksList constructor.
     * @param array $data           Asociativni pole polozek
     * @param string|string[] $actualValue     Aktivni polozka (pripadne vice aktivnich polozek)
     * @param null $url
     */
    public function __construct($data = null, $actualValue = null, $url = null)
    {
        /** @noinspection HtmlUnknownTarget */
        //$this ->separator = '<div class="linksSeparator"></div>';

        $this ->wrapElement = 'div';
        $this ->wrapElementParams = ['class' => 'text_link'];

        $this ->url = $url;
        $this ->noGroupInParam = true;  // pro prepinaci polozky se generuji linky ve tvaru $url.$key (a ne $url $group = $key)

        $group = is_array($actualValue) ? null : 'group1';
        if ($data) {
            $this->addItemsFromArray($data, $group);
        }

        $this ->activateItem($actualValue);
    }


    /**
     * Vraci true, pokud neexistuje zadna polozka
     * @return bool
     */
    function isEmpty()
    {
        return !count($this ->items);
    }


    /**
     * Najde polozku se zadanym kodem
     * @param string $key
     * @return LinksListItem|null
     */
    public function findItem($key)
    {
        foreach ($this ->items as $item) {
            if (!strcasecmp($item ->key, $key)) return $item;
        }

        return null;
    }


    /**
     * Najde prvni polozku ze zadane skupiny
     * @param string $group
     * @param bool [$active]    Pokud je true, tak hleda jen aktivni polozku.
     * @return null|LinksListItem
     * @internal param string $key
     */
    public function findItemFromGroup($group, $active = false)
    {
        foreach ($this ->items as $item) {
            if (!strcasecmp($item ->group, $group)) {
                if (!$active  ||  $item ->active) return $item;
            }
        }

        return null;
    }


    /**
     * Prida zadanou polozku. Pokud patri do skupiny, tak ji zneaktivni, pokud uz je ve skupine nejaka polozka aktivni.
     * @param LinksListItem $item
     * @return $this
     */
    public function addItem($item)
    {
        if ($item ->group  &&  $item ->active) {
            // zjisti, jestli uz je nejaka aktivni polozka ze stejne skupiny
            $pom = $this ->findItemFromGroup($item ->group, true);
            // a pokud ano, tak tuto novou zneaktivni
            if ($pom) $item ->active = false;
        }

        $this ->items[] = $item;

        return $this;
    }


    /**
     * Prida polozku se zadanymi parametry. Pokud uz existuje tak u ni pripadne doplni caption a group
     * @param string $key
     * @param string $caption
     * @param string $group
     * @return $this
     */
    public function addOrUpdateItem($key, $caption, $group = null)
    {
        if ($item = $this ->findItem($key)) {
            if ($caption && !$item ->caption) $item ->caption = $caption;
            if ($group && !$item ->group) $item ->group = $group;
        } else {
            $item = new LinksListItem();
            $item ->key = $key;
            $item ->caption = $caption;
            $item ->group = $group;

            $this ->addItem($item);
        }

        return $this;
    }

    /**
     * Vytvori polozky ze zadaneho asociativniho pole
     * @param string[] $data Asociativni pole polozek, ktere se maji pridat (key => text)
     * @param string [$group]
     * @param bool $defaultActive   Priznak, jestli maji byt vytvorene polozky aktivni
     * @return $this
     */
    public function addItemsFromArray($data, $group = null, $defaultActive = true)
    {
        foreach ($data as $key => $value) {
            if ($value instanceof LinksListItem) {
                $item = $value;
            } else {
                $item = new LinksListItem();
                $item->key = $key;
                $item->caption = $value;
                $item->group = $group;
                $item->active = $defaultActive;
            }

            $this ->addItem($item);
        }

        return $this;
    }


    /**
     * Zaktivni zadanou polozku. Pokud patri do skupiny, tak zneaktivni vsechny ostatni ze stejne skupiny.
     * V pripade ze je $key pole, tak zaktivni vsechny zadane polozky a ostatni zneaktivni - skupiny nehlida!
     * @param string|string[] $key
     * @return $this
     */
    public function activateItem($key)
    {
        if (is_array($key)) {
            foreach ($this ->items as $item) {
                $item ->active = in_array($item ->key, $key);
            }
        } else {
            $item = $this->findItem($key);
            if (!$item) return $this;

            $item->active = true;

            // pokud je zapnuta polozka ve skupine, tak vypne vsechny ostatni polozky teto skupiny
            if ($item->group) {
                foreach ($this->items as $otherItem) {
                    if ($otherItem != $item && !strcasecmp($item->group, $otherItem->group)) {
                        $otherItem->active = false;
                    }
                }

            }
        }

        return $this;
    }


    public function itemIsActive($key)
    {
        foreach ($this ->items as $item) {
            if ($item ->key == $key) return $item ->active;
        }

        return false;
    }

    /**
     * Vraci konfiguracni retezec (aktualni nastaveni jednotlivych sekci)
     * @return string
     */
    public function getSettingsString()
    {
        $result = [];

        foreach ($this ->items as $item) {
            if ($item->group) {
                if ($item ->active) $result[$item ->group] = $item ->key;
            } else {
                $result[$item ->key] = $item ->active ? '' : '0';
            }
        }

        $resultStr = '';
        foreach ($result as $key => $value) {
            $resultStr .= ($resultStr ? ';' : '') . $key . ($value != '' ? '=' . $value : '');
        }

        return $resultStr;
    }


    /**
     * Nacte konfiguracni retezec (vytvoreny pomoci getSettingsString)
     * @param $settingsStr
     * @return $this
     */
    public function loadSettingString($settingsStr)
    {
        $strItems = (isset($settingsStr)  &&  $settingsStr !== '')
            ? explode(';', $settingsStr)
            : [];

        foreach ($strItems as $strItem) {
            // $strItem je polozka ve formatu group=key nebo key nebo key=0
            $pom = explode('=', $strItem, 2);
            foreach ($this ->items as $item) {
                if ($item ->group  &&  $item ->group == $pom[0]) {
                    $item ->active = $item ->key == $pom[1];
                } else if ($item ->key == $pom[0]) {
                    $item ->active = count($pom) == 1 || $pom[1];
                }
            }
        }

        return $this;
    }


    function getHtml()
    {
        $result = '';

        //$first = true;

        if ($this ->wrapElement) {
            $result .= "<{$this ->wrapElement}";
            foreach ($this ->wrapElementParams as $key => $value) {
                $result .= ' ' . $key . '="' . StringUtils::htmlTextConvert($value) . '"';
            }
            $result .= '>';
        }

        if ($this ->caption) $result .= '<span class="linkscaption">' . $this ->caption . '</span> ';

        $result .= "<ul class='{$this ->cssClass}'>";

        foreach ($this ->items as $itemKey => $item) {
            //if (!$first && $this ->separator) $result .= $this ->separator;


            $result .= '<li' .
                ($item ->active ? ' class="selectedlink"' : '') .
                ($this ->itemsIdPrefix ? " id='{$this ->itemsIdPrefix}{$itemKey}'" : '') .
                '>';

            $urlParam = $item ->group
                ? ($this ->noGroupInParam ? '' : $item ->group . '=') . $item ->key // prepinaci skupinova polozka
                : $item ->key . '=' . ($item ->active ? '0' : '1');     // samostatna vy/zapinaci polozka

            $result .= "<a href='{$this ->url}" . $urlParam . "' " . ($item ->title ? 'title="' . StringUtils::htmlTextConvert($item ->title) . '"' : '') . '>';
            $result .= StringUtils::htmlTextConvert($item ->caption);
            $result .= "</a>";

            $result .= '</li>';
            //if ($item ->active) $result .= '</span>';

            //$first = false;
        }

        $result .= '</ul>';

        if ($this ->wrapElement) $result .= "</{$this ->wrapElement}>";
        $result .= "\n";

        return $result;
    }

    function render()
    {
        echo $this ->getHtml();
    }


    public function loadGet()
    {
        foreach ($this ->items as $item) {
            if ($item ->group) {
                if (isset($_GET[$item ->group])) {
                    $item ->active = $item ->key == $_GET[$item ->group];
                }
            } else {
                if (isset($_GET[$item ->key])) {
                    $item ->active = (int)$_GET[$item ->key];
                }
            }
        }
    }


    /**
     * Vyrenderuje seznam odkazu ze zadaneho pole
     * @param string[]  $data       Asociativni pole zobrazenych hodnot "odkaz" => "text"
     * @param string|string[]  $actualValue    Aktualne vybrana (zvyraznena) polozka - muze jich byt i vice
     * @param string    $url        Vysledna url se slozi z $url . $key
     * @param string    $caption
     * @param bool      $showDiv    Priznak, jestli se ma zobrazit obalujici div
     */
    public static function renderLinks($data, $actualValue, $url, $caption = '', $showDiv = true)
    {
        $list = new static();
        $list ->url = $url;
        $list ->caption = $caption;
        $list ->noGroupInParam = true;  // pro prepinaci polozky se generuji linky ve tvaru $url.$key (a ne $url $group = $key)

        $group = is_array($actualValue) ? null : 'group1';
        $list ->addItemsFromArray($data, $group);

        if (!$showDiv) {
            $list->wrapElement = null;
        }

        $list ->activateItem($actualValue);

        $list ->render();
    }
}
