<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\DB;

use Exception;
use Ron\Profiler;

abstract class Database
{
    public $concatenateOperator = '+';

    /** @var string[] */
    public $functions = [];


    /**
     * @return string
     */
    abstract public function getDbType();

    /**
     * @param string $user
     * @param string $password
     * @throws DBException
     */
    abstract public function connect($user, $password);

    /**
     * @return bool
     */
    abstract public function close();

    /**
     * Vraci objekt pro nastaveni parametru SQL dotazu - interface Parameters
     * @return Parameters
     */
    abstract public function createParameters();


    public function replaceFunction($functionName, $argument)
    {
        $name = strtoupper($functionName);
        switch ($name) {
            case 'CASTWIDEMEMO' :
                return $this ->castWideMemo($argument);

            case 'CONCAT' :
                return $this ->concatenateOperator;

            case 'YEAR' :
                return $this->extractYear($argument);

            default :
                $func = isset($this ->functions[$name])
                    ? $this ->functions[$name]
                    : $functionName;
        }

        if ($func  &&  $func[0] == '#') {
            return substr($func, 1);    // bez argumentu
        }

        return $func
            ? $func . '(' . $argument . ')'
            : $argument;
    }

    /**
     * Nahradi "Makra" pro funkce {fn_...(...)} volanim prislusne funkce
     * @param string $sqlText
     * @param Parameters $params
     * @return string
     */
    function processSQLText($sqlText, /** @noinspection PhpUnusedParameterInspection */ $params)
    {
        $result = '';

        $lastPos = 0;
        $pos = 0;
        do {
            $pos = strpos($sqlText, '{fn_', $pos);
            if ($pos !== false) {
                $result .= substr($sqlText, $lastPos, $pos - $lastPos);
                // najde ( - za ni zacina parametr
                $leftBrPos = strpos($sqlText, '(', $pos + 4);
                $functionName = $leftBrPos ? trim(substr($sqlText, $pos + 4, $leftBrPos - $pos - 4)) : null;
                $rightBrPos = $leftBrPos ? strpos($sqlText, ')}', $leftBrPos + 1) : null;

                if ($functionName && $leftBrPos && $rightBrPos) {
                    // nasel funkci, levou i pravou zavorku
                    $argument = substr($sqlText, $leftBrPos + 1, $rightBrPos - $leftBrPos - 1);

                    $result .= $this ->replaceFunction($functionName, $argument);
                    $pos = $rightBrPos + 2; // posune se za )}
                }
            } else {
                $result .= substr($sqlText, $lastPos);
            }
            $lastPos = $pos;
        } while ($pos !== false);

        return $result;
    }


    /**
     * @param string $sqlText
     * @param Parameters $params
     * @return Cursor
     * @throws DBException
     */
    abstract protected function executeSelect($sqlText, $params = null);


    /**
     * @param SqlStatement|string $sqlText
     * @param Parameters [$params]  Pouzije se pouze v pripade, ze je prvni parametr string!
     * @return Cursor
     * @throws Exception
     */
    public function select($sqlText, $params = null)
    {
        if ($sqlText instanceof SqlStatement) {
            if ($params) throw new Exception('Chyba pri volani Database::select - params musi byt null!');
            $params = $sqlText ->parameters;
            $sqlText = $sqlText ->sqlText;
        }

        $sqlText = $this ->processSQLText($sqlText, $params);
        $profilerItem = Profiler::addItem($sqlText, 'SQL SELECT');
        if ($profilerItem  &&  $params) {
            $text = $params ->getLogText();
            if ($text) {
                $profilerItem->texts[] = $text;
            }
        }

        $result = $this ->executeSelect($sqlText, $params);
        Profiler::endItem($profilerItem);

        return $result;
    }

    /**
     * @param string $sqlText
     * @param Parameters $params
     * @throws DBException
     */
    abstract protected function executeSql($sqlText, $params = null);

    /**
     * @param SqlStatement|string $sqlText
     * @param Parameters [$params]  Pouzije se pouze v pripade, ze je prvni parametr string!
     * @returns void    Poznamka - return je deprecated - jen pro stary kod, ktery testoval navratovou hodnotu
     * @throws DBException
     */
    public function sql($sqlText, $params = null)
    {
        if ($sqlText instanceof SqlStatement) {
            if ($params) {
                throw new DBException($this, $sqlText ->sqlText, 'Chyba pri volani Database::sql - params musi byt null!');
            }
            $params = $sqlText ->parameters;
            $sqlText = $sqlText ->sqlText;
        }

        $sqlText = $this ->processSQLText($sqlText, $params);
        $profilerItem = Profiler::addItem($sqlText, 'SQL COMMAND');
        $this ->executeSql($sqlText, $params);
        Profiler::endItem($profilerItem);

        return true; // Pro kompatibilitu se starym kodem, ktery testuje vysledek. Nyni se pri chybe vyhazuje vyjimka.
    }


    /**
     * @return mixed
     * @throws Exception
     */
    public function lastAutoincrementValue()
    {
        throw new Exception(get_class($this) . ' error - method lastAutoincrementValue is not implemented!');
    }

    /**
     * @return string
     */
    abstract public function lastErrorText();


    /**
     * @throws DBException
     */
    abstract public function beginTransaction();

    /**
     * @throws DBException
     */
    abstract public function commitTransaction();

    /**
     * @throws DBException
     */
    abstract public function rollbackTransaction();

    /**
     * @param string $str
     * @return string
     * @deprecated
     */
    abstract public function addSlashes($str);

    /**
     * @param string $text
     * @return string;
     */
    abstract public function castWideMemo($text);

    /**
     * @param string $text
     * @return string;
     */
    abstract public function extractYear($text);

    /**
     * @param string $str
     * @return int|null
     */
    abstract public function stringToTimestamp($str);

    /**
     * @param int $timestamp
     * @return string
     * @deprecated
     */
    abstract public function timestampToString($timestamp);

    /**
     * Prevede timestamp na sql konstantu
     * @param int $timestamp
     * @return string
     * @deprecated
     */
    abstract public function dateToString($timestamp);

    /**
     * prevede cas na sql konstantu
     * @param int $timestamp
     * @return string
     * @deprecated
     */
    abstract public function timeToString($timestamp);

    /**
     * @param int $timestamp
     * @return mixed
     */
    abstract public function timestampToParam($timestamp);

    /**
     * @param int $timestamp
     * @return mixed
     */
    abstract public function dateToParam($timestamp);

    /**
     * @param int $timestamp
     * @return mixed
     */
    abstract public function timeToParam($timestamp);


    /**
     * prevede retezec z databaze na denni cas - vraci pocet hodin (realne cislo)
     * napr "... 7:30" => 7.5
     * @param string $str
     * @return integer
     */
    abstract public function stringToTimeOfDay($str);

    /**
     * prevede retezec z databaze na denni cas - vraci pocet sekund od pulnoci
     * @param string $str
     * @return integer|null
     */
    public function stringToSecondsOfDay($str)
    {
        $val = $this ->stringToTimeOfDay($str);
        if (!isset($val)) return null;

        return  (int)($val * 3600);
    }


    /**
     * Vraci vysledek zadaneho sql dotazu. Pri chybe vyhodi vyjimku.
     * @param string|SqlStatement   $sql
     * @param Parameters    $params
     * @param string        $errorMessage
     * @return Cursor
     * @throws DBException
     */
    function getSqlResult($sql, $params = null, $errorMessage = null)
    {
        // pro kompatibilitu starsiho kodu (druhy parametr byl errorMessage)
        if (is_string($params)) {
            if (!$errorMessage) $errorMessage = $params;
            $params = null;
        }

        try {
            $cursor = $this->select($sql, $params);
        } catch (DBException $e) {
            throw new DBException($this, $sql, $errorMessage, $e ->getPrevious(), $e ->dbMessage);
        }

        return $cursor;
    }


    /**
     * Zpracuje hodnotu nactenou z blob pole
     * @param $value
     * @return mixed
     */
    function stringToBinary($value)
    {
        return $value;
    }


    function getBlobParam($data)
    {
        return $data;
    }
}
