<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\DB;

use Ron\StringUtils;

class SQLUtils
{

    /**
     * Sestavi dotaz ze zadanych casti. Select part muze byt retezec nebo pole retezcu. Pokud neni zadana vubec, vlozi *
     * wherePart a orderPart mohou byt prazdne
     * @param string|string[] $selectPart
     * @param string $fromPart
     * @param string|SqlStatement $wherePart
     * @param string $orderPart
     * @param bool $distinct Urcuje, jestli se ma do dotazu vlozit DISTINCT
     * @param string $groupByPart
     * @return string|SqlStatement
     */
    static function createSql($selectPart, $fromPart, $wherePart, $orderPart, $distinct, $groupByPart = '')
    {
        $params = null;

        // vytvori select cast dotazu
        $sqltext = 'SELECT ';
        if ($distinct) $sqltext .= 'DISTINCT ';

        if (isset($selectPart)) {
            if (is_array($selectPart)) {
                $sqltext = $sqltext . $selectPart[0];
                for ($i = 1; $i < count($selectPart); $i++)
                    $sqltext = $sqltext . ', ' . $selectPart[$i];
            } else if (is_string($selectPart)) {
                $sqltext = $sqltext . $selectPart;
            } else {
                $sqltext = $sqltext . '*';
            }
        } else {
            $sqltext = $sqltext . '*';
        }

        // prida from cast dotazu
        $sqltext = $sqltext . "\nFROM " . $fromPart;

        // prida filtr pokud je zadan
        if (isset($wherePart)) {
            if ($wherePart instanceof SqlStatement) {
                if ($wherePart ->sqlText) {
                    $params = $wherePart->parameters;
                    $sqltext .= "\nWHERE " . $wherePart ->sqlText;
                }
            } else if ($wherePart != '') {
                $sqltext = $sqltext . "\nWHERE " . $wherePart;
            }
        }

        // prida group by
        if (isset($groupByPart)) if ($groupByPart != '')
            $sqltext = $sqltext . "\nGROUP BY " . $groupByPart;

        // prida razeni
        if (isset($orderPart)) if ($orderPart != '')
            $sqltext = $sqltext . "\nORDER BY " . $orderPart;

        if ($params) return new SqlStatement($sqltext, $params);

        return $sqltext;
    }


    /**
     * @param Database  $database
     * @param string    $fieldName
     * @param string|string[]|int|int[] $values
     * @param Parameters|bool $params   Objekt parametru, nebo true pokud se ma pouzit addSlashes nebo false, pokud se ma vlozit primo zadana hodnota
     * @return string
     */
    static function getINCondition($database, $fieldName, $values, $params = true)
    {
        $result = $fieldName;
        $useParams = $params instanceof Parameters;

        if (is_array($values)) {
            $result .= ' IN (';
            $first = true;

            foreach ($values as $value) {
                if (!$first) $result .= ', ';
                else $first = false;

                if ($useParams) {
                    $result .= $params ->addParameter($value);
                } else if ($params) {
                    $result .= "'" . $database->addSlashes($value) . "'";
                } else {
                    $result .= $value;
                }
            }

            $result .= ')';
        } else {
            $result .= ' = ';

            if ($useParams) {
                $result .= $params->addParameter($values);
            } else if ($params) {
                $result .= "'" . $database->addSlashes($values) . "'";
            } else {
                $result .= $values;
            }
        }

        return $result;
    }


    /**
     * Spoji zadane WHERE casti SQL vyrazu (stringy nebo instance SqlStatement) logickou spojkou AND.
     * Vraci vysledny vyraz - pokud jsou v $parts jen stringy, vraci string, jinak vraci SqlStatement.
     * Jednotlive casti mohou byt i prazdne.
     * @param (string|SqlStatement)[] $parts
     * @return string|SqlStatement
     */
    static function concatenateWherePartsOfSql($parts)
    {
        $result = '';

        foreach ($parts as $part) {
            if (isset($part)) {

                if ($part instanceof SqlStatement) {
                    if ($part ->sqlText != '') {
                        if (!$result instanceof SqlStatement) {
                            // $result je zatim retezec - udela z nej SqlFilter
                            $result = new SqlStatement($result, $part->parameters->newInstance());
                        }

                        $result ->sqlText .= ($result ->sqlText ? ' AND ' : '') . '(' . $part ->sqlText . ')';
                        $result ->parameters ->appendParameters($part ->parameters);
                    }

                } else if ($part != '') {
                    // pripoji retezec
                    if ($result instanceof SqlStatement) {
                        $result ->sqlText .= ($result ->sqlText ? ' AND ' : '') . '(' . $part . ')';
                    } else {
                        $result .= ($result ? ' AND ' : '') .  '(' . $part . ')';
                    }
                }
            }
        }

        return $result;
    }


    /**
     * @param \Ron\DB\Database $database
     * @param string $tableName
     * @param string $fieldName
     * @param string [$filter]
     * @param Parameters [$filterParams]
     * @return int
     * @throws DBException
     */
    static function newSerialNumber($database, $tableName, $fieldName, $filter = "", $filterParams = null)
    {
        $result = 1;

        $sql = "SELECT MAX($fieldName) FROM $tableName";
        if ($filter) $sql .= ' WHERE ' . $filter;

        $cursor = $database->getSqlResult($sql, $filterParams);

        if ($row = $cursor->fetch()) {
            $result = $row[0] + 1;
        }

        $cursor ->close();

        return $result;
    }


    /**
     * Ze zadaneho SQL vyrazu zjisti jmeno tabulky
     * @param string $sql
     * @return string
     */
    static function getTableNameFromSQL($sql)
    {
        if (preg_match('/^\s*(\w+)\s*(\w+)\s*(\w+)/', $sql, $pomarray)) {
            // prvni, druhe a treti slovo
            if (!$pomarray[1]) return '';

            if (!strcasecmp('INSERT', $pomarray[1])) {
                if (!strcasecmp('INTO', $pomarray[2])) return $pomarray[3];
                else return $pomarray[2]; // podle gfiles
            }

            if (!strcasecmp('UPDATE', $pomarray[1])) return $pomarray[2];

            if (!strcasecmp('DELETE', $pomarray[1])) return $pomarray[3];
        }

        // u selectu jmeno tabulky nevyhledava

        return '';
    }


    /**
     * Ze zadaneho SQL vyrazu zjisti typ (insert/update/delete/select)
     * @param string $sql
     * @return string
     */
    static function getEventTypeFromSQL($sql)
    {
        if ( preg_match('/^\s*(\w+)/', $sql, $pomarray) ) {
            // prvni slovo za whitespace
            if ( !$pomarray[1] ) return '';

            if ( !strcasecmp('INSERT', $pomarray[1]) ) return 'INSERT';
            if ( !strcasecmp('UPDATE', $pomarray[1]) ) return 'UPDATE';
            if ( !strcasecmp('DELETE', $pomarray[1]) ) return 'DELETE';
            if ( !strcasecmp('SELECT', $pomarray[1]) ) return 'SELECT';
            if ( !strcasecmp('CREATE', $pomarray[1]) ) return 'CREATE';
            if ( !strcasecmp('ALTER', $pomarray[1]) ) return 'ALTER';
        }

        return '';
    }


    /**
     * @param Database $database
     * @param $tableName
     * @param $serialCodeFieldName
     * @param string $mask
     * @param int $serialCodeMin
     * @param int $serialCodeMax
     * @return bool|string
     * @throws \Exception
     */
    static function getNextRecordSerialCode($database, $tableName, $serialCodeFieldName, $mask, $serialCodeMin, $serialCodeMax)
    {
        //Vytvori min a max dle masky a dle tohoto bude filtrovat
        $min = StringUtils::assemblyStringIDFromValueAndMask($mask, $serialCodeMin);
        $max = StringUtils::assemblyStringIDFromValueAndMask($mask, $serialCodeMax);

        //Delka retezce pro podminku
        $lengthMask = strlen($mask);

        //Nacteni posledni(max) hodnotu v dane tabulce pro potreby vygenerovani nasledujici
        $params = $database->createParameters();
        $sql =
            "SELECT Max($serialCodeFieldName) AS cislo FROM $tableName" .
            " WHERE $serialCodeFieldName >= " . $params->addParameter($min). " AND $serialCodeFieldName <= " . $params->addParameter($max) .
            " AND {fn_LEN($serialCodeFieldName)} = " .$params->addParameter($lengthMask);

//        if ($filtr != ""){
//            $sql .= " AND (" . $filtr . ")";
//        }

        $cursor = $database->getSqlResult($sql, $params);
        $lastSerialCode = $cursor->fetch()[0];

        if ($lastSerialCode == $max){
            throw new \Exception('dosazeno_max_ID');
        }

        //Pokud neexistuje predchozi v rozsahu, pak nastav na prvni hodnotu
        if ($lastSerialCode == ""){
            $newSerialCode = StringUtils::assemblyStringIDFromValueAndMask($mask, $serialCodeMin);
        } else {
            //Pokud existuje predchozi v rozsahu, pak vytvori nasledujici hodnotu v rozsahu
            $newSerialCode = StringUtils::assemblyStringIDFromValueAndMask($mask, StringUtils::getNumberOrRangeFromStringID($mask, $lastSerialCode) , true);
        }
        $cursor->close();

        return $newSerialCode;
    }

    /** Zkontroluje zda vkladany zaznam zaznam ($record) existuje v nadrazene tabulce ($table).
     * @param Database $database
     * @param string $record
     * @param string $table
     * @param string $primaryFieldName
     * @return bool vraci true, pokud nadrazeny zaznam existuje
     * @throws \Exception
     */
    public static function existsMasterRecord($database, $record, $table, $primaryFieldName)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT $primaryFieldName FROM $table WHERE $primaryFieldName = " . $params->addParameter($record);

        return $record == DBUtils::getSqlValue($database, $sql, $params);
    }
}


//if (file_exists('ron/sqlutils_old.php')) {
//    // zajisteni kompatibility stareho kodu
//    require_once('ron/sqlutils_old.php');
//}
