<?php
/**
 * stanislav.elbl, 22.02.2021
 */

namespace RonPortal\Dokumenty;

use Ron\DateUtils;
use Ron\FileUtils;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use RonPortal\Log;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalUtils;


class DokumentyUtils
{
    /**
     * Vlozi do databaze novy dokument. Vraci id dokumentu.
     * @param Database $database
     * @param string $data      Obsah dokumentu
     * @param string $fileName  Jmeno souboru
     * @param int    $typ       Typ dokumentu (odkaz do tabulky TypDokumentu)
     * @param string $popis
     * @param object|string $logSrcDescription
     * @return int
     * @throws DBException
     * @throws \Exception
     */
    public static function vytvorDokument($database, $data, $fileName, $typ, $popis, $logSrcDescription = 'Dokumenty')
    {
        // zkontroluje, jestli jsou zadana povinna pole
        if (!isset($typ) || $typ === '') {
            throw new \Exception( TextManager::_('Neni_zadan_typ_dokumentu') );
        } else if (!isset($popis) || $popis == '') {
            throw new \Exception( TextManager::_('Neni_zadan_popis_dokumentu') );
        }

        $fileName = FileUtils::truncateFileName($fileName, 160);

        $data = gzcompress($data);

        $rec = new RonPortalDBRecord($database, 'Dokument', $logSrcDescription);
        //$rec ->addField('Id', 'int', false, 0, $newId);
        $f_serial = $rec ->addField('Id', 'serial');
        $rec ->addField('Popis', '', false, 160, $popis);
        $rec ->addField('Typ', 'int', false, 0, (int)$typ);
        $rec ->addField('Datum', 'timestamp', false, 0, DateUtils::now());
        $rec ->addField('TypKomprimace', 'int', false, 0, 1);
        $rec ->addField('JmenoSouboru', '', false, 160, $fileName);
        $rec ->addField('Insert_UserName', '', false, 25, RonPortalUtils::getLogUserName());
        $rec ->addField('Insert_TimeStamp', 'timestamp', false, 0, time());
        $rec ->addField('Data', 'binary', false, 0, $data);
        $rec ->insertRecord();

        //return $newId;
        return $f_serial ->value;
    }


    /**
     * Vlozi do databaze novy dokument. Vraci id dokumentu.
     * @param Database $database
     * @param string $htmlField
     * @param int $typ
     * @param string $popis
     * @param object|string $logSrcDescription
     * @param bool $utf8    pokud je true, tak se nazev souboru prevede z utf8 (je to potreba pri uploadu z javascriptu)
     * @return int
     * @throws DBException
     * @throws \Exception
     */
    public static function vlozDokument($database, $htmlField, $typ, $popis, $logSrcDescription = 'Dokumenty', $utf8 = false)
    {
//        if ($_FILES[$htmlField]['error'] == UPLOAD_ERR_INI_SIZE ||
//            $_FILES[$htmlField]['error'] == UPLOAD_ERR_FORM_SIZE
//        ) {
//            throw new \Exception( \Ron\ronText('Err_velky_soubor') );
//        }

        switch ($_FILES[$htmlField]['error']) {
            case UPLOAD_ERR_OK :
                break;

            case UPLOAD_ERR_INI_SIZE :
            case UPLOAD_ERR_FORM_SIZE :
                throw new \Exception( TextManager::_('Err_velky_soubor') );

            case UPLOAD_ERR_PARTIAL :
                throw new \Exception( TextManager::_('Pri nahravani souboru doslo k chybe - soubor neni kompletni!') );

            case UPLOAD_ERR_NO_FILE :
                throw new \Exception('Chyba! Neni zadan soubor!');

            case UPLOAD_ERR_NO_TMP_DIR :
                throw new \Exception( TextManager::_('Pri nahravani souboru doslo k chybe - neexistuje slozka pro nahravani souboru!') );

            case UPLOAD_ERR_CANT_WRITE :
                throw new \Exception( TextManager::_('Pri nahravani souboru doslo k chybe - soubor nelze zapsat!') );

            case UPLOAD_ERR_EXTENSION :
            default :
                throw new \Exception( 'Pri nahravani souboru doslo k chybe: chyba cislo ' . $_FILES[$htmlField]['error']);
        }

        if (!is_uploaded_file($_FILES[$htmlField]['tmp_name'])) {
            throw new \Exception('Chyba! Neni zadan soubor!');
        }

        $fileName = $_FILES[$htmlField]['name']; // jmeno puvodniho souboru
        $tmpfilename = $_FILES[$htmlField]['tmp_name']; // jmeno docasneho souboru (data pro upload do databaze)

        if ($utf8) {
            $fileName = StringUtils::fromUTF8($fileName);
        }

        $data = file_get_contents($tmpfilename);

        return self::vytvorDokument($database, $data, $fileName, $typ, $popis, $logSrcDescription);
    }


    /**
     * Smaze z databaze dokument se zadanym Id. Pokud je zadana podrizenaTabulka, tak nejdrive smaze zaznam z ni.
     * @param Database $database
     * @param int $id
     * @param object|string $logDescription
     * @param string $podrizenaTabulka
     * @param string $filtr
     * @param Parameters $filtrParams
     * @param string $foreignKeyName    Jmeno pole s ID dokumentu v podrizene tabulce
     * @throws DBException
     * @throws \Exception
     */
    public static function smazDokument($database, $id, $logDescription, $podrizenaTabulka = '', $filtr = '', $filtrParams = null, $foreignKeyName = 'Id')
    {
        // smazani zaznamu z podrizene tabulky
        if ($podrizenaTabulka) {
            $params = $database ->createParameters();
            $sql =
                'DELETE FROM ' . $podrizenaTabulka .
                " WHERE $foreignKeyName = " . $params ->addParameter((int)$id) .   // ? presunout tuto podminku do filtru?
                ' AND (' . $filtr . ')';
            if ($filtrParams) {
                $params ->appendParameters($filtrParams);
            }

            try {
                $database->sql($sql, $params);
            } catch (DBException $e) {
                Log::addRecordToDBLog($database,
                    $podrizenaTabulka, 'ERROR', $logDescription,
                    "Chyba pri mazani dokumentu!\n" . $e ->dbMessage . "\n\n$sql\n\nId = $id");

                throw $e;
            }

            Log::addRecordToDBLog($database, $podrizenaTabulka, 'DELETE', $logDescription, $sql . "\n\n$foreignKeyName = $id");
        }

        $rec = new RonPortalDBRecord($database, 'Dokument', $logDescription);
        $rec ->addField('Id', 'int');
        $rec ->addField('Popis', '');
        $rec ->addField('Typ', 'int');
        $rec ->addField('Datum', 'timestamp');
        $rec ->addField('JmenoSouboru', '');
        $rec ->addField('Insert_UserName', '');
        $rec ->addField('Insert_Timestamp', 'timestamp');

        $params = $database ->createParameters();
        $rec ->deleteRecord('Id = ' . $params ->addParameter((int)$id), $params);
    }

    /**
     * @param Database $database
     * @return bool
     * @throws \Exception
     */
    public static function lzeZobrazitVyplatniListky($database)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT zobrazitNaIntrawebu FROM TypDokumentu WHERE kod = " . $params->addParameter(-1);

        $cursor = $database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        $cursor->close();

        return !!$row[0];
    }

}
