<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Editace;

use Exception;
use Ron\DateUtils;
use Ron\DB\DBException;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\Dochazka\MesicniVysledek;
use RonPortal\Dochazka\MzdovaPolozka;
use RonPortal\Mzdy\MzdyUtils;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\Rozdeleni;
use RonPortal\RozdeleniOsob;
use RonPortal\XML\RonPortalXMLForm;

/**
 * Data pro zobrazeni/editaci mesicnich vysledku - bud hromadne (EditaceMesicnichHromadna)
 * nebo jednotlive pod zpracovanou dochazkou (EditaceMesicnich)
 */
abstract class MesicniData
{
    const SETTINGS_UZAMKNOUT_MESICNI = 'uzamknoutmesicni'; // 0 = neuzamykat, 1 (default) = pri schvaleni alespon urovni 1 do konce mesice, 2 = ..., 3 = ...
    const SETTINGS_UZAMKNOUT_MZDOVE = 'uzamknoutmzdove';   //  -||-

    /** @var \Ron\DB\Database */
    var $database;

    /** @var PolozkaEditace[]   seznam editovanych polozek (radku/sloupcu)*/
    var $polozky;

    /** @var \Ron\RonPortal\Obdobi */
    var $obdobi;

    var $f_readonly;

    /** @var string     prefix poli v HTML formulari */
    var $prefixHTML;
    /** @var string     css styl zobrazene tabulky */
    var $cssStyl;
    /** @var int        sirka prole pro editaci (vychozi je 55) */
    var $sirkaPole;

    /** @var string[]   strednikem oddelene kody casovych slozek, ktere se pouziji pro nastaveni class jednotlivych radku v hromadne editaci */
    var $classMV;

    /** @var int */
    var $f_TypRozdeleni;
    /** @var string     rozdeleni pro mzdove polozky */
    var $f_Rozdeleni;

    /** @var bool */
    var $vcetnePodrizenych;

    /** @var int        urcuje, jestli budou zobrazeni zamestnanci, kteri nastoupili/vystoupili v prubehu obdobi
     *                  tj. zamestnanci musi byt zarazeni v rozdeleni
     *                  0 = kdykoliv, 1 = na zacatku obdobi, 2 = na konci obdobi
     *                  POZOR - v pripade 0 muze byt zamestnanec zapocitan na vice rozdelenich!
     */
    var $vyberPPVObdobi;

    /** @var PravaNaOsoby   pravo, podle ktereho se ridi zobrazeni seznamu osob. Pokud neni zadano, pouzije se [f_pravaMP, f_pravaMV] */
    var $pravaZobrazeni;

    /** @var PravaNaOsoby   prava uzivatele pro zobrazeni mzdovych polozek */
    var $f_pravaMP;
    /** @var PravaNaOsoby   prava pro editaci MP */
    var $f_pravaMPEdit;

    /** @var PravaNaOsoby  jestli muze zobrazit mesicni vysledky */
    var $f_pravaMV;
    /** @var PravaNaOsoby  jestli muze editovat mesicni vysledky */
    var $f_pravaMVEdit;
    // pokud je pouzito v XML, tak se nastavi vse na true - ridi se pouze pravem na XML

    var $style;

    /** @var string     Urcuje, jakym zpusobem se kontroluje uzavrene obdobi
     *      'ads' - obdobi nesmi byt uzavreno v dochazce
     *      'mzdy' - obdobi nesmi byt uzavreno ve mzdach
     *      'prvniotevrene' - obdobi nesmi byt uzavreno a predchozi jiz musi byt uzavreno
     */
    protected $zpusobKontrolyObdobi = 'ads';


    function __construct($database)
    {
        $this ->database = $database;

        $this ->polozky = [];

        //$this ->f_styl = 'display:inline-table';
        $this ->sirkaPole = 55;
    }


    /**
     * Nastavi editovane obdbi. Provede kontrolu uzavreneho obdobi a podle toho pripadne nastavi "readonly". Mozne hodnoty pro
     * @param int|Obdobi $obdobi
     * @throws Exception
     */
    function nastavObdobi($obdobi)
    {
        $this ->obdobi = ($obdobi instanceof Obdobi)
            ? $obdobi
            : Obdobi::vratObdobi($this->database, 'ADS', $obdobi);

        if (!$this ->kontrolaObdobi()) {
            $this ->f_readonly = true;
        }
    }


    /**
     * @param PolozkaEditace $polozka
     * @param bool $naZacatek   Pokud je true, vlozi se polozka na zacatek
     * @return PolozkaEditace
     */
    function pridejPolozku($polozka, $naZacatek = false)
    {
        if ($naZacatek) {
            array_unshift($this ->polozky, $polozka);
        } else {
            $this->polozky[] = $polozka;
        }

        return $polozka;
    }


    function pridejPolozkuPodleNastaveni($settingsArray, $naZacatek = false)
    {
        $polozka = new PolozkaEditace();
        $polozka ->loadSettings($settingsArray);

        $this ->pridejPolozku($polozka, $naZacatek);

        return $polozka;
    }

    /**
     * @param string $name
     * @param string $value
     * @return void   Zpracuje XML atribut hlavniho tagu
     */
    function processXMLAttribute($name, $value)
    {
        switch (strtolower($name)) {
            case 'sirkapole' :
                $this->sirkaPole = (int)$value ?: 55;
                break;

            case 'styl' :
                $this->cssStyl = $value;
                break;

            case 'prefix' :
                if ($value) {
                    $this->prefixHTML = $value;
                }
                break;

            case 'classmv' :
                $this ->classMV = explode(';', $value);
                break;

            case 'prava' :
                if ($value) {
                    $this ->pravaZobrazeni = Prava::vratPrava($value);
                }
                break;
        }
    }


    /**
     * @param string $name
     * @param array  $settingsArray
     */
    function loadChildNode($name, $settingsArray)
    {
        switch ($name) {
            case 'style' :
                $this ->style = $settingsArray['#text'];
                break;

            case 'sloupeceditacemesicnich' :
            case 'radekeditacemesicnich' : // pro editaci ve zpracovane dochazce
                $this->pridejPolozkuPodleNastaveni($settingsArray['#attr']);
                break;
        }
    }


    function loadSettings($settingsArray)
    {
        // nacte atributy hlavniho tagu
        if (isset($settingsArray['#attr'])) {
            foreach ($settingsArray['#attr'] as $name => $value) {
                $this ->processXMLAttribute($name, $value);
            }
        }

        // nacteni vnorenych elementu - radku/sloupcu atd.
        foreach ($settingsArray['#children'] as $child) {
            $this ->loadChildNode($child['name'], $child['element']);
        }
    }

    /**
     * @param $element \DOMNode
     * @return void
     * @deprecated
     */
    function loadDOMElement($element)
    {
        $loader = new XmlLoader();
        $loader ->convertFromUtf8 = true;
        $settingsArray = $loader ->readDOMElement($element);
        $this ->loadSettings($settingsArray);
    }


    /**
     * @param $xmlText string
     * @return void
     * @throws Exception
     */
    function loadXML($xmlText)
    {
        MessageManager::addDebugLog('editacemesicnich - nacitam nastaveni:<PRE>' . StringUtils::xmlTextConvert($xmlText) . '</PRE>');

        $settingsArray = XmlLoader::loadXmlToArray($xmlText);
        $this ->loadSettings($settingsArray);
    }


    /**
     * @param $nastaveni string
     */
    function nactiNastaveni($nastaveni)
    {
        $this ->loadXML($nastaveni);
    }


    /** @return bool   vraci true, pokud alespon jeden z radku je mesicni vysledek */
    function obsahujeMV()
    {
        foreach ($this ->polozky as $polozka)
            if ($polozka ->zobrazit) {
                if ($polozka->typ == 'MV' || $polozka->typ == 'PMV') return true;
            }

        return false;
    }


    /** @return bool   vraci true, pokud alespon jeden z radku je mzdova polozka */
    function obsahujeMP()
    {
        foreach ($this ->polozky as $polozka)
            if ($polozka ->zobrazit) {
                if ($polozka->typ == 'MP' || $polozka->typ == 'PMP') return true;
            }

        return false;
    }


    /**
     * @param PolozkaEditace $polozka
     * @param OsobaProEditaci $osoba
     * @return bool  Vraci true, v pripade, ze ma uzivatel pravo na editaci zadane polozky
     * @throws Exception
     */
    protected function jePravoNaPolozku($polozka, $osoba)
    {
        if ($polozka ->pravaEditace) {
            return Prava::vratPrava($polozka ->pravaEditace) ->lzeZobrazitPP($osoba ->idPP);
        }

        if ($polozka ->typ === 'MP'  ||  $polozka ->typ === 'PMP') {
            if (Prava::existujePravo('vkladani_mzdpol_' . $polozka ->kod)) {
                return Prava::vratPrava('vkladani_mzdpol_' . $polozka->kod) ->lzeZobrazitPP($osoba ->idPP);

            }
            return $osoba ->f_PravoEditovatMP;
        }

        if ($polozka ->typ === 'MV'  ||  $polozka ->typ === 'PMV') {
            if (Prava::existujePravo('editace_mv_' . $polozka ->kod)) {
                return Prava::vratPrava('editace_mv_' . $polozka->kod) ->lzeZobrazitPP($osoba ->idPP);
            }

            return $osoba ->f_PravoEditovatMV;
        }

        return false;
    }


    /**
     * @param $osoba OsobaProEditaci
     * @param MesicniVysledek $vysledek
     */
    protected function nastavMesicniVysledek($osoba, $vysledek)
    {
        foreach ($this ->polozky as $i => $polozka) {
            if ($polozka ->typ === 'PMV'  &&  $polozka ->kod === $vysledek ->typCasoveSlozkyMzdy) {
                // poznamka mzdove polozky
                $osoba ->f_data[$i] ->f_hodnota = null;
                $osoba ->f_data[$i] ->f_text = $vysledek ->poznamka;
                $osoba ->f_data[$i] ->f_puvodniText = $vysledek ->poznamka;
                $osoba ->f_data[$i] ->f_hint = $vysledek ->vratHint();

                if ( isset($polozka ->defaultValue)  &&   $polozka ->defaultValue !== '' &&  ( !isset($vysledek ->poznamka)  ||  $vysledek ->poznamka === '' ) ) {
                    $osoba ->f_data[$i] ->f_text = $polozka ->defaultValue;
                }
            }
            else if ($polozka ->typ === 'MV'  &&  $polozka ->kod === $vysledek ->typCasoveSlozkyMzdy) { // MV
                $osoba ->f_data[$i] ->f_hodnota = $vysledek ->vratHodnotu($polozka ->jednotka);
                $osoba ->f_data[$i] ->f_text = $polozka ->vratTextProHodnotu($osoba ->f_data[$i] ->f_hodnota);
                $osoba ->f_data[$i] ->f_puvodniText = $osoba ->f_data[$i] ->f_text;
                $osoba ->f_data[$i] ->f_hint = $vysledek ->vratHint();

                // pokracuje dale - stejna casova slozka tam muze byt vicekrat (napr. hodiny i dny)
            }

            if ($polozka ->typ === 'MV'  &&  $polozka ->kodLimitMaximum === $vysledek ->typCasoveSlozkyMzdy) {
                // hodnota je limitem pro aktualni sloupec
                switch ($polozka ->jednotka) {
                    case 'D' :
                        $osoba ->f_data[$i] ->f_maxHodnota = $vysledek ->dny;
                        break;

                    case 'K' :
                        $osoba ->f_data[$i] ->f_maxHodnota = $vysledek ->kalendarniDny;
                        break;

                    default :
                        $osoba ->f_data[$i] ->f_maxHodnota = $vysledek ->hodiny;
                        break;
                }
            }

            if ($polozka ->typ === 'MV'  &&  $polozka ->kodLimitMinimum == $vysledek ->typCasoveSlozkyMzdy) {
                // hodnota je spodnim limitem pro aktualni sloupec
                switch ($polozka ->jednotka) {
                    case 'D' :
                        $osoba ->f_data[$i] ->f_minHodnota = $vysledek ->dny;
                        break;

                    case 'K' :
                        $osoba ->f_data[$i] ->f_minHodnota = $vysledek ->kalendarniDny;
                        break;

                    default :
                        $osoba ->f_data[$i] ->f_minHodnota = $vysledek ->hodiny;
                        break;
                }
            }
        }
    }


    /**
     * @param $osoba OsobaProEditaci
     * @throws DBException
     */
    protected function nactiMesicniVysledkyOsoby($osoba)
    {
        $typMV = [];

        if ($this ->classMV) {
            foreach ($this ->classMV as $kodMV) {
                $tymMV[] = $kodMV;
            }
        }

        foreach ($this ->polozky as $polozka) {
            if ($polozka ->typ === 'PMV') {
                $typMV[] = $polozka ->kod;
            }

            if ($polozka ->typ === 'MV') {
                $typMV[] = $polozka ->kod;
                if ($polozka ->kodLimitMaximum) {
                    $typMV[] = $polozka ->kodLimitMaximum;
                }
                if ($polozka ->kodLimitMinimum) {
                    $typMV[] = $polozka ->kodLimitMinimum;
                }
            }
        }

        if (!count($typMV)) {
            // zadny sloupec neobsahuje mzdove polozy
            return;
        }

        $vysledky = MesicniVysledek::nactiMesicniVysledky($this ->database, $osoba ->idPP, $this ->obdobi ->zacatek, $typMV);

        foreach($vysledky as $vysledek) {
            $this ->nastavMesicniVysledek($osoba, $vysledek); // ->typCasoveSlozkyMzdy, (float)$vysledek ->hodiny, (float)$vysledek ->dny, (float)$vysledek ->kalendarniDny, $vysledek ->poznamka);

            if ($this ->classMV  &&  (float)$vysledek ->hodiny  &&  in_array($vysledek ->typCasoveSlozkyMzdy, $this ->classMV)) {
                $osoba ->rowClass .= ($osoba ->rowClass ? ' ' : '') . 'MV' . $vysledek ->typCasoveSlozkyMzdy;
            }
        }
    }


    /**
     * @param OsobaProEditaci $osoba
     * @param MzdovaPolozka $mzdovaPolozka
     * @throws DBException
     */
    protected function nastavMzdovouPolozku($osoba, $mzdovaPolozka) // $kodPolozky, $castka, $cenaZaJednotku, $pocetJednotek, $typRozdeleni, $rozdeleni)
    {
        foreach ($this ->polozky as $i => $polozka) {
            if ($polozka ->typ === 'MP'  &&  $polozka ->kod === $mzdovaPolozka ->mzdovaPolozka) {
                $jeStejneRozdeleni = false;
                if ($polozka ->rozlisitRozdeleni) {
                    if ($polozka->rozlisitRozdeleni === 2) {
                        // podle rozdeleni osoby - to, ktere je zobrazeno na radku
                        $jeStejneRozdeleni = $this->f_TypRozdeleni == $mzdovaPolozka->kodTypuRozdeleni && $osoba->kodRozdeleni == $mzdovaPolozka->rozdeleni;
                    } else if ($polozka->rozlisitRozdeleni === 3) {
                        // podle rozdeleni osoby - nacte se rozdeleni z databaze
                        $kodTypuRozdeleni = $polozka->kodTypuRozdeleni ?: $this->f_TypRozdeleni;
                        $rozdeleni = Rozdeleni::vratRozdeleniOsoby($this->database, $osoba->idPP, $kodTypuRozdeleni, $this->obdobi->zacatek, $this->obdobi->konec, false);
                        if (isset($rozdeleni)) {
                            $jeStejneRozdeleni = $rozdeleni == $mzdovaPolozka->rozdeleni;
                        }
                    } else {
                        // podle rozdeleni formulare
                        $jeStejneRozdeleni = $this->f_TypRozdeleni == $mzdovaPolozka->kodTypuRozdeleni && $this->f_Rozdeleni == $mzdovaPolozka->rozdeleni;
                    }
                }

                if (!$polozka ->rozlisitRozdeleni || $jeStejneRozdeleni) {
                    $osoba ->f_data[$i] ->f_hodnota = $mzdovaPolozka ->castka;
                    $osoba ->f_data[$i] ->f_text = $polozka ->vratTextProHodnotu($mzdovaPolozka ->castka);
                    $osoba ->f_data[$i] ->f_puvodniText = $osoba ->f_data[$i] ->f_text;
                    $osoba ->f_data[$i] ->f_hint = $mzdovaPolozka ->vratHint();

//                    if ($mzdovaPolozka ->pocetJednotek != 1) $osoba ->f_data[$i] ->f_hint =
//                        $mzdovaPolozka ->pocetJednotek . ' x ' . StringUtils::format($polozka ->format, $mzdovaPolozka ->cenaZaJednotku);

                    //5.9.2016 RON-PR zrusen return, pokud je zobrazena polozka zaroven limitem, nenacetl se limit
                    //return true;
                }
            }

            if ($polozka ->typ === 'MP'  &&  $polozka ->kodLimitMaximum == $mzdovaPolozka ->mzdovaPolozka) {
                // hodnota je limitem pro aktualni sloupec
                $osoba ->f_data[$i] ->f_maxHodnota = $mzdovaPolozka ->castka;
            }

            if ($polozka ->typ === 'MP'  &&  $polozka ->kodLimitMinimum == $mzdovaPolozka ->mzdovaPolozka) {
                // hodnota je spodnim limitem pro aktualni sloupec
                $osoba ->f_data[$i] ->f_minHodnota = $mzdovaPolozka ->castka;
            }
        }
    }


    /**
     * @param OsobaProEditaci $osoba
     */
    protected function nactiMzdovePolozkyOsoby($osoba)
    {
        // sestavi seznam typu mzdovych polozek
        $typMP = [];
        foreach ($this ->polozky as $polozka) {
            if ($polozka ->typ === 'MP') {
                $typMP[] = $polozka ->kod;
                if ($polozka ->kodLimitMaximum) $typMP[] = $polozka ->kodLimitMaximum;
                if ($polozka ->kodLimitMinimum) $typMP[] = $polozka ->kodLimitMinimum;
            }
        }

        if (!count($typMP)) {
            // zadna MP k nacteni
            return;
        }

        $polozky = MzdovaPolozka::nactiMzdovePolozky($this ->database, $osoba, $this ->obdobi ->zacatek, $typMP);
        foreach ($polozky as $mp) {
            $this ->nastavMzdovouPolozku($osoba, $mp);
        }
    }


    /**
     * @param OsobaProEditaci $osoba
     */
    function nactiRozdeleniOsoby($osoba)
    {
        foreach ($this ->polozky as $i => $polozka) {
            $typrozdeleni = null;

            if ($polozka ->typ == 'ROZD' || $polozka ->typ == 'ROZDP') {
                // R a RP netreba
                $typrozdeleni = (int)$polozka ->kodTypuRozdeleni;
            }

            if ($typrozdeleni) {
                if ($polozka ->datumRozdeleni < 0) $datumrozdeleni = $this ->vyberPPVObdobi;
                else $datumrozdeleni = $polozka ->datumRozdeleni;

                switch ($datumrozdeleni) {
                    case 1 : // zacatek obdobi
                        $datumod = $this ->obdobi ->zacatek;
                        $datumdo = $this ->obdobi ->zacatek;
                        break;

                    case 2 : // konec obdobi
                        $datumod = $this ->obdobi ->konec;
                        $datumdo = $this ->obdobi ->konec;
                        break;

                    default : // kdykoliv behem obdobi
                        $datumod = $this ->obdobi ->zacatek;
                        $datumdo = $this ->obdobi ->konec;
                        break;
                }

                $params = $this ->database ->createParameters();
                $sql =
                    'SELECT Rozdeleni.Kod, Rozdeleni.Popis ' .
                    ' FROM RozdeleniOsob, Rozdeleni' .
                    ' WHERE RozdeleniOsob.KodTypuRozdeleni = ' . $params ->addParameter($typrozdeleni) .
                    '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
                    '   AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni ' .
                    '   AND RozdeleniOsob.RC = ' . $params ->addParameter($osoba ->rc) .
                    '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($osoba ->osobniCislo) .
                    '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($this ->database ->timestampToParam($osoba ->datumNastupu)) .
                    '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($this ->database ->timestampToParam($datumdo)) .
                    '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($this ->database ->timestampToParam($datumod)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)';

                if ($polozka ->sqlFilter) {
                    $sql .= ' AND ' . $polozka ->sqlFilter;
                }

                $text = '';
                $hint = '';

                $cursor = $this ->database ->getSqlResult($sql, $params);
                while ($row = $cursor ->fetch()) {
                    if ($text) $text .= ', ';

                    switch ($polozka ->typ) {
                        case 'ROZD' :
                            $text .= $row[0];
                            if ($hint) $hint .= ', ';
                            $hint .= $row[1];
                            break;
                        case 'ROZDP' :
                            $text .= $row[1];
                            break;
                    }
                }
                $cursor ->close();

                $osoba ->f_data[$i] ->f_text = $text;
                $osoba ->f_data[$i] ->f_hint = $hint;
            }
        }
    }

    /**
     * @param $osoba OsobaProEditaci
     */
    function nactiIkonyRozdeleni($osoba)
    {
        foreach ($this ->polozky as $i => $polozka) {
            if ($polozka->typ == 'ROZDIMG' && $polozka->kodTypuRozdeleni) {
                $ikony = RozdeleniOsob::nactiIkonyRozdeleni($this ->database, $osoba->idPP, $polozka ->kodTypuRozdeleni, $this ->obdobi ->zacatek, $this ->obdobi ->konec);
                $osoba ->f_data[$i] ->f_text = RozdeleniOsob::getIkonyHtml($ikony);
            }
        }
    }

    /**
     * @param $osoba OsobaProEditaci
     */
    function nactiPoznamkyMP($osoba)
    {
        // Pozor priznak "rozlisitrozdeleni" nefunguje spravne!
        foreach ($this ->polozky as $i => $polozka) {
            if ($polozka ->typ == 'PMP' && $polozka ->kod) {
                $params = $this ->database ->createParameters();
                $sql =
                    'SELECT MzdovaPolozka, ' . $this ->database ->castWideMemo('Poznamka') .
                    ' FROM PoznamkaMzdovePolozky ' .
                    ' WHERE RC = ' . $params->addParameter($osoba ->rc) .
                    '   AND OsobniCislo = ' . $params->addParameter($osoba ->osobniCislo) .
                    '   AND DatumNastupu = ' . $params ->addParameter($this ->database ->timestampToParam($osoba ->datumNastupu)) .
                    '   AND Obdobi = ' . $params ->addParameter($this ->database ->timestampToParam($this ->obdobi ->zacatek)) .
                    '   AND MzdovaPolozka = ' . $params ->addParameter($polozka ->kod) .
                    '   AND DruhZdrojeMzdovePolozky = 4'; // jen MP z dochazky

                $cursor = $this ->database ->getSqlResult($sql, $params);

                $row = $cursor ->fetch();
                if ($row) {
                    $osoba ->f_data[$i] ->f_text = $row[1];
                    $osoba ->f_data[$i] ->f_puvodniText = $osoba ->f_data[$i] ->f_text;
                }
                $cursor ->close();
            }
        }
    }


    /**
     * @param OsobaProEditaci $osoba
     * @param int $i
     * @param PolozkaEditace $polozka
     */
    function nactiSQLPolozku($osoba, $i, $polozka)
    {
        $result = $polozka ->loadSQLProOsobu($this ->database, $osoba, $this ->obdobi, $this ->f_TypRozdeleni, $this ->f_Rozdeleni);

        if ($result) {
            $osoba->f_data[$i]->f_hodnota = $result['hodnota'];
            $osoba->f_data[$i]->f_puvodniText = $osoba->f_data[$i]->f_text = $result['text'];
            $osoba->f_data[$i]->f_hint = $result['title'];
        }

        /*
        $sql = $polozka ->sql;
        $sql = str_replace('%RC%', "'" . $this ->database ->addSlashes($osoba ->rc) . "'", $sql);
        $sql = str_replace('%OsobniCislo%', "'" . $this ->database ->addSlashes($osoba ->osobniCislo) . "'", $sql);
        $sql = str_replace('%DatumNastupu%', $this ->database ->timestampToString($osoba ->datumNastupu), $sql);

        $sql = str_replace('%PocatekObdobi%', $this ->database ->timestampToString($this ->obdobi ->zacatek), $sql);
        $sql = str_replace('%KonecObdobi%', $this ->database ->timestampToString($this ->obdobi ->konec), $sql);

        $sql = str_replace('%PocatekNasledujicihoObdobi%', $this ->database ->timestampToString( DateUtils::incMonth($this ->obdobi ->zacatek, 1) ), $sql);
        $predchoziObdobi = DateUtils::incMonth($this ->obdobi ->zacatek, -1);
        $sql = str_replace('%PocatekPredchozihoObdobi%', $this ->database ->timestampToString($predchoziObdobi), $sql);
        $sql = str_replace('%KonecPredchozihoObdobi%', $this ->database ->timestampToString(DateUtils::lastDayOfMonth($predchoziObdobi)), $sql);

        $month = (int)date('n', $this ->obdobi ->zacatek);
        $quarter = DateUtils::incMonth($this ->obdobi ->zacatek,  $month -($month - 1) % 3 );

        $sql = str_replace('%PocatekCtvrtleti%', $this ->database ->timestampToString($quarter), $sql);
        $sql = str_replace('%KonecCtvrtleti%', $this ->database ->timestampToString( DateUtils::lastDayOfMonth(DateUtils::incMonth($quarter, 2)) ), $sql);

        $sql = str_replace('%PocatekPredchozihoCtvrtleti%', $this ->database ->timestampToString(DateUtils::incMonth($quarter, -3)), $sql);
        $sql = str_replace('%KonecPredchozihoCtvrtleti%', $this ->database ->timestampToString( DateUtils::lastDayOfMonth(DateUtils::incMonth($quarter, -1)) ), $sql);

        if ($this ->f_TypRozdeleni  &&  $this ->f_Rozdeleni) {
            $sql = str_replace('%TypRozdeleni%', (int)$this ->f_TypRozdeleni, $sql);
            $sql = str_replace('%Rozdeleni%', "'" . $this ->database ->addSlashes($this ->f_Rozdeleni) . "'", $sql);
        } else {
            $sql = str_replace('%TypRozdeleni%', '0', $sql);
            $sql = str_replace('%Rozdeleni%', "''", $sql);
        }

        if (!$sql) return;

        $cursor = $this ->database ->getSqlResult($sql);

        $text = '';
        $hint = '';

        while ($row = $cursor ->fetch()) {
            if ($row && isset($row[0])) {
                if ($text) $text .= ', ';
                $text .= DBUtils::getFormatedValue($this ->database, $polozka ->format, $row[0]);

                if ( isset($row[1])) {
                    if ($hint) $hint .= ', ';
                    $hint .= $row[1];
                }

                $osoba ->f_data[$i] ->f_hodnota = $row[0];
            }
        }

        $osoba ->f_data[$i] ->f_text = $text;
        $osoba ->f_data[$i] ->f_puvodniText = $osoba ->f_data[$i] ->f_text;
        $osoba ->f_data[$i] ->f_hint = $hint;
        */
    }


    /**
     * @param OsobaProEditaci $osoba
     */
    function nactiDataOsoby($osoba)
    {
        $osoba ->f_data = [];
        foreach ($this ->polozky as $i =>$polozka) {
            $osoba ->f_data[$i] = new EditaceMesicnichField();

            if ($polozka ->defaultValue) $osoba ->f_data[$i] ->f_text = $polozka ->defaultValue;
        }

        // rozdeleni
        $this ->nactiRozdeleniOsoby($osoba);
        $this ->nactiIkonyRozdeleni($osoba);

        // mzdove polozky
        if ($osoba ->f_PravoZobrazitMP || $osoba ->f_PravoEditovatMP) {  // TODO: Nebude fungovat, kdyz bude mit prava jen na konkretni mzdovou polozku!
            foreach ($this ->polozky as $i => $polozka) {
                if (isset($polozka ->maximum)) $osoba ->f_data[$i] ->f_maxHodnota = $polozka ->maximum;
                if (isset($polozka ->minimum)) $osoba ->f_data[$i] ->f_minHodnota = $polozka ->minimum;
            }

            $this ->nactiMzdovePolozkyOsoby($osoba);
            $this ->nactiPoznamkyMP($osoba);
        }

        // mesicni vysledky
        if ($osoba ->f_PravoZobrazitMV || $osoba ->f_PravoEditovatMV) {  // TODO: Nebude fungovat, kdyz bude mit prava jen na konkretni casovou slozku!
            $this ->nactiMesicniVysledkyOsoby($osoba);
        }

        // ostatni pole jsou uz nactene v osobe (datum nastupu, ukonceni atd.)
        foreach ($this ->polozky as $i =>$polozka) {
            if ($polozka ->typ == 'SQL') $this ->nactiSQLPolozku($osoba, $i, $polozka);
        }

    }


    /**
     * @param OsobaProEditaci $osoba
     * @throws Exception
     */
    function nastavPravaNaOsobu($osoba)
    {
        // nastaveni prav
        switch ($this ->vyberPPVObdobi) {
            case 1 : // zacatek obdobi
                $datumOd = $this ->obdobi ->zacatek;
                $datumDo = $this ->obdobi ->zacatek;
                break;

            case 2 : // konec obdobi
                $datumOd = $this ->obdobi ->konec;
                $datumDo = $this ->obdobi ->konec;
                break;

            default : // kdykoliv behem obdobi
                $datumOd = $this ->obdobi ->zacatek;
                $datumDo = $this ->obdobi ->konec;
                break;
        }

        $osoba ->f_PravoEditovatMP = $this ->f_pravaMPEdit ->lzeZobrazitPP($osoba ->idPP, $datumOd, $datumDo);
        $osoba ->f_PravoEditovatMV = $this ->f_pravaMVEdit ->lzeZobrazitPP($osoba ->idPP, $datumOd, $datumDo);

        $osoba ->f_PravoZobrazitMV = $osoba ->f_PravoEditovatMV || $this ->f_pravaMV ->lzeZobrazitPP($osoba ->idPP, $datumOd, $datumDo);
        $osoba ->f_PravoZobrazitMP = $osoba ->f_PravoEditovatMP || $this ->f_pravaMP ->lzeZobrazitPP($osoba ->idPP, $datumOd, $datumDo);
    }


    /**
     * @param array $row
     * @return OsobaProEditaci
     */
    function vytvorOsobuPodleRadku($row)
    {
        $osoba = new OsobaProEditaci();
        $osoba ->loadRow($this ->database, $row);

        $this ->nastavPravaNaOsobu($osoba);

        return $osoba;
    }



    /**
     * @param OsobaProEditaci $osoba
     * @param int $index
     * @return string
     */
    function vratHTMLText($osoba, $index)
    {
        if ($this->polozky[$index]->prava) {
            if (!$this->polozky[$index]->prava ->lzeZobrazitPP($osoba ->idPP)) {
                return '';
            }
        }

        switch ($this->polozky[$index]->typ) {
            case 'SCHVAL1' :
                return $osoba->schvaleni->schvaleni1
                    ? '<SPAN class="textschvaleni1">' . date('j.n.Y', $osoba->schvaleni->schvaleni1) . '</SPAN>'
                    : '';

            case 'SCHVAL2' :
                return $osoba->schvaleni->schvaleni2
                    ? '<SPAN class="textschvaleni2">' . date('j.n.Y', $osoba->schvaleni->schvaleni2) . '</SPAN>'
                    : '';

            case 'SCHVAL3' :
                return $osoba->schvaleni->schvaleni3
                    ? '<SPAN class="textschvaleni3">' . date('j.n.Y', $osoba->schvaleni->schvaleni3) . '</SPAN>'
                    : '';

            case 'SCHVAL' : // zobrazi nejvyssi nastavenou uroven schvaleni
                return
                    $osoba->schvaleni->schvaleni3 ? '<SPAN class="textschvaleni3">' . date('j.n.Y', $osoba->schvaleni->schvaleni3) . '</SPAN>' :
                        ($osoba->schvaleni->schvaleni2 ? '<SPAN class="textschvaleni2">' . date('j.n.Y', $osoba->schvaleni->schvaleni2) . '</SPAN>' :
                            ($osoba->schvaleni->schvaleni1 ? '<SPAN class="textschvaleni1">' . date('j.n.Y', $osoba->schvaleni->schvaleni1) . '</SPAN>' : '')
                        );

            case 'SCHVALMZDY' :
                return \Ron\ronText($osoba->schvaleni->zpracovanoVeMzdach ? 'Str_Yes' : 'Str_No');

            case 'EDIT' : // odkaz na editaci dochazky
                $targetstr = $this->polozky[$index]->target;
                $targetstr = $targetstr ? 'target="' . $targetstr . '"' : '';

                return
                    '<A onclick="StopEventPropagation(event)" ' . $targetstr . ' href="?menuid=zpracovanadochazka&amp;osoba=' . $osoba->idPP .
                    '&amp;obdobi=' . DateUtils::dateToUrlString($this->obdobi->zacatek) . '">' . \Ron\ronText('dochazka') . '</A>';

            case 'JMENOPRIJMENI' :
            case 'JMENO' :
            case 'PRIJMENI' :
            case 'OSOBNICISLO' :
            case 'UZIVATELSKA' :
            case 'UZIVATELSKAPP' :
            case 'SCHVALENI' :
            case 'FOTO' :
            case 'NASTUP' :
            case 'UKONCENI' :
            case 'TYPPP' :
                return $this->polozky[$index]->getText($osoba, $this ->obdobi, null);

            case 'ROZDIMG' :
                return $osoba->f_data[$index]->f_text; // obsahuje primo HTML

            default :
                return StringUtils::htmlTextConvert($osoba->f_data[$index]->f_text);
        }
    }



    /**
     * Vraci true, pokud jeste obdobi neni uzavreno - zpusob kontroly urcuje vlastnost $kontrolaObdobi
     * @return bool
     */
    function kontrolaObdobi()
    {
        // kontrola uzavreni v dochazce
        switch ($this ->zpusobKontrolyObdobi) {
            case 'mzdy' :
                // obdobi v dochazce se nekontroluje
                break;

            case 'prvniotevrene' :
                if ($this ->obdobi ->uzavreno) {
                    return false;
                } else {
                    // zakaze editaci, pokud predchozi obdobi jeste neni uzavreno
                    $obdobi2 = Obdobi::vratObdobi($this->database, 'ADS', DateUtils::incMonth($this ->obdobi->zacatek, -1));
                    if (!$obdobi2->uzavreno) {
                        MessageManager::addWarning(TextManager::_('MV_nelze_editovat_Predchozi_obdobi_neuzavreno'));
                        return false;
                    }
                }
                break;

            case 'nekontrolovat' :
                // kontrola uzavreneho obdobi se vubec neprovadi
                return true;

            case 'jenads' :
                // mzdy se vubec nekontroluji
                return !$this ->obdobi ->uzavreno;

            case 'ads' :
            default :
                if ($this ->obdobi ->uzavreno) {
                    return false;
                }
        }

        // kontrola uzavreni ve mzdach - pokud vubec neexistuje, tak je to OK
        if (Obdobi::jeObdobiUzavreno($this ->database, 'MZDY', $this ->obdobi ->zacatek, $this ->obdobi ->konec)) {
            MessageManager::addWarning(TextManager::_('MV_nelze_editovat_Obdobi_uzavreno_ve_mzdach'));
            return false;
        }

        return true; // OK, muze se editovat
    }



    /**
     * @param $polozka PolozkaEditace
     * @param $osoba OsobaProEditaci
     * @return bool
     */
    function uzamknoutPolozku($polozka, $osoba)
    {
        if ($polozka ->typ == 'MP'  ||  $polozka ->typ == 'PMP') {
            return $osoba ->uzamknout($this ->obdobi ->konec, RonPortalSettings::getIntValue(self::SETTINGS_UZAMKNOUT_MZDOVE, 1));
        }

        if ($polozka ->typ == 'MV'  ||  $polozka ->typ == 'PMV') {
            return $osoba ->uzamknout($this ->obdobi ->konec, RonPortalSettings::getIntValue(self::SETTINGS_UZAMKNOUT_MESICNI, 1));
        }

        return false;
    }


    /**
     * @param bool $povolitvse
     * Nastavi prava na editaci a prohlizeni mzdovych polozek
     */
    function nastavPrava($povolitvse)
    {
        if ($povolitvse) {
            $this ->f_pravaMP = new PravaNaOsoby(3, '', '', 0);
            $this ->f_pravaMPEdit = new PravaNaOsoby(3, '', '', 0);
            $this ->f_pravaMV = new PravaNaOsoby(3, '', '', 0);
            $this ->f_pravaMVEdit = new PravaNaOsoby(3, '', '', 0);
        } else {
            $this ->f_pravaMP = Prava::vratPrava('zobrazit_mzdpol');
            $this ->f_pravaMPEdit = Prava::vratPrava('vkladani_mzdpol');
            $this ->f_pravaMV = Prava::vratPrava('prohlizeni');
            $this ->f_pravaMVEdit = Prava::vratPrava('editace_mv');
        }
    }


    /** @return object|string */
    abstract public function getLogDescription();


    /**
     * @param PracovniPomer $pp
     * @return bool   Zkontroluje jestli je mzda jeste editovatelna a v pripade potreby doplni zaznam mzdy
     */
    function zkontrolujMzdu($pp)
    {
        if (Obdobi::jeObdobiUzavreno($this ->database, 'MZDY', $this ->obdobi ->zacatek, $this ->obdobi ->zacatek)) {
            return false;
        }

        return MzdyUtils::zkontrolujZaznamMzdy($this ->database, $pp, $this ->obdobi ->zacatek, $this ->getLogDescription() );
    }


    /**
     * @param OsobaProEditaci $pp
     * @param PolozkaEditace $polozka
     * @param float $castka
     * @param float $cenaZaJednotku
     * @param float $pocetJednotek
     * @return bool|int
     * @throws Exception
     */
    function zapisMzdovouPolozkuDoDB($pp, $polozka /*$kodMP, $rozlisitRozdeleni*/, $castka, $cenaZaJednotku = 0.0, $pocetJednotek = 0.0)
    {
        if (!$this ->zkontrolujMzdu($pp)) {
            return MessageManager::addErrorMessage('Chyba pri zapisu mzdovych polozek - pravdepodobne je jiz uzamcena mzda!');
        }

        if (!$cenaZaJednotku && !$pocetJednotek) {
            $cenaZaJednotku = (float)$castka;
            if ($cenaZaJednotku) {
                $pocetJednotek = 1;
            }
        }

        $kodTypuRozdeleni = null;
        $rozdeleni = null;

        if ($polozka ->rozlisitRozdeleni == 3) {
            return true; // nemelo by nastat - pokud je rozlisitRozdeleni=3, tak je editace zakazana
        } else if ($polozka ->rozlisitRozdeleni == 2) {
            // rozlisuje se rozdeleni - podle rozdeleni zamestnance
            $kodTypuRozdeleni = $this ->f_TypRozdeleni;
            $rozdeleni = $pp ->kodRozdeleni;
            if (!isset($rozdeleni)) {
                throw new Exception('Chyba - ' . $pp ->jmenoPrijmeni . ', ' . ' os. c. ' . $pp ->osobniCislo . ' nema prirazeno rozdeleni!');
            }
        } else if ($polozka ->rozlisitRozdeleni && $this ->f_TypRozdeleni && $this ->f_Rozdeleni) {
            // rozlisuje se rozdeleni - podle uzivatelem vybraneho rozdeleni ve formulari
            $kodTypuRozdeleni = $this->f_TypRozdeleni;
            $rozdeleni = $this->f_Rozdeleni;
        }

        $rec = new MzdovaPolozkaVeMzdeRecord($this ->database,
            $pp, $this ->obdobi ->zacatek, $polozka ->kod,
            $this,
            $kodTypuRozdeleni, $rozdeleni);

        return $rec ->zapisHodnotu(
            (float)$castka, (float)$pocetJednotek, (float)$cenaZaJednotku,
            $kodTypuRozdeleni, $rozdeleni);
    }


    /**
     * Vraci seriove cislo mzdove polozky s kodem $kodMP ve mzde osoby nebo null
     * @param OsobaProEditaci $osoba
     * @param PolozkaEditace $polozka
     * @param string $tabulka
     * @return bool|int|null
     * @throws DBException
     */
    function vratSerioveCisloMP($osoba, $polozka, $tabulka = 'MzdovaPolozkaVeMzde')
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT SerioveCislo FROM ' . $tabulka .
            ' WHERE RC = ' . $params->addParameter($osoba ->rc) .
            '   AND OsobniCislo = ' . $params->addParameter($osoba ->osobniCislo) .
            '   AND DatumNastupu = ' . $params ->addParameter($this ->database ->timestampToParam($osoba ->datumNastupu)) .
            '   AND Obdobi = ' . $params ->addParameter($this ->database ->timestampToParam($this ->obdobi ->zacatek)) .
            '   AND DruhZdrojeMzdovePolozky = 4' . // jen MP z dochazky
            '   AND MzdovaPolozka = ' . $params->addParameter($polozka ->kod);

        //$polozka ->kod, $polozka ->rozlisitRozdeleni
        if ($polozka ->rozlisitRozdeleni && $this ->f_TypRozdeleni && $this ->f_Rozdeleni) {
            // jen mzdove polozky daneho rozdeleni
            $sql .=
                ' AND KodTypuRozdeleni = ' . $params ->addParameter((int)$this ->f_TypRozdeleni) .
                ' AND Rozdeleni = ' . $params->addParameter($this ->f_Rozdeleni);
        }

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            return $row[0];
        }

        return null; // mzdova polozka jeste neexistuje
    }


    /**
     * @param OsobaProEditaci $osoba
     * @param PolozkaEditace $polozka
     * @param $poznamka
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    public function zapisPoznamkuMzdovePolozkyDoDB($osoba, $polozka, $poznamka)
    {
        // nejdrive zjisti seriove cislo MP
        $serial = $this ->vratSerioveCisloMP($osoba, $polozka);
        if (!$serial) {
            if (!$poznamka) {
                // poznamka je prazdna a MP neexistuje (neexistuje ani poznamka) => neni co zapsat
                return true;
            }

            $this ->zapisMzdovouPolozkuDoDB($osoba, $polozka, 0, 0.0, 0.0);
            // TODO: tady by bylo lepsi pouzit vlozene seriove cislo a ne ho zbytecne znovu nacitat
            $serial = $this ->vratSerioveCisloMP($osoba, $polozka);
        }

        // POZOR, priznak "rozlisitRozdeleni" nefunguje spravne!
        $rec = new RonPortalDBRecord($this ->database, 'PoznamkaMzdovePolozky', $this);
        $rec ->addField('RC', '', true, 15, $osoba ->rc);
        $rec ->addField('OsobniCislo', '', true, 15, $osoba ->osobniCislo);
        $rec ->addField('DatumNastupu', 'timestamp', true, 0, $osoba ->datumNastupu);
        $rec ->addField('', '', true, 0, '   ' . $osoba ->jmenoPrijmeni);
        $rec ->addField('Obdobi', 'timestamp', true, 0, $this ->obdobi ->zacatek);
        $rec ->addField('DruhZdrojeMzdovePolozky', 'int', true, 0, 4);
        $rec ->addField('MzdovaPolozka', '', true, 10, $polozka ->kod);
        $rec ->addField('SerioveCislo', 'int', true, 0, $serial);
        $rec ->addField('Poznamka', 'memo', false, 0, $poznamka);
        $rec ->addField('Insert_UserName', '', true, 25, RonPortalUtils::getLogUserName(true));
        $rec ->addField('Insert_Timestamp', 'timestamp', true, 0, time());
        $f = $rec ->addField('Update_UserName', '', false, 25, RonPortalUtils::getLogUserName(true));
        $f ->updateLog = true;
        $f = $rec ->addField('Update_Timestamp', 'timestamp', false, 0, time());
        $f ->updateLog = true;

        $params = $this ->database ->createParameters();
        $whereSQL =
            'RC = ' . $params->addParameter($osoba ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($osoba ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->timestampToParam($osoba ->datumNastupu)) .
            ' AND Obdobi = ' . $params ->addParameter($this ->database ->timestampToParam($this ->obdobi ->zacatek)) .
            ' AND DruhZdrojeMzdovePolozky = 4' . // jen MP z dochazky
            ' AND MzdovaPolozka = ' . $params->addParameter($polozka ->kod) .
            ' AND SerioveCislo = ' . $params ->addParameter((int)$serial);
        $rec ->setWhere($whereSQL, $params);

        $rec ->updateOrInsert();

        return true;
    }


    /**
     * @param OsobaProEditaci $osoba
     * @param string $eventName
     * @param bool $jeZmena
     * @return bool
     */
    function executeXmlEvent($osoba, $eventName, $jeZmena)
    {
        try {
            $variables = [
                'idPP' => $osoba->idPP,
                'RC' => $osoba->rc,
                'osobniCislo' => $osoba->osobniCislo,
                'datumNastupu' => $osoba->datumNastupu,
                'jmeno' => $osoba->jmeno,
                'prijmeni' => $osoba->prijmeni,
                'jmenoPrijmeni' => $osoba->jmenoPrijmeni,
                'obdobi' => $this->obdobi->zacatek,
                'jeZmena' => $jeZmena ? 1 : 0
            ];

            return RonPortalXMLForm::xmlEvents($this ->database, $eventName, $variables, '', false, false);
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            return false;
        }
    }
}
