<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\RonException;

class Kalendar
{
    private static $svatkyVObdobi = null;

    /** @var string */
    public $kod;
    /** @var string */
    public $popis;

    /** @var int */
    public $dochazkaPodlePlanKapacit;

    /** @var int    Platnost prirazeni kalendare od */
    public $platnostOd;
    /** @var int    Platnost prirazeni kalendare do */
    public $platnostDo;

    /**
     * Nacte seznam svatku pro zadane obdobi.
     * Pokud je datumod (datumdo) = 0, tak vezme datum podle aktualne zvoleneho obdobi
     * @param Database $database
     * @param int    $datumOd
     * @param int    $datumDo
     * @param string $kalendar
     * @return array
     */
    static function nactiSvatkyVObdobi($database, $datumOd = 0, $datumDo = 0, $kalendar = '')
    {
        $result = [];

        if (!$datumOd) $datumOd = RonPortalSession::obdobi();
        if (!$datumDo) $datumDo = RonPortalSession::konecObdobi();

        $params = $database ->createParameters();
        $sql =
            'SELECT DISTINCT Den FROM DenniUdajKalendare ' .
            ' WHERE Den >= ' . $params ->addParameter($database ->dateToParam($datumOd)) .
            ' AND Den <= ' . $params ->addParameter($database ->dateToParam($datumDo)) .
            ' AND (TypDne = 2 OR TypDne = 3) '; // placeny nebo neplaceny svatek

        if ($kalendar) $sql .= ' AND Kalendar = ' . $params ->addParameter($kalendar);

        $sql .= ' ORDER BY Den ';

        $cursor = $database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $result[] = $cursor ->stringToTimestamp($row[0]);
        }

        $cursor ->close();

        return $result;
    }


    /**
     * Vraci true, pokud je zadany den svatek
     * @param Database $database
     * @param int $datum
     * @param string $kalendar
     * @return bool
     */
    static function jeSvatek($database, $datum, $kalendar)
    {
        if (!isset(self::$svatkyVObdobi)) {
            try {
                self::$svatkyVObdobi = self::nactiSvatkyVObdobi($database, 0, 0, $kalendar);
            } catch (RonException $e) {
                MessageManager::addDebugLog($e ->getMessage() . "\n" . $e ->getErrorDescription());
            } catch (\Exception $e) {
                MessageManager::addDebugLog($e ->getMessage());
            }
        }

        return in_array($datum, self::$svatkyVObdobi);
    }

    const VIKEND = 1;
    const SVATEK = 2;

    /**
     * Vraci 0 pro pracovni den, 1 pro vikend, 2 pro svatek, 3 pro vikend-svatek
     * pokud je $nacistsvatky = false, tak svatky nevyhledava
     * (napr. v historii pruchodu, protoze v APS nejsou kalendare)
     * pokud pri nacitani svatku neni zadan kalendar, tak hleda svatky ve vsech kalendarich
     * POZOR! Pokud je metoda volana s nejakym kalendarem, tak pri dalsim volani uz se pouzije ten samy!
     *
     * @param Database $database
     * @param int    $datum
     * @param bool   $nacistSvatky
     * @param string $kalendar
     * @return int
     */
    static function vratTypDne($database, $datum, $nacistSvatky = true, $kalendar = '')
    {
        $jesvatek = $nacistSvatky ? self::jeSvatek($database, $datum, $kalendar) : false;

        switch (date('w', $datum)) {
            case 6 : // sobota
            case 0 : // nedele
                return self::VIKEND + ($jesvatek ? self::SVATEK : 0); // vikend (pripadne i svatek)

            default :
                return $jesvatek ? self::SVATEK : 0; // svatek / normalni den
        }
    }


    /**
     * Vraci jmeno css tridy pro zadany index dne (nedele=0, pondeli=1, ...)
     * @param $dayIndex
     * @return string
     */
    static function getStyleForDay($dayIndex)
    {
        if ( $dayIndex == 6 || $dayIndex == 0 ) return 'browser_row_high'; // vikend

        return '';
    }


    /**
     * Vraci jmeno css tridy pro zadany datum
     * @param $database
     * @param $datum
     * @param bool|true $nacistSvatky
     * @param string $kalendar
     * @return string
     */
    static function getStyleForDate($database, $datum, $nacistSvatky = true, $kalendar = '')
    {
        $result = (DateUtils::startOfTheDay($datum) == DateUtils::today()) ? 'today ' : '';

        if ($nacistSvatky) {
            $typdne = self::vratTypDne($database, $datum, $nacistSvatky, $kalendar);
            switch ($typdne) {
                case 1 :
                    return $result . 'browser_row_high vikend';

                case 2 :
                    return $result . 'browser_row_high svatek';

                case 3 :
                    return $result . 'browser_row_high vikend svatek';

                default :
                    break;
            }
        }

        return $result . self::getStyleForDay(date('w', $datum));
    }


    /**
     * @param Database $database
     * @param string $kod
     * @return Kalendar
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public static function nactiKalendar($database, $kod)
    {
        $params = $database ->createParameters();
        $cursor = $database ->getSqlResult(
            'SELECT Kod, Popis, DochazkaPodlePlanKapacit' .
            ' FROM Kalendar WHERE Kod = ' . $params ->addParameter($kod),
            $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new \Exception('Chyba ve funkci Kalendar::nactiKalendar - zadany kalendar neexistuje!');
        }

        $result = new Kalendar();
        $result ->kod = $row[0];
        $result ->popis = $row[1];
        $result ->dochazkaPodlePlanKapacit = $row[2];

        return $result;
    }


    /**
     * Nacte kalendare prirazene PP v zadanem obdobi
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return Kalendar[]
     */
    static function nactiKalendareOsoby($database, $pp, $obdobiOd, $obdobiDo)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT Kalendar.Kod, Kalendar.Popis, Kalendar.DochazkaPodlePlanKapacit, PrirazeniKalendare.PlatnostOd, PrirazeniKalendare.PlatnostDo' .
            ' FROM PrirazeniKalendare, Kalendar ' .
            ' WHERE Kalendar.Kod = PrirazeniKalendare.Kalendar' .
            '   AND RC = '. $params ->addParameter($pp ->rc) .
            '   AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            '   AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            '   AND PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
            '   AND (PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . ' OR PlatnostDo IS NULL)' .
            ' ORDER BY PlatnostOd ';

        $result = [];

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani prirazeneho kalendare!');
        while ($row = $cursor ->fetch()) {
            $kalendar = new static();

            $kalendar ->kod = $row[0];
            $kalendar ->popis = $row[1];
            $kalendar ->dochazkaPodlePlanKapacit = $row[2];

            $kalendar ->platnostOd = $database ->stringToTimestamp($row[3]);
            $kalendar ->platnostDo = $database ->stringToTimestamp($row[4]);

            $result[] = $kalendar;
        }
        $cursor ->close();

        return $result;
    }


    /**
     * @param Database $database
     * @param string $rc
     * @param string $osobniCislo
     * @param int  $datumNastupu
     * @param int  $obdobiOd
     * @param int  $obdobiDo
     * @return string|null
     */
    static function vratKalendarOsoby($database, $rc, $osobniCislo, $datumNastupu, $obdobiOd, $obdobiDo)
    {
        global $bezdochazky;
        if (isset($bezdochazky) && $bezdochazky) return null; // v pristupaku nejsou kalendare

        $params = $database ->createParameters();
        $sql =
            'SELECT Kalendar FROM PrirazeniKalendare ' .
            ' WHERE RC = '. $params ->addParameter($rc) .
            '   AND OsobniCislo = ' . $params ->addParameter($osobniCislo) .
            '   AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($datumNastupu)) .
            '   AND PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
            '   AND (PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . ' OR PlatnostDo IS NULL)' .
            ' ORDER BY PlatnostOd ';

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani prirazeneho kalendare!');
        $row = $cursor ->fetch();
        $cursor ->close();
        return $row ? $row[0] : null;
    }


    /**
     * @param Database $database
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return string|null
     */
    static function vratKalendarUzivatele($database, $obdobiOd, $obdobiDo)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT Kalendar FROM PrirazeniKalendare ' .
            ' WHERE RC = '. $params ->addParameter(User::getRC()) .
            '   AND PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
            '   AND (PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . ' OR PlatnostDo IS NULL)' .
            ' ORDER BY PlatnostOd ';

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani prirazeneho kalendare!');
        $row = $cursor ->fetch();
        $cursor ->close();
        return $row ? $row[0] : null;
    }
}
