<?php

/**
 * stanislav.elbl, 02.03.2021
 */

namespace RonPortal\Zamestnanci;

use Exception;
use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Personalistika\PersonUtils;
use RonPortal\Personalistika\RodneCislo;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalUtils;
use RonPortal\RozdeleniOsob;
use RonPortal\XML\RonPortalXMLForm;


//class NovyZamestnanecFormTable
//{
//    public $sqlTableName;
//    public $alias = null;
//
//    public $enabled; // ? ?
//
//    public $foreignKeyFields; // RC nebo RC;OsobniCislo;DatumNastupu
//
//    public $platnostOdField = null;
//    public $platnostDoField = null;
//}


/**
 * Formularove pole pro nastaveni PlatnostOd u podrizenych zaznamu.
 * Class NovyZamestnanecPlatnostOdField
 * @package RonPortal\Zamestnanci
 */
class NovyZamestnanecPlatnostOdField extends FormField
{
    /** platnost od se nastavi podle datumu nastupu */
    const PLATNOST_DLE_PP = 'datumnastupu';
    /** platnost od se nastavi podle aktualniho data */
    const PLATNOST_TODAY = 'datum';
    /** platnost od bude editovatelna */
    const PLATNOST_EDIT = 'edit';


    /** @var string     Urcuje, podle ceho se nastavi vychozi hodnota */
    public $vychoziNastaveni = self::PLATNOST_DLE_PP;

    /** @var bool Urcuje, jestli se musi vzdy priradit platnost od zacatku mesice */
    public $zacatekMesice = false;

    /**
     * NovyZamestnanecPlatnostOdField constructor.
     * @param string $vychoziNastaveni  Jedna z konstant PLATNOST_...
     * @param bool $zacatekMesice
     */
    public function __construct($vychoziNastaveni = self::PLATNOST_DLE_PP, $zacatekMesice = false)
    {
        parent::__construct($vychoziNastaveni == self::PLATNOST_EDIT
            ? FormField::FIELD_TYPE_DATE
            : FormField::FIELD_TYPE_CONSTANT);

        $this ->vychoziNastaveni = $vychoziNastaveni;
        $this ->zacatekMesice = $zacatekMesice;

        $this ->setDataType('timestamp');
    }


    /**
     * @param int $datumNastupu
     */
    public function nastavPodleDatumuNastupu($datumNastupu)
    {
        //echo "Nastavuji podle nastupu: {$this ->tableName}.{$this ->fieldName} = $datumNastupu <br/>\n";
        if ($this ->fieldType == FormField::FIELD_TYPE_CONSTANT) {
            $datum = $datumNastupu;
            if ($this ->vychoziNastaveni == self::PLATNOST_TODAY) {
                $datum = DateUtils::today();
            }

            $this ->defaultValue = $this ->zacatekMesice
                ? DateUtils::firstDayOfMonth($datum)
                : $datum;
        }
    }
}


class NovyZamestnanecRCField extends FormField
{
    public $showReadonly = false;

    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($this ->showReadonly) {
            echo '<input type="hidden" name="' . $this ->fieldName . '" value="' . StringUtils::htmlTextConvert($value) . '"/>';
            echo StringUtils::htmlTextConvert($value);
        } else {
            parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
        }
    }
}


/**
 * Pole pro potvrzeni vadneho RC (spatny format). Zobrazuje zaskrtavatko pro potvrzeni zapisu vadneho RC.
 * Class NovyZamestnanecRCErrorField
 * @package RonPortal\Zamestnanci
 */
class NovyZamestnanecRCErrorField extends FormField
{
    /** @var string     Text chybove hlasky z kontroly RC */
    public $errorMessage;

    /** @var RodneCislo */
    public $analyzaRC = null;

    /**
     * NovyZamestnanecRCErrorField constructor.
     */
    public function __construct()
    {
        parent::__construct(self::FIELD_TYPE_CHECK);
        $this
            ->setFieldName('*RC_Zkontrolovano')
            ->setRowClass('row errortext')
            ->setCaption2('Potvrzuji_spatne_RC');
    }

    public function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if ($this ->errorMessage) {
            echo '<div class="errorcolor">' . $this ->errorMessage . '</div>';
        }

        if (!$this ->readOnly) {
            parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
        } else {
            echo '<input type="hidden" name="RC_Zkontrolovano" value="1"/>';
        }
    }


    /**
     * Provede analyzu zadaneho RC a podle vysledku nastavi pole formulare
     * @param string $rc
     */
    public function analyzujRC($rc)
    {
        $this ->analyzaRC = PersonUtils::analyzujRC($rc);

        $this ->errorMessage = $this ->analyzaRC ->error;
        $this ->setVisible((bool)$this ->analyzaRC ->error);
    }


    public function lzePokracovat()
    {
        if (!$this ->visible) {
            return true;
        }

        return isset($_POST['RC_Zkontrolovano'])  &&  (int)$_POST['RC_Zkontrolovano'];
    }
}




class NovyZamestnanecForm extends RonPortalForm
{
    const STATUS_RC = 'rc';
    const STATUS_ALL = 'all';

//    /**
//     * @var string     Mod formulare:
//     *      rc  Nejdrive se zadava RC a pak teprve zbytek. Po zadani RC se provede kontrola existence a pripadne nacteni udaju osoby
//     */
//    protected $mode = 'rc';

    /**
     * @var string      Aktualni stav formulare
     *      rc      Uzivatel zadava jen RC. Po jeho predani se prepne do dalsiho kroku
     *      all     Uzivatel zadava vsechny dalsi udaje
     */
    protected $status = self::STATUS_RC;


    /** @var bool   Priznak, ze je nainstalovana personalistika nebo mzdy */
    protected $jePerson;

    /** @var string[]   Seznam podrizenych tabulek napojenych na tabulku Osoba (musi obsahovat pole RC) */
    protected $tablesOsoba = [
        'Osoba', 'OsobniUdaj'
    ];

    /** @var string[]   Seznam podrizenych tabulek napojenych na tabulku PracovniPomer (musi obsahovat RC, OsobniCislo, DatumNastupu */
    protected $tablesPP = [
        //'PracovniPomer'
    ];

    /** @var NovyZamestnanecRCField */
    protected $fRC;
    /** @var NovyZamestnanecRCErrorField  Pole informujici o pripadne chybe v rodnem cisle */
    protected $fRCError;
    /** @var FormField */
    protected $fDatumNarozeni;
    /** @var FormField */
    protected $fPohlavi;
    /** @var FormField */
    protected $fCisloPojistence;
    /** @var FormField */
    protected $fSerioveCisloUctu;

    /** @var RozdeleniZamestnanceField[] */
    protected $rozdeleniFields = [];

    /** @var string     Id osoby, pro kterou se vytvari novy PP (Osoba.RC). Pouzije se v pripade zadavani noveho PP pro existujici osobu */
    protected $osobaRC = null;
    /** @var bool       Nastavi se na true, pokud se jedna o osobu, ktera ma jeste dalsi aktivni pracovni pomer */
    protected $existujeAktivniPP = false;
    /** @var int        Id editovaneho PP - pouzije se pri editaci z XML formulare. Nastavi se take po uspesnem zapisu zaznamu do databaze. */
    protected $idPP = null;
    /** @var PracovniPomer  Editovany pracovni pomer nacteny podle $idPP */
    protected $pp = null;
    /** @var int            Celkovy pocet existujicich pracovnich pomeru osoby (nastavi se v pripade editace) */
    protected $pocetPP = 0;

    protected $zobrazenoZXML = false;

    protected $novyZamestnanecConfiguration = null;

    /**
     * NovyZamestnanecForm constructor.
     * @param Database $database
     * @param string $formAction
     * @param int $idPP Editovany pracovni pomer - pro pouziti v XML formulari
     * @param bool $zobrazenoZXML
     */
    public function __construct($database, $formAction = 'new', $idPP = null, $zobrazenoZXML = false)
    {
        parent::__construct($database, $formAction);

        $this ->idPP = $idPP;   // editovany pracovni pomer - pouzije se pri editaci z XML formulare

        if ($this ->idPP) {
            $this ->status = self::STATUS_ALL;

            $this ->pp = PracovniPomer::nactiPP($database, $this ->idPP);
            $this ->pocetPP = ZamestnanciUtils::pocetPPOsoby($database, $this ->pp ->rc);
        }

        $this ->title = TextManager::_('Novy_zamestnanec');
        $this ->code = 'formnovyzamestnanec';
        $this ->htmlParams['id'] = 'formnovyzamestnanec';

        $this ->defaultRowClass = 'row';
        $this ->defaultCaptionColClass = 'col-12 col-sm-3 caption';
        $this ->defaultDataColClass = 'col-12 col-sm-9';

        $this ->zobrazenoZXML = $zobrazenoZXML;
        if ($this ->zobrazenoZXML) {
            $this ->renderHtmlForm = false; // negeneruje html tag <form>, protoze je soucasti XML formulare
        }

        $this ->jePerson = RonPortalUtils::jeNainstalovanaAplikace($database, ['MZDY', 'PERSON']);

        if ($this ->jePerson) {
            array_push($this ->tablesOsoba,
                'TrvaleBydliste', 'PrechodneBydliste',
                'ZdravotniStavOsoby', 'PojisteniOsobyU_ZP',
                'VzdelaniOsoby', 'RodinnyStav'
            );
        }

        // Jednotliva editovaci pole prida az po nacteni zakladni konfigurace
    }

    /**
     * @return int
     */
    public function getIdPP()
    {
        return $this->idPP;
    }

    /**
     * @return string
     */
    public function getStatus()
    {
        return $this->status;
    }


    /**
     * Nastavi dodatecnou konfiguraci formulare - pouziva se z XML tagu <nastupniformular>
     * @param array $novyZamestnanecConfiguration
     */
    public function setNovyZamestnanecConfiguration($novyZamestnanecConfiguration)
    {
        $this->novyZamestnanecConfiguration = $novyZamestnanecConfiguration;
    }

    /**
     * Prida pole pro zadani rodneho cisla a jeho kontrolu
     */
    private function addFieldsRodneCislo()
    {
        $this ->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('Id', 'PracovniPomer') ->setPrimary(true)
            ->setReadOnly(true)
            ->setCode('field-idpp');

        $this ->fRC = new NovyZamestnanecRCField();
        $this ->fRC
            ->setFieldName('RodneCislo', 'OsobniUdaj') ->setCaption('RC') ->setRequired($this ->jePerson)
            ->setCode('field-rodnecislo') ->setFieldId('osoba_rodnecislo') ->setMaxLength(15)
            ->setFieldCssClass('form-control keyFilter')
            ->setOtherHtmlParams([
                'data-allowed-chars' => '01123456789', // jen cisla
                'autocomplete' => 'off']
            )
            ->setVisible($this ->jePerson); // kdyz je jen dochazka, tak je implicitne skryte

        $this->addField($this->fRC);

        $this ->fRCError = new NovyZamestnanecRCErrorField();
        $this ->fRCError
            ->setCode('field-rc-error')
            ->setVisible(false);    // zobrazi se jen pri chybe v RC
        $this ->addField($this ->fRCError);

        if (isset($_POST['OsobniUdaj#RodneCislo'])) {
            $this ->fRCError ->analyzujRC(trim($_POST['OsobniUdaj#RodneCislo']));
        }
    }


    private function addNovyZamestnanecFields()
    {
        $this->fRC->showReadonly = true;
        $this->fRCError ->setReadOnly(true);

        $this ->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-zakladniudaje') ->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('zakladni_udaje');

        // zakladni udaje
        $this ->addField()
            ->setFieldName('Titul', 'Osoba') ->setCaption('Titul')
            ->setCode('field-titul')
            ->setFieldId('osoba_titul')
            ->setMaxLength(20);

        // TODO: podle nastaveni udelat editovatelne
        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('RC', 'Osoba') ->setCaption('Id_osoby') ->setRequired(false) // nastavuje se automaticky az tesne pred POST
            ->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode('field-osoba-rc');

        $this ->addField()
            ->setFieldName('Jmeno', 'Osoba') ->setCaption('Jmeno') ->setRequired(true)
            ->setMaxLength(30)
            ->setCode('field-jmeno');

        $this ->addField()
            ->setFieldName('Prijmeni', 'Osoba') ->setCaption('Prijmeni') ->setRequired(true)
            ->setMaxLength(30)
            ->setCode('field-prijmeni');

        $this ->addField()
            ->setFieldName('TitulZa', 'Osoba') ->setCaption('Titul_za')
            ->setMaxLength(20)
            ->setCode('field-titulza');

        // osobni udaje
        $this ->addFieldsOsobniUdaje();

        $this ->addFieldsKontakty();

        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka', 'Osoba') ->setCaption('Poznamka')
            ->setCode('field-poznamka');

        $this ->addUserLogFields('Osoba');

        // TODO:
        // OsobniUdaj.RC_Zahranicni 15

        if ($this ->jePerson) {
            // Trvale a prechodne bydliste
            $this ->addFieldsAdresa('TrvaleBydliste', 'Trvale_bydliste', 1);
            $this ->addFieldsAdresa('PrechodneBydliste', 'Prechodne_bydliste', 0);

            // Zdravotni pojisteni
            $this ->addFieldsZP();
            // Zdravotni stav
            $this ->addFieldsZdravotniStav();
            // vzdelani
            $this ->addFieldsVzdelani();
            // rodinny stav
            $this ->addFieldsRodinnyStav();
        }

        // Rodinni prislusnici
        // - zadavani
        // + najit rodinne prislusniky

        // Pracovni pomer
        $this ->addFieldsPracovniPomer();

        if ($this ->jePerson) {
            $this->addFieldsVyplaceniMzdy();
        }

        // !Evidencni stav

        // !Kalendar

        // Medium

        // Pracovni pozice

        // Mzdovy vymer
        // - !typ mzdy, !kategorie, popis, zaloha, premie pevne, premie pohyblive, osobni priplatek, priplatek za vedeni,jiny

        // Udaje tykajici se smluvni mzdy
        // - mzdovy vymer
        // - nejnizsi uroven zarucene mzdy

        // Login, Profil
    }


    private function addFieldsOsobniUdaje()
    {
        $this ->addField(FormField::FIELD_TYPE_CONSTANT)    // nastavi se podle Osoba.RC
            ->setFieldName('RC', 'OsobniUdaj') ->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode('field-osobniudaj-rc');

        $this ->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-osobniudaje') ->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('osobni_udaje');

        $this ->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-osobniudaje')
            ->setCaption('Osobni_udaje');

        $this ->fDatumNarozeni = $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumNarozeni', 'OsobniUdaj') ->setCaption('Datum_narozeni')
            ->setFieldId('osoba_datumnarozeni')
            ->setCode('field-datumnarozeni');

        $this ->addField()
            ->setFieldName('RodneJmeno', 'OsobniUdaj') ->setCaption('Rodne_jmeno')
            ->setMaxLength(15)
            ->setCode('field-rodneprijmeni');

        $this ->fPohlavi = $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Pohlavi', 'OsobniUdaj') ->setDataType('int') ->setCaption('Pohlavi')
            ->setCode('field-pohlavi')
            ->setFieldId('osoba_pohlavi')
            ->setSelectSql('SELECT Kod, Popis FROM S_Pohlavi ORDER BY Kod');

        // narodnost atd.
        $this ->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-osobniudaje2') ->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('osobni_udaje2');

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Narodnost', 'OsobniUdaj') ->setDataType('int') ->setCaption('Narodnost')
            ->setCode('field-narodnost')
            ->setFieldId('osoba_narodnost')
            ->setSelectSql('SELECT Kod, Popis FROM Narodnost WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setPreferredValues([1, 2])     // 1=ceska, 2=slovenska
            ->setVisible($this ->jePerson); // kdyz je jen dochazka, tak je implicitne skryte

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('StatniPrislusnost', 'OsobniUdaj') ->setCaption('Statni_prislusnost')
            ->setCode('field-statniprislusnost')
            ->setFieldId('osoba_statniprislusnost')
            ->setSelectSql('SELECT Kod, Popis FROM Stat WHERE (%FILTER%) ORDER BY %ORDER%','1 = 1', 'Popis')
            ->setFieldCssClass('form-control ronselect')
            ->setPreferredValues(['CZ', 'SK'])
            ->setVisible($this ->jePerson); // kdyz je jen dochazka, tak je implicitne skryte

        $this ->addField()
            ->setFieldName('MistoNarozeni', 'OsobniUdaj') ->setCaption('Misto_narozeni')
            ->setMaxLength(30)
            ->setCode('field-mistonarozeni')
            ->setVisible($this ->jePerson); // kdyz je jen dochazka, tak je implicitne skryte

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('OkresNarozeni', 'OsobniUdaj') ->setCaption('Okres_narozeni')
            ->setCode('field-okresnarozeni')
            ->setFieldId('osoba_okresnarozeni')
            ->setSelectSql('SELECT Kod, Popis FROM Okres WHERE (%FILTER%) ORDER BY %ORDER%','1 = 1', 'Popis')
            ->setFieldCssClass('form-control ronselect')
            ->setVisible($this ->jePerson); // kdyz je jen dochazka, tak je implicitne skryte
    }

    /**
     * Prida pole pro editaci zakladnich kontaktu
     */
    private function addFieldsKontakty()
    {
        // zakladni kontakty (email, telefon)
        $this ->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-kontakty') ->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('kontakty');

        $this ->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-kontakty')
            ->setCaption('Kontakty');

        $this ->addField()
            ->setFieldName('EMail', 'Osoba') ->setCaption('E_mail')
            ->setMaxLength(60)
            ->setCode('field-email');

        $this ->addField()
            ->setFieldName('Telefon', 'Osoba') ->setCaption('Telefon')
            ->setMaxLength(60)
            ->setCode('field-telefon');

        $this ->addField()
            ->setFieldName('Mobil', 'Osoba') ->setCaption('Mobil')
            ->setMaxLength(20)
            ->setCode('field-mobil');
    }


    private function addFieldsAdresa($tableAlias, $header, $trvale)
    {
        $code = strtolower($tableAlias);

        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode($trvale ? 'group-trvalebydliste' : 'group-prechodnebydliste')
            ->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId($code);

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode("header-{$code}")
            ->setCaption($header);

        if (!$trvale) {
            $this->addField(FormField::FIELD_TYPE_NOT_NULL_FLAG)
                ->setCode("field-{$code}-exists") ->setReadOnly(true)
                ->setSqlFieldName('PlatnostOd', 'AdresaOsoby', $tableAlias)->setFieldName("{$code}_existuje")
                ->setFieldId("osoba_{$code}_existuje")
                ->setCaption2('Prechodne_bydliste_je_rozdilne')
                ->setFieldCssClass('toggleClass toggleOnLoad')
                ->setAppendFormClass('je-prechodne-bydliste')
                ->setOtherHtmlParams([
                    'data-selector' => '#formnovyzamestnanec',
                    'data-check-class' => 'je-prechodne-bydliste'
                ]);
        }

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)    // nastavi se podle Osoba.RC
            ->setFieldName('RC', 'AdresaOsoby', $tableAlias) ->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode("field-{$code}-rc");

        $this ->addField(new NovyZamestnanecPlatnostOdField(NovyZamestnanecPlatnostOdField::PLATNOST_TODAY))
            ->setFieldName('PlatnostOd', 'AdresaOsoby', $tableAlias)
            ->setCode("field-{$code}-platnostod")
            ->setDataType('date')
            ->setDefaultValue(DateUtils::today());

        // priznak "Trvale bydliste"
        $this->addField(FormField::FIELD_TYPE_CONSTANT) ->setReadOnly(true)
            ->setCode("field-{$code}-trvale")
            ->setSqlFieldName('TrvaleBydliste', 'AdresaOsoby', $tableAlias)
            ->setDefaultValue($trvale);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Stat', 'AdresaOsoby', $tableAlias)->setCaption('Stat')
            ->setCode("field-{$code}-stat")
            ->setFieldId("osoba_{$code}_stat")
            ->setSelectSql('SELECT Kod, Popis FROM Stat WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setPreferredValues(['CZ', 'SK'])
            ->setFieldCssClass('form-control ronselect');

        $this ->addField()
            ->setFieldName('Ulice', 'AdresaOsoby', $tableAlias) ->setCaption('ulice')
            ->setMaxLength(60)
            ->setCode("field-{$code}-ulice")
            ->setFieldId("osoba_{$code}_ulice");

        $this ->addField()
            ->setFieldName('PSC', 'AdresaOsoby', $tableAlias) ->setCaption('psc')
            ->setMaxLength(10)
            ->setCode("field-{$code}-psc")
            ->setFieldId("osoba_{$code}_psc")
            ->setFieldCssClass('form-input psc-input')
            ->setOtherHtmlParams([
                "data-mesto-input-selector" => "#osoba_{$code}_mesto"
            ]);


        $this ->addField()
            ->setFieldName('Mesto', 'AdresaOsoby', $tableAlias) ->setCaption('mesto')
            ->setMaxLength(40)
            ->setCode("field-{$code}-mesto")
            ->setFieldId("osoba_{$code}_mesto")
            ->setFieldCssClass('form-input ronselect mesto-input')
            ->setOtherHtmlParams([
                'data-max-visible-rows' => '50',
                'data-ronselect-class' => 'form-input',
                'data-auto-load-items' => '1',
                'data-items-url' => '?menuid=download&seznammest=1',
                'data-columns' => 'mesto;psc;okresPopis',
                'data-filter-columns' => 'mesto;psc',   // nevyhledava v okresu
                'data-captions' => TextManager::_('Nazev') . ';' . TextManager::_('PSC') . ';' . TextManager::_('Okres'),
                'data-value-property' => 'mesto',
                'data-text-column' => 'mesto',
                'data-only-autocomplete' => '1',        // umoznuje zadat jakykoliv text
                // napojeni na dalsi inputy
                "data-psc-input-selector" => "#osoba_{$code}_psc",
                "data-obec-input-selector" => "#osoba_{$code}_obec"
            ]);

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Obec', 'AdresaOsoby', $tableAlias) ->setCaption('obec')
            ->setMaxLength(40)
            ->setCode("field-{$code}-obec")
            ->setFieldId("osoba_{$code}_obec")
            ->setFieldCssClass('form-input ronselect obec-input') ->setRowClass('row margin-bottom-2')
            ->setSelectSql('SELECT Kod, Popis FROM Obec WHERE 0 = 1')
            ->setOtherHtmlParams([
                'data-max-visible-rows' => '50',
                'data-ronselect-class' => 'form-input',
                'data-auto-load-items' => '1',
                'data-items-url' => '?menuid=download&seznamobci=1',
                'data-columns' => 'popis;okresPopis',
                'data-filter-columns' => 'popis',   // nevyhledava v okresu
                'data-captions' => TextManager::_('Nazev') . ';' . TextManager::_('Okres'),
                'data-value-property' => 'kod',
                'data-value-column' => 'kod',
                'data-text-column' => 'popis',
                // napojeni na dalsi inputy
                "data-mesto-input-selector" => "#osoba_{$code}_mesto"
            ]);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('ZpusobDopravy', 'AdresaOsoby', $tableAlias)->setCaption('Zpusob_dopravy_do_zamestnani')  ->setDataType('int')
            ->setCode("field-{$code}-zpusobdopravy")
            ->setSelectSql('SELECT Kod, Popis FROM DopravaDoZamestnani WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setDefaultValue(0);

        $this ->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('VzdalenostDoZamestnani', 'AdresaOsoby', $tableAlias) ->setCaption('Vzdalenost_do_zamestnani')
            ->setCode("field-{$code}-vzdalenostdozamestnani")
            ->setFieldId("osoba_{$code}_vzdalenostdozamestnani")
            ->setFieldCssClass('inputFloat margin-right-2')
            ->setCaption2('km');

        $this ->addField()
            ->setFieldName('Telefon1', 'AdresaOsoby', $tableAlias) ->setCaption('Telefon')
            ->setMaxLength(20)
            ->setCode("field-{$code}-telefon")
            ->setFieldId("osoba_{$code}_telefon");

        $this ->addField()
            ->setFieldName('Telefon2', 'AdresaOsoby', $tableAlias) ->setCaption('Telefon_2')
            ->setMaxLength(20)
            ->setCode("field-{$code}-telefon2")
            ->setFieldId("osoba_{$code}_telefon2");

        $this ->addField()
            ->setFieldName('Mobil', 'AdresaOsoby', $tableAlias) ->setCaption('Mobil')
            ->setMaxLength(20)
            ->setCode("field-{$code}-mobil")
            ->setFieldId("osoba_{$code}_mobil");

        $this ->addField()
            ->setFieldName('E_Mail', 'AdresaOsoby', $tableAlias) ->setCaption('E_Mail')
            ->setMaxLength(60)
            ->setCode("field-{$code}-email")
            ->setFieldId("osoba_{$code}_email");

    }


    private function addFieldsPracovniPomer()
    {
        // -!Typ PP, !Osobni cislo, !Datum nastupu, Poradove cislo,
        // - cislo smlouvy, datum podpisu, duvod vzniku PP atd.
        $this ->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-pracovnipomer') ->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('pracovnipomer');

        $this ->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-pracovnipomer')
            ->setCaption('Pracovni_pomer');

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)    // doplni se podle RC osoby
            ->setFieldName('RC', 'PracovniPomer')
            ->setReadOnly(true)
            ->setCode('field-pp-rc') ->setMaxLength(15);

        $this ->addField()
            ->setFieldName('OsobniCislo', 'PracovniPomer') ->setCaption('Osobni_cislo')
            ->setCode('field-osobnicislo')
            ->setMaxLength(15) ->setRequired(true);

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumNastupu', 'PracovniPomer') ->setCaption('Datum_nastupu')
            ->setCode('field-datumnastupu') ->setDefaultValue(StringUtils::format('date', DateUtils::today()))
            ->setRequired(true);

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumUkonceni', 'PracovniPomer') ->setCaption('Datum_ukonceni')
            ->setCode('field-datumukonceni');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('TypPracovnihoPomeru', 'PracovniPomer')->setCaption('Typ_pracovniho_pomeru')
            ->setCode('field-typpracovnihopomeru') ->setDefaultValue('HPP')
            ->setRequired(true)
            ->setSelectSql('SELECT Kod, Popis FROM TypPracovnihoPomeru WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('DuvodVznikuPP', 'PracovniPomer')->setCaption('Duvod_vzniku_pp')
            ->setCode('field-duvodvznikupp')
            ->setSelectSql('SELECT Kod, Popis FROM DuvodVznikuPP WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setVisible($this ->jePerson);

        $this ->addUserLogFields('PracovniPomer');
        // Zkusebni doba
        // - mesicu / do, vypoctena
    }


    /**
     * Prida pole z tabulky PojisteniOsobyU_ZP
     */
    private function addFieldsZP()
    {
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-zdravotnipojisteni')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('zdravotnipojisteni');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode("header-zdravotnipojisteni")
            ->setCaption(TextManager::_('Zdravotni_pojisteni'));

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)    // nastavi se podle Osoba.RC
        ->setFieldName('RC', 'PojisteniOsobyU_ZP') ->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode('field-zdravotnipojisteni-rc');

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('KodZdravotniPojistovny', 'PojisteniOsobyU_ZP') ->setCaption('Zdravotni_pojistovna')
            ->setCode('field-kodzdravotnipojistovny')
            ->setSelectSql('SELECT Kod, Popis FROM ZdravotniPojistovna WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setFieldCssClass('form-control ronselect')
            ->setSelectShowCode(true);

        $this ->addField(new NovyZamestnanecPlatnostOdField(NovyZamestnanecPlatnostOdField::PLATNOST_TODAY))
            ->setFieldName('PlatnostOd', 'PojisteniOsobyU_ZP') ->setCaption('Platnost_od')
            ->setCode('field-zdravotnipojisteni-platnostod');

        $this ->fCisloPojistence = $this ->addField()
            ->setFieldName('CisloPojistence', 'PojisteniOsobyU_ZP') ->setCaption('Cislo_pojistence')
            ->setCode('field-cislopojistence') ->setMaxLength(15);

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('CisloUctarny', 'PojisteniOsobyU_ZP') ->setCaption('Cislo_uctarny')
            ->setCode('field-cislouctarny')
            ->setSelectSql('SELECT Kod, Popis FROM Uctarna WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis');

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('TypZamestnance', 'PojisteniOsobyU_ZP') ->setDataType('int') ->setCaption('Typ_zamestnance_ZP')
            ->setCode('field-zdravotnipojisteni-typzamestnance')
            ->setSelectSql('SELECT Kod, Popis FROM S_TypZamestnanceProZP WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Kod');

    }

    /**
     * Prida pole z tabulky ZdravotniStavOsoby
     */
    private function addFieldsZdravotniStav()
    {
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-zdravotnistav')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('zdravotnistav');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode("header-zdravotnistav")
            ->setCaption(TextManager::_('Zdravotni_stav'));

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)    // nastavi se podle Osoba.RC
            ->setFieldName('RC', 'ZdravotniStavOsoby') ->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode("field-zdravotnistav-rc");

//        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
//            ->setFieldName('PlatnostOd', 'ZdravotniStavOsoby') ->setReadOnly(true)
//            ->setCode('field-zdravotnistav-platnostod')
//            ->setDataType('date')
//            ->setDefaultValue(DateUtils::today());

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('ZdravotniStav', 'ZdravotniStavOsoby') ->setDataType('int') ->setCaption('Zdravotni_stav')
            ->setCode('field-zdravotnistav')
            ->setSelectSql('SELECT Kod, Popis FROM S_ZdravotniStav WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Kod');

        $this ->addField(new NovyZamestnanecPlatnostOdField(NovyZamestnanecPlatnostOdField::PLATNOST_TODAY))
            ->setFieldName('PlatnostOd', 'ZdravotniStavOsoby') ->setCaption('Platnost_od')
            ->setCode('field-zdravotnistav-platnostod');

        $this ->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('DrzitelPrukazuZTTP', 'ZdravotniStavOsoby') ->setDataType('int') ->setCaption('Drzitel_prukazu_ZTP')
            ->setCode('field-zdravotnistav-drzitelprukazuztp');

        $this ->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('ZPS', 'ZdravotniStavOsoby') ->setDataType('int') ->setCaption('ZPS')
            ->setCode('field-zdravotnistav-zps');

        $this ->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('ZPS_TZP', 'ZdravotniStavOsoby') ->setDataType('int') ->setCaption('ZPS_TZP')
            ->setCode('field-zdravotnistav-zps_tzp');

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('OsetrujiciLekar', 'ZdravotniStavOsoby') ->setCaption('Osetrujici_lekar') ->setVisible(false)
            ->setCode('field-osetrujicilekar')
            ->setFieldCssClass('form-control ronselect')
            ->setSelectSql('SELECT Kod, JmenoLekare FROM OsetrujiciLekar WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'JmenoLekare');
    }


    private function addFieldsVzdelani()
    {
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-vzdelani')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('vzdelani');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode("header-vzdelani")
            ->setCaption(TextManager::_('Nejvyssi_dosazene_vzdelani'));

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)    // nastavi se podle Osoba.RC
            ->setFieldName('RC', 'VzdelaniOsoby') ->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode("field-vzdelani-rc");

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('KodVzdelani', 'VzdelaniOsoby') ->setDataType('int') ->setCaption('Vzdelani')
            ->setCode('field-kodvzdelani')
            ->setFieldId('kodVzdelani')
            ->setSelectSql('SELECT Kod, Popis, KodProCSU FROM TypVzdelani WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Kod')
            ->setAttrIndexes(['kodcsu' => 2])
            ->setFieldCssClass('form-control toggleClass toggleOnLoad')
            ->setAppendFormClass('je-vzdelani')
            ->setOtherHtmlParams([
                'data-selector' => '#formnovyzamestnanec',
                'data-not-empty-class' => 'je-vzdelani'
            ]);


        $this ->addField(FormField::FIELD_TYPE_COMBO)
            ->setFieldName('MistoStudia', 'VzdelaniOsoby') ->setDataType('int') ->setCaption('Misto_studia')
            ->setCode('field-mistostudia')
            ->setFieldId('mistoStudia')
            ->setOptions([0 => TextManager::_('Vzdelani_CR'), 1 => TextManager::_('Vzdelani_zahranici')])
            ->setDefaultValue(0);

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('OborVzdelani', 'VzdelaniOsoby') ->setCaption('Obor_vzdelani')
            ->setCode('field-oborvzdelani')
            ->setFieldCssClass('form-control ronselect')
            ->setSelectSql('SELECT Kod, Popis, KodKKOV, PopisKKOV FROM OborVzdelani WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setPreferredValues(['0000']);

        $this ->addField()
            ->setFieldName('Zamereni', 'VzdelaniOsoby') ->setCaption('Zamereni_vzdelani')
            ->setMaxLength(40)
            ->setCode('field-vzdelani-zamereni');

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumUkonceni', 'VzdelaniOsoby') ->setCaption('Datum_ukonceni_vzdelani')
            ->setCode('field-vzdelani-datumukonceni')
            ->setFieldId('datumUkonceniStudia');

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('VzdelaniUkonceno', 'VzdelaniOsoby') ->setDataType('int') ->setCaption('Zpusob_ukonceni_vzdelani')
            ->setCode('field-vzdelaniukonceno')
            ->setSelectSql('SELECT Kod, Popis FROM S_ZpusobUkonceniSkoly WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Popis');

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Skola', 'VzdelaniOsoby') ->setCaption('Skola')
            ->setCode('field-skola')
            ->setSelectSql('SELECT Kod, Nazev FROM Skola WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Nazev')
            ->setVisible(false);    // implicitne skryto, protoze ve vychozim stavu je ciselnik prazdny

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumOd', 'VzdelaniOsoby') ->setCaption('Datum_vzdelani_od')
            ->setCode('field-vzdelani-datumod')
            ->setDataColumnClass('col-12 col-sm-3');

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumDo', 'VzdelaniOsoby') ->setCaption('Datum_vzdelani_do')
            ->setNewRow(false)
            ->setCode('field-vzdelani-datumdo')
            ->setCaptionColumnClass('col-12 col-sm-1 caption')
            ->setDataColumnClass('col-12 col-sm-5');

        $this ->addField()
            ->setFieldName('OborVzdelaniISPV', 'VzdelaniOsoby') ->setCaption('Obor_vzdelani_dle_ISPV')
            ->setMaxLength(40)
            ->setCode('field-oborvzdelaniispv')
            ->setFieldId('oborVzdelaniISPV')
            ->setDataColumnClass('col12 col-sm-7');

        $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->setCode('field-vyhledatoborvzdelani')
            ->setNewRow(false) ->setText( TextManager::_('Vyhledat_obor_ISPV'))
            ->setFieldId('vyhledatVzdelaniISPV')
            ->setFieldCssClass('vyhledat-obor-vzdelani')
            ->setCaptionColumnClass('d-none') ->setDataColumnClass('col12 col-sm-2 align-right js')
            ->setOtherHtmlParams([
                'data-misto-studia-selector' => '#mistoStudia',
                'data-kod-vzdelani-selector' => '#kodVzdelani',
                'data-datum-ukonceni-selector' => '#datumUkonceniStudia',
                'data-obor-vzdelani-ispv-selector' => '#oborVzdelaniISPV'
            ]);


        //        $this ->addField(FormField::FIELD_TYPE_DATE)
//            ->setFieldName('PlatnostOd', 'ZdravotniStavOsoby') ->setCaption('Platnost_od')
//            ->setCode('field-zdravotnistav-platnostod');
//
//        $this ->addField(FormField::FIELD_TYPE_CHECK)
//            ->setFieldName('DrzitelPrukazuZTTP', 'ZdravotniStavOsoby') ->setDataType('int') ->setCaption('Drzitel_prukazu_ZTP')
//            ->setCode('field-zdravotnistav-drzitelprukazuztp');
//
//        $this ->addField(FormField::FIELD_TYPE_CHECK)
//            ->setFieldName('ZPS', 'ZdravotniStavOsoby') ->setDataType('int') ->setCaption('ZPS')
//            ->setCode('field-zdravotnistav-zps');
//
//        $this ->addField(FormField::FIELD_TYPE_CHECK)
//            ->setFieldName('ZPS_TZP', 'ZdravotniStavOsoby') ->setDataType('int') ->setCaption('ZPS_TZP')
//            ->setCode('field-zdravotnistav-zps_tzp');
//
//        $this ->addField(FormField::FIELD_TYPE_SELECT)
//            ->setFieldName('OsetrujiciLekar', 'ZdravotniStavOsoby') ->setCaption('Osetrujici_lekar')
//            ->setCode('field-osetrujicilekar')
//            ->setSelectSql('SELECT Kod, JmenoLekare FROM OsetrujiciLekar ORDER BY Kod');

    }

    private function addFieldsRodinnyStav()
    {
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-rodinnystav')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('rodinnystav');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode("header-rodinnystav")
            ->setCaption(TextManager::_('Rodinny_stav'));

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)    // nastavi se podle Osoba.RC
            ->setFieldName('RC', 'RodinnyStav') ->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode("field-rodinnystav-rc");

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('KodRodinnehoStavu', 'RodinnyStav') ->setDataType('int') ->setCaption('Rodinny_stav')
            ->setCode('field-kodrodinnehostavu')
            ->setSelectSql('SELECT Kod, Popis FROM S_RodinnyStav WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Kod')
            ->setFieldId('kodRodinnehoStavu');

        $this ->addField(new NovyZamestnanecPlatnostOdField(NovyZamestnanecPlatnostOdField::PLATNOST_TODAY))
            ->setFieldName('PlatnostOd', 'RodinnyStav') ->setCaption('Platnost_od')
            ->setCode('field-rodinnystav-platnostod')
            ->setFieldId('platnostRodinnehoStavuOd');
    }


    private function addFieldsVyplaceniMzdy()
    {
        $this->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-vyplacenimzdy')->setFieldCssClass('form-field-group margin-top-2')
            ->setFieldId('vyplacenimzdy');

        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-vyplacenimzdy')
            ->setCaption(TextManager::_('Vyplaceni_mzdy'));
        /*
        if ($this ->osobaRC  ||  $this ->idPP) {
            $params = $this ->database ->createParameters();

            $selectSql = $this ->osobaRC
                ? new SqlStatement(
                    'SELECT SerioveCislo, CisloUctu, KodBanky FROM BankovniUcet' .
                    ' WHERE RC = ' . $params ->addParameter($this ->osobaRC) .
                    ' AND (%FILTER%) ORDER BY %ORDER%',
                    $params)
                :  new SqlStatement(
                    'SELECT SerioveCislo, CisloUctu, KodBanky FROM BankovniUcet, PracovniPomer' .
                    ' WHERE PracovniPomer.RC = BankovniUcet.RC' .
                    ' AND PracovniPomer.Id = ' . $params ->addParameter($this ->idPP) .
                    ' AND (%FILTER%) ORDER BY %ORDER%',
                    $params);

            $this ->addField(FormField::FIELD_TYPE_SELECT)
                ->setFieldName('SerioveCisloUctuVyplaty', 'VyplaceniMzdy') ->setDataType('int') ->setCaption('Ucet')
                ->setCode('field-seriovecislouctuvyplaty')
                ->setSelectSql($selectSql, '1 = 1', 'CisloUctu')
                ->setAttrIndexes(['kodbanky' => 2]);
        } else {
        */
            $this ->addField(FormField::FIELD_TYPE_CONSTANT)
                ->setFieldName('SerioveCisloUctuVyplaty', 'VyplaceniMzdy') ->setDataType('int') ->setReadOnly(true)
                ->setCode('field-noveseriovecislouctuvyplaty');
        //}

//        $this ->addField() ->setFieldName('ZpusobPlatbyVyplaty', 'VyplaceniMzdy');
        $this ->addField(FormField::FIELD_TYPE_CONSTANT)    // nastavi se podle Osoba.RC
            ->setFieldName('RC', 'VyplaceniMzdy')  ->setReadOnly(true) ->setMaxLength(15)
            ->setCode('field-vyplacenimzdy-rc');

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)    // nastavi se podle PP
            ->setFieldName('OsobniCislo', 'VyplaceniMzdy') ->setReadOnly(true) ->setMaxLength(15)
            ->setCode('field-vyplacenimzdy-osobnicislo');

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)    // nastavi se podle PP
            ->setFieldName('DatumNastupu', 'VyplaceniMzdy') ->setReadOnly(true) ->setDataType('date')
            ->setCode('field-vyplacenimzdy-datumnastupu');

        $this ->addField(new NovyZamestnanecPlatnostOdField(NovyZamestnanecPlatnostOdField::PLATNOST_DLE_PP, true)) // podle zacatku mesice
            ->setFieldName('PlatnostOd', 'VyplaceniMzdy')
            ->setCode('field-vyplaceni-platnostod');

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('ZpusobPlatbyVyplaty', 'VyplaceniMzdy') ->setDataType('int') ->setDefaultValue(1) // na ucet
            ->setCode('field-vyplacenimzdy-zpusobplatbyvyplaty');

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('ZpusobPlatbyZalohy', 'VyplaceniMzdy') ->setDataType('int') ->setDefaultValue(0)
            ->setCode('field-vyplacenimzdy-zpusobplatbyzalohy');

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('DefinovatCastkuHotove', 'VyplaceniMzdy') ->setDataType('int') ->setDefaultValue(0)
            ->setCode("field-vyplacenimzdy-definovatcastkuhotove");

        $this ->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('SerioveCislo', 'BankovniUcet') ->setDataType('int') ->setReadOnly(true);

        $this ->addField()
            ->setFieldName('CisloUctu', 'BankovniUcet') ->setMaxLength(20) ->setCaption('Cislo_uctu')
            ->setCode('field-bankonvniucet-cislouctu')
            ->setDataColumnClass('col-12 col-sm-5')
            ->setOtherHtmlParams(['autocomplete' => 'off']);


        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('KodBanky', 'BankovniUcet') ->setCaption('Kod_banky')
            ->setNewRow(false)
            ->setCode('field-bankovniucet-kodbanky')
            ->setCaptionColumnClass('col-12 col-sm-2 caption')
            ->setDataColumnClass('col-12 col-sm-2')
            ->setFieldCssClass('form-control ronselect')
            ->setSelectSql('SELECT Kod, Kod, Nazev FROM Banka WHERE (%FILTER%) ORDER BY %ORDER%', '1 = 1', 'Nazev')
            ->setAttrIndexes(['data-nazev' => 2])
            ->setOtherHtmlParams(['data-columns' => 'value;nazev']);

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)    // nastavi se automaticky (hodnota 1 = ucet zamestnance)
        ->setFieldName('TypUctu', 'BankovniUcet') ->setDataType('int') ->setDefaultValue(1)
            ->setCode('field-bankovniucet-typuctu');

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)    // nastavi se podle Osoba.RC
            ->setFieldName('RC', 'BankovniUcet') ->setReadOnly(true)
            ->setMaxLength(15)
            ->setCode("field-bankovniucet-rc");

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)    // nastavi se automaticky
            ->setFieldName('JmenoUctu', 'BankovniUcet') ->setMaxLength(40) ->setReadOnly(true)
            ->setCode('field-bankovniucet-jmenouctu')
            ->setDefaultValue(TextManager::_('Maska_jmena_uctu_vyplaty'));

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)    // nastavi se automaticky
            ->setFieldName('ZahranicniPlatba', 'BankovniUcet') ->setDataType('int') ->setDefaultValue(0) ->setReadOnly(true)
            ->setCode('field-bankovniucet-zahranicniplatba');

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)    // nastavi se automaticky
            ->setFieldName('KonstantniSymbol', 'BankovniUcet') ->setDefaultValue('0138') ->setReadOnly(true)
            ->setCode('field-bankovniucet-konstantnisymbol');

        $this ->addUserLogFields('BankovniUcet');

        // 0 hotove, 1 prevodem
        // RC, Os, DatumNastupu, PlatnostOd
        // ZpusobPlatbyVyplaty: 0=hotove, 1=na ucet
        // ZpusobPlatbyZalohy=0, DefinovatCastkuHotove=0
        // ----- BankovniUcet ----
        // SerioveCislo
        // JmenoUctu[40],KodBanky[10],CisloUctu[20]
        // TypUctu: 1=ucty zamestnancu
        // RC[15]
        // SpecifickySymbol[20], KonstantniSymbol[10], VariabilniSymbol[20]
        // Insert_UserName, Insert_Timestamp
        // ZahranicniPlatba=0

    }


    private function addUserLogFields($tableName)
    {
        $code = strtolower($tableName);

        $this->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('Insert_UserName', $tableName) ->setReadOnly(true)
            ->setCode("field-{$code}-insert_username")
            ->setDefaultValue(RonPortalUtils::getLogUserName());

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Insert_Timestamp', $tableName) ->setReadOnly(true)
            ->setCode("field-{$code}-insert_timestamp");

        $this->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('Update_UserName', $tableName) ->setUpdateLog(true)
            ->setCode("field-{$code}-update_username")
            ->setDefaultValue(RonPortalUtils::getLogUserName());

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Update_Timestamp', $tableName) ->setUpdateLog(true)
            ->setCode("field-{$code}-update_timestamp");
    }


    protected function createNewFieldFromConfiguration($fieldSettings)
    {
        // pripadna nova pole pridava pouze v pripade, ze je zobrazen cely formular
        if ($this ->status === self::STATUS_RC) {
            return null;
        }

        if (($fieldSettings['fieldType'] ?? '') === 'rozdeleni') {
            $f = new RozdeleniZamestnanceField();
            $this ->rozdeleniFields[] = $f;
            return $f;
        }

        return parent::createNewFieldFromConfiguration($fieldSettings);
    }


    public function getSqlFromPart()
    {
        $params = $this ->database ->createParameters();

        // Osoba
        $sql =
            'Osoba';

        // Pracovni pomer
        if ($this ->osobaRC) {
            // existujici osoba - nenacita zadny pracovni pomer
            $sql .=
                ' LEFT OUTER JOIN PracovniPomer ON 0 = 1';
        } else {
            $sql .=
                ' INNER JOIN PracovniPomer ON PracovniPomer.RC = Osoba.RC';
        }

        // OsobniUdaj
        $sql .=
            ' LEFT OUTER JOIN OsobniUdaj ON OsobniUdaj.RC = Osoba.RC';

        // datum, ke kteremu se nacitaji platne podrizene zaznamy
        $platnostTimestamp = $this ->pp
            ? $this ->pp ->datumNastupu
            : time();

        if ($this ->jePerson) {
            $sql .=
                // Trvale bydliste
                ' LEFT OUTER JOIN AdresaOsoby TrvaleBydliste' .
                '  ON TrvaleBydliste.RC = Osoba.RC AND TrvaleBydliste.TrvaleBydliste = 1' .
                '  AND (TrvaleBydliste.PlatnostDo IS NULL OR TrvaleBydliste.PlatnostDo >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND TrvaleBydliste.PlatnostOd <= ' . $params->addDateParameter($platnostTimestamp) .
                // Prechodne bydliste
                ' LEFT OUTER JOIN AdresaOsoby PrechodneBydliste' .
                '  ON PrechodneBydliste.RC = Osoba.RC AND PrechodneBydliste.TrvaleBydliste = 0' .
                '  AND (PrechodneBydliste.PlatnostDo IS NULL OR PrechodneBydliste.PlatnostDo >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND PrechodneBydliste.PlatnostOd <= ' . $params->addDateParameter($platnostTimestamp) .
                // Zdravotni stav
                ' LEFT OUTER JOIN ZdravotniStavOsoby' .
                '  ON ZdravotniStavOsoby.RC = Osoba.RC' .
                '  AND (ZdravotniStavOsoby.PlatnostDo IS NULL OR ZdravotniStavOsoby.PlatnostDo >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND ZdravotniStavOsoby.PlatnostOd <= ' . $params->addDateParameter($platnostTimestamp) .
                // Zdravotni pojisteni
                ' LEFT OUTER JOIN PojisteniOsobyU_ZP' .
                '  ON PojisteniOsobyU_ZP.RC = Osoba.RC' .
                '  AND (PojisteniOsobyU_ZP.PlatnostDo IS NULL OR PojisteniOsobyU_ZP.PlatnostDo >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND PojisteniOsobyU_ZP.PlatnostOd <= ' . $params->addDateParameter($platnostTimestamp) .
                // Rodinny stav
                ' LEFT OUTER JOIN RodinnyStav' .
                '  ON RodinnyStav.RC = Osoba.RC' .
                '  AND (RodinnyStav.PlatnostDo IS NULL OR RodinnyStav.PlatnostDo >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND RodinnyStav.PlatnostOd <= ' . $params->addDateParameter($platnostTimestamp) .
                // VyplaceniMzdy ...
                ' LEFT OUTER JOIN VyplaceniMzdy' .
                '  ON VyplaceniMzdy.RC = PracovniPomer.RC ' .
                '  AND VyplaceniMzdy.OsobniCislo = PracovniPomer.OsobniCislo' .
                '  AND VyplaceniMzdy.DatumNastupu = PracovniPomer.DatumNastupu' .
                '  AND (VyplaceniMzdy.PlatnostDo IS NULL OR VyplaceniMzdy.PlatnostDo >= ' . $params->addDateParameter($platnostTimestamp) . ')' .
                '  AND VyplaceniMzdy.PlatnostOd <= ' . $params->addDateParameter($platnostTimestamp) .
                // ... a prislusny bankovni ucet
                ' LEFT OUTER JOIN BankovniUcet ON BankovniUcet.SerioveCislo = VyplaceniMzdy.SerioveCisloUctuVyplaty' .
                // Vzdelani osoby
                ' LEFT OUTER JOIN VzdelaniOsoby' .
                '  ON VzdelaniOsoby.RC = Osoba.RC' .
                '  AND 0 = 1';  // TODO: nacist posledni zaznam vzdelani
        }

        return new SqlStatement($sql, $params);
    }


    public function getWhereCondition()
    {
        $params = $this ->database ->createParameters();

        if ($this ->osobaRC) {
            return new SqlStatement(
                'Osoba.RC = ' . $params ->addParameter($this ->osobaRC),
                $params
            );
        } else if ($this ->idPP) {
            return new SqlStatement(
                'PracovniPomer.Id = ' . $params ->addParameter($this ->idPP),
                $params
            );
        }

        return '';
    }

    protected function renderPrimaryFields()
    {
        // Negeneruje nic, jedna se vzdy o novy zaznam
        // a pokud to je pres XML, kde je umoznena i editace, tak se editovany PP zadava v constructoru.
        // Navic implicitni hidden pole "record" se pouziva v xml
    }


    public function renderCommonFields()
    {
        parent::renderCommonFields();

        echo "<input type='hidden' name='novyZamestnanecFormStatus' value='{$this ->status}'/>";
    }



    public function renderMessages()
    {
        if (!$this ->zobrazenoZXML) {
            parent::renderMessages();
        }
    }


    public function renderButtons($insert)
    {
        if (!$this ->zobrazenoZXML) { // v XML tlacitka nezobrazuje
            parent::renderButtons($insert);
        }
    }


    /**
     * Nacte informace o osobe se zadanym rodnym cislem (OsobniUdaj.RodneCislo). Pokud jeste neexistuje, vraci null.
     * @param string $rc
     * @return array|null   Vraci asociativni pole s nactenymi hodnotami - vysvetlivky nize v return
     * @throws Exception
     * @throws \Ron\DB\DBException
     */
    private function najdiRC($rc)
    {
        $params = $this->database->createParameters();
        $sql =
            'SELECT Osoba.RC, OsobniUdaj.RodneCislo, PracovniPomer.Id' . // , PracovniPomer.DatumNastupu, PracovniPomer.DatumUkonceni' .
            ' FROM Osoba INNER JOIN OsobniUdaj' .
            '  ON OsobniUdaj.RC = Osoba.RC' .
            ' LEFT OUTER JOIN PracovniPomer' .
            '  ON PracovniPomer.RC = Osoba.RC' .
            '  AND PracovniPomer.DatumNastupu <= ' . $params->addDateParameter(time()) .
            '  AND (PracovniPomer.DatumUkonceni IS NULL  OR  PracovniPomer.DatumUkonceni > ' . $params->addDateParameter(time()) . ')' .
            ' WHERE OsobniUdaj.RodneCislo = ' . $params->addParameter($rc);

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = null;
        $dalsiPP = [];
        $idPP = null;

        while ($nextRow = $cursor->fetch()) {
            if (!$row) {
                // prvni radek si zapamatuje
                $row = $nextRow;
            }

            if (!$idPP) {
                if (!$this ->idPP  ||  (int)$nextRow[2] == $this ->idPP) {
                    $idPP = (int)$nextRow[2];
                }
            }

            if ((int)$nextRow[2] != $idPP) {
                $dalsiPP[] = (int)$nextRow[2];
            }
        }

        $cursor ->close();

        return $row
            ? [
                'rc' => $row[0],            // Osoba.RC
                'rodneCislo' => $row[1],    // rodne cislo
                'idPP' => $idPP,            // id pracovniho pomeru osoby. V pripade, ze uz je u formulare nastaven (tj. jedna se o editaci), tak vraci ten nastaveny, jinak prvni nalezeny
                'dalsiPP' => $dalsiPP       // seznam id dalsich pracovnich pomeru
            ]
            : null;
    }


    protected function nastavReadonly()
    {
        /** @var FormField $field */
        foreach ($this ->fields as $field) {
            if (in_array($field ->tableName, $this ->tablesOsoba)  ||  in_array($field ->tableAlias, $this ->tablesOsoba)) {
                // jedna se o data k osobe s aktivnim pracovnim pomerem => nastavi je readonly
                $field ->otherHtmlParams['disabled'] = 'disabled';
                $field ->setReadOnly(true); // kvuli kontroly required poli
            }
        }
    }



    protected function loadRozdeleni()
    {
        foreach ($this ->rozdeleniFields as $field) {
            $seznamRozdeleni = RozdeleniOsob::nactiRozdeleniOsoby($this ->database, $this ->idPP, $field ->kodTypuRozdeleni, null);

            if ($field ->mode == RozdeleniZamestnanceField::MODE_SELECT) {
                // najde jen prvni prirazene rozdeleni
                /** @var RozdeleniOsob $prvniRozdeleni */
                $prvniRozdeleni = null;
                foreach ($seznamRozdeleni as $rozdeleni) {
                    if (!$prvniRozdeleni  ||  $rozdeleni ->platnostOd < $prvniRozdeleni ->platnostOd) {
                        $prvniRozdeleni = $rozdeleni;
                    }
                }

                $field ->setSeznamRozdeleniOsoby($prvniRozdeleni ? [$prvniRozdeleni] : []);
            } else {
                $field ->setSeznamRozdeleniOsoby($seznamRozdeleni);
            }
        }
    }


    protected function loadFieldValues($insert, $values, $usePostValues, $canEditRecord)
    {
//        if ($usePostValues  &&  $_POST['novyZamestnanecFormStatus'] === self::STATUS_RC) {
//            // Bylo odeslano jen RC. Pro vsechna ostatni pole pouzije defaultValue.
//            foreach ($this->fields as $field) {
//                if ($field->visible) {
//                    if ($this->data) {
//                        $field ->loadEditValue($this ->database, $usePostValues, $this ->data, $canEditRecord);
//                    } else {
//                        $field ->loadInsertValue($this ->database, $usePostValues);
//                    }
//                    parent::loadFieldValues($insert, $values, false, $canEditRecord);
//                    // ... jen RC preplacne odeslanou hodnotou
//                    $this->fRC->loadInsertValue($this->database, $usePostValues);
//                    //$this ->status = self::STATUS_ALL;
//
//
//                    if ($insert) {
//                        $field->loadInsertValue($this->database, $usePostValues);
//                    } else {
//                        $field->loadEditValue($this->database, $usePostValues, $values, $canEditRecord);
//                    }
//                }
//            }
//        } else {
//            parent::loadFieldValues($insert, $values, $usePostValues, $canEditRecord);
//        }


        if ($usePostValues  &&  $_POST['novyZamestnanecFormStatus'] === self::STATUS_RC) {
            // Bylo odeslano jen RC. Pro vsechna ostatni pole pouzije defaultValue.
            parent::loadFieldValues($insert, $values, false, $canEditRecord);
            // ... jen RC preplacne odeslanou hodnotou
            $this ->fRC ->loadInsertValue($this ->database, $usePostValues);
            //$this ->status = self::STATUS_ALL;
        } else {
            parent::loadFieldValues($insert, $values, $usePostValues, $canEditRecord);

            if ($values && !$usePostValues) {
                $this ->loadRozdeleni();
            }
        }
    }


    private function overRC($rc)
    {
        // najde osobu podle zadaneho rodneho cisla
        $osobaArray = $this ->najdiRC($rc);

        if ($osobaArray && $osobaArray['idPP']) {
            // osoba ma aktivni pracovni pomer
            $prava = Prava::vratPrava('zamest_novy');
            if (!$prava->priznak) {
                // uzivatel nema pravo pridavat PP pro aktivni osoby
                throw new Exception( TextManager::_('Chyba_novy_zamestnanec_osoba_ma_aktivni_pracovni_pomer'));
            }
        }

        return $osobaArray;
    }

    /**
     * Doplni hodnoty podle zadaneho rc
     * @param string $rc
     * @return bool|int
     */
    private function handleActionRC($rc)
    {
        if ($this ->fRCError ->analyzaRC->datumNarozeni) {
            $this->fDatumNarozeni->defaultValue = StringUtils::format('date', $this ->fRCError ->analyzaRC->datumNarozeni);
        }

        if ($this ->fRCError ->analyzaRC->pohlavi) {
            $this->fPohlavi->defaultValue = $this ->fRCError ->analyzaRC->pohlavi;
        }

        if ($this ->fCisloPojistence  &&  isset($this ->fRCError ->analyzaRC)) {
            if (!$this ->fRCError ->analyzaRC ->ecp) {
                $this->fCisloPojistence->defaultValue = $rc;
            }
        }
    }

    public function loadFieldsConfiguration($fieldsConfArray)
    {
        parent::loadFieldsConfiguration($fieldsConfArray);

        if ($this ->status === self::STATUS_ALL  &&  $this ->existujeAktivniPP) {
            // osoba ma aktivni pracovni pomer
            $this->nastavReadonly();
        }
    }

    function loadConfiguration()
    {
        parent::loadConfiguration();

        // nacte navic konfiguraci z XML formulare
        if ($this ->novyZamestnanecConfiguration) {
            $this ->loadConfigurationArray($this ->novyZamestnanecConfiguration);
        }
    }


    public function handleAction()
    {
        $this ->addFieldsRodneCislo();

        try {

            if (isset($_POST) && $_POST) {
                $this->status = isset($_POST['novyZamestnanecFormStatus'])
                    ? $_POST['novyZamestnanecFormStatus']
                    : self::STATUS_RC;

                // overi zadane RC
                $rc = isset($_POST['OsobniUdaj#RodneCislo'])
                    ? trim($_POST['OsobniUdaj#RodneCislo'])
                    : null;

                // overi existenci osoby se zadanym RC v databazi
                $osobaArray = $this->overRC($rc);

                //if ($this->status === self::STATUS_ALL  ||  $this ->fRCError ->lzePokracovat()) {
                if ($this ->fRCError ->lzePokracovat()) {
                    if ($osobaArray) {
                        $this->osobaRC = $osobaArray['rc'];
                    }

                    $this->addNovyZamestnanecFields();

                    if ($this->status === self::STATUS_RC) {
                        // v predchozim kroku bylo odeslano jen RC - doplni vychozi datum narozeni atd.
                        $this->handleActionRC($rc);
                    }

                    // nastavi status na "vse" (=zobrazeni celeho formulare)
                    $this ->status = self::STATUS_ALL;

                    if ($osobaArray) {
                        $this->data = $this->loadRecord();

                        if ($osobaArray['dalsiPP']  ||  ($osobaArray['idPP']  && $osobaArray['idPP'] != $this ->idPP)) {
                            $this ->existujeAktivniPP = true;
                            MessageManager::addWarning(TextManager::_('Existuje_aktivni_pp_nelze_editovat_osobni'));
                        }
                    }
                }
            } else if ($this ->idPP) {
                $this->addNovyZamestnanecFields();
            }

            return parent::handleAction();
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            return false;
        }
    }


    /**
     * Vytvori DBRecords pro tabulku VyplaceniMzdy a pripadne BankovniUcet
     * @param PracovniPomer $pp
     * @return array
     */
    private function createUcetVyplatyRecords($pp)
    {
        $jeZadanUcet = isset($_POST['BankovniUcet#CisloUctu'])  &&  $_POST['BankovniUcet#CisloUctu'] != '';

        $result = [];

        if ($jeZadanUcet) {
            $serioveCislo = null;

            $osobaRC = $this ->osobaRC;

            if ($pp) {
                $aktualniSerioveCislo = ZamestnanciUtils::najdiBankovniUcetPP($this ->database, $pp);
                $pocetPouzitiUctu = $aktualniSerioveCislo
                    ? ZamestnanciUtils::pocetPouzitiBankovnihoUctu($this ->database, $aktualniSerioveCislo)
                    : 0;

                if ($aktualniSerioveCislo  &&  $pocetPouzitiUctu <= 1) {
                    // Ucet pro vyplatu uz je prirazen => je potreba provest aktualizaci
                    // Ucet je pouzit jen u editovaneho PP => muze se provest update
                    $recBankovniUcet = $this->createDBRecord('BankovniUcet');
                    $this->createDBRecordFieldsForTable($recBankovniUcet, 'BankovniUcet', true);
                    $params = $this->database->createParameters();
                    $recBankovniUcet->setWhere('SerioveCislo = ' . $params->addParameter($aktualniSerioveCislo), $params);
                    $result['BankovniUcet'] = $recBankovniUcet;

                    return $result;
                } else {
                    // Ucet jeste neni vubec prirazen, nebo je aktualni ucet pouzit i u jineho PP ($pocetPouzitiUctu > 1)
                    $osobaRC = $pp ->rc;
                }
            }

            // zjisti, jestli uz k zamestnanci existuje stejny ucet (pri zadavani noveho PP k existujici osobe)
            if ($osobaRC) {
                $serioveCislo = ZamestnanciUtils::najdiBankovniUcetOsoby($this ->database, $osobaRC, $_POST['BankovniUcet#CisloUctu'], $_POST['BankovniUcet#KodBanky']);
            }

            if (!$serioveCislo) {
                // nenasel existujici ucet => vytvori novy
                $recBankovniUcet = $this->createDBRecord('BankovniUcet');
                $this->createDBRecordFieldsForTable($recBankovniUcet, 'BankovniUcet', true);
                $recBankovniUcet ->setWhere('0 = 1'); // aby sel record pouzit i pri update
                $result['BankovniUcet'] = $recBankovniUcet;
            }

            $recVyplaceni = $this ->createDBRecord('VyplaceniMzdy');
            $this ->createDBRecordFieldsForTable($recVyplaceni, 'VyplaceniMzdy', true);
            if ($pp) {
                $params = $this ->database ->createParameters();
                $recVyplaceni->setWhere(
                    'RC = ' . $params ->addParameter($pp ->rc) .
                    ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
                    ' AND DatumNastupu = ' . $params ->addDateParameter($pp ->datumNastupu),    // platnost by nemela byt potreba resit - jedna se o novy pp => prepoklada se, ze bude jen jeden zaznam
                    $params);
            }
            if ($serioveCislo) {
                // uzivatel zadal ucet, ktery uz ma osoba prirazen u jineho PP
                $recVyplaceni['SerioveCisloUctuVyplaty'] = $serioveCislo;
            }
            $result['VyplaceniMzdy'] = $recVyplaceni;
        } else {
            // TODO: smazat, pokud byl uz zadan drive
        }

        return $result;
    }


    /**
     * @param PracovniPomer $pp
     * @return \Ron\DB\DBRecord[]
     */
    private function createZamestnanecRecords($pp = null)
    {
        $result = [];


        if (!$this ->existujeAktivniPP) {
            $recOsoba = $this->createDBRecord('Osoba');
            $this->createDBRecordFieldsForTable($recOsoba, 'Osoba', true);
            if ($pp) {
                $params = $this->database->createParameters();
                $recOsoba->setWhere('RC = ' . $params->addParameter($pp->rc), $params);
            }

            $result['Osoba'] = $recOsoba;
        }

        $recPP = $this ->createDBRecord('PracovniPomer');
        $this ->createDBRecordFieldsForTable($recPP, 'PracovniPomer', true);
        if ($pp) {
            $params = $this ->database ->createParameters();
            $recPP ->setWhere(
                'RC = ' . $params ->addParameter($pp ->rc) .
                ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
                ' AND DatumNastupu = ' . $params ->addDateParameter($pp ->datumNastupu),
                $params);
        }

        $result['PracovniPomer'] = $recPP;

        $datumNastupu = $recPP['DatumNastupu'];

        foreach ($this ->fields as $formField) {
            if ($formField instanceof NovyZamestnanecPlatnostOdField) {
                $formField ->nastavPodleDatumuNastupu($datumNastupu);
            }
        }

//        $recOsobniUdaj = $this->createDBRecord('OsobniUdaj');
//        $this ->createDBRecordFieldsForTable($recOsobniUdaj, 'OsobniUdaj', true);

        if (!$this ->existujeAktivniPP) {
            foreach ($this->tablesOsoba as $tableName) {
                $sqlTableName = null;

                switch ($tableName) {
                    case 'Osoba' :
                    case 'PracovniPomer' :
                        // uz bylo vytvoreno
                        break;

                    case 'TrvaleBydliste' :
                        $sqlTableName = 'AdresaOsoby';
                        break;

                    case 'PrechodneBydliste' :
                        if (isset($_POST['prechodnebydliste_existuje']) && $_POST['prechodnebydliste_existuje']) {
                            // prechodne bydliste - jen kdyz je zaskrtnuto "... je rozdilne"
                            $sqlTableName = 'AdresaOsoby';
                        }
                        break;

                    case 'ZdravotniStavOsoby' :
                        if (isset($_POST['ZdravotniStavOsoby#ZdravotniStav']) && $_POST['ZdravotniStavOsoby#ZdravotniStav'] !== '') {
                            // zdravotni stav - jen kdyz je vyplneno Zdravotni_stav
                            $sqlTableName = $tableName;
                        }
                        break;

                    case 'PojisteniOsobyU_ZP' :
                        if (isset($_POST['PojisteniOsobyU_ZP#KodZdravotniPojistovny']) && $_POST['PojisteniOsobyU_ZP#KodZdravotniPojistovny']) {
                            // zdravotni pojistovna - jen kdyz je vybrana pojistovna
                            $sqlTableName = $tableName;
                        }
                        break;

                    case 'VzdelaniOsoby' :
                        if (isset($_POST['VzdelaniOsoby#KodVzdelani']) && $_POST['VzdelaniOsoby#KodVzdelani']) {
                            // vzdelani osoby - jen kdyz je vybran kod vzdelani
                            $sqlTableName = $tableName;
                        }
                        break;

                    case 'RodinnyStav' :
                        if (isset($_POST['RodinnyStav#KodRodinnehoStavu']) && $_POST['RodinnyStav#KodRodinnehoStavu'] !== '') {
                            // zdravotni pojistovna - jen kdyz je zadan kod stavu
                            $sqlTableName = $tableName;
                        }
                        break;

                    case 'BankovniUcet' :
                    case 'VyplaceniMzdy' :
                        // vytvari se samostatne
                        break;

                    default :
                        $sqlTableName = $tableName;
                }


                if ($sqlTableName) {
                    $rec = $this->createDBRecord($sqlTableName);
                    $this->createDBRecordFieldsForTable($rec, $tableName, true);
                    if ($pp) {
                        $params = $this->database->createParameters();
                        $rec->setWhere(
                            'RC = ' . $params->addParameter($pp->rc),
                            $params
                        );

                        // TODO: doplnit filtr na PlatnostOd a PlatnostDo
                    }
                    $result[$tableName] = $rec;
                }
            }
        }

        $ucetRecords = $this ->createUcetVyplatyRecords($pp);
        if ($ucetRecords) {
            $result = array_merge($result, $ucetRecords);
        }

        return $result;
    }


    private function nactiUdajeOsoby($rc)
    {
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT Jmeno, Prijmeni, Titul FROM Osoba' .
            ' WHERE RC = ' . $params ->addParameter($rc),
            $params
        );

        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            // nemelo by nastat
            throw new Exception('Chyba ve funkci nactiUdajeOsoby - osoba nenalezena!');
        }

        return [
            'Jmeno' => $row[0],
            'Prijmeni' => $row[1],
            'Titul' => $row[2]
        ];
    }


    /**
     * @return bool
     * @throws Exception
     */
    public function insertRecord()
    {
        if ($_POST['novyZamestnanecFormStatus'] === self::STATUS_RC) {
            // bylo odeslano jen RC => zobrazi znovu formular
            // zpracovani odeslanych udaju bylo provedeno v handleActionRC
            return false;
        }

        if (!$this->checkRequiredFields(false)) {
            return false;
        }

        $records = $this ->createZamestnanecRecords();
        // TODO: Pridat zaznamy pro rozdeleni pro moznost zpracovani v XML! ! !

        RonPortalXMLForm::xmlEvents($this ->database, 'NOVY_ZAMESTNANEC_BEFORE_POST', $records, 'insert', false, true);

        $this ->database ->beginTransaction();
        try {
            // Pokud se jedna o novou osobu, tak vygeneruje nove ID osoby
            $rc = $this ->osobaRC ?: ZamestnanciUtils::getIdOsoby($this ->database);
            $recPP = $records['PracovniPomer'];
            $recOsoba = $records['Osoba'];

            $serioveCisloUctu = null;

            foreach ($records as $tableName => $record) {
                $record['RC'] = $rc;

                if ($record != $recPP  &&  $f = $record ->findFieldByName('OsobniCislo')) {
                    // u tabulek napojenych na pracovni pomer doplni OsobniCislo a DatumNastupu
                    $f ->setNewValue($recPP['OsobniCislo']);
                    $record['DatumNastupu'] = $recPP['DatumNastupu'];
                }

                if (strtolower($tableName) === 'vyplacenimzdy' && isset($serioveCisloUctu)) {
                    //echo("Nastavuji seriove cislo uctu $serioveCisloUctu <br/>");
//                    var_dump($record);
                    $record['SerioveCisloUctuVyplaty'] = $serioveCisloUctu;
                }

                if (strtolower($tableName) === 'bankovniucet'  &&  $f = $record ->findFieldByName('JmenoUctu')) {
                    if (!$recOsoba) {
                        $recOsoba = $this ->nactiUdajeOsoby($rc);
                    }
                    // nastavi jmeno uctu podle jmena osoby
                    $value = str_replace(
                        ['%JMENO%', '%PRIJMENI%', '%TITUL%', '%OSOBNICISLO%'],
                        [$recOsoba['Jmeno'], $recOsoba['Prijmeni'], $recOsoba['Titul'], $recPP['OsobniCislo']],
                        $f ->value
                    );

                    $f ->setNewValue($value);
                }

                $record ->insertRecord();

                if (strtolower($tableName) === 'bankovniucet') {
                    // zaznamena si nove seriove cislo bankovniho uctu
                    $serioveCisloUctu = $record['SerioveCislo'];
                }
            }

            foreach ($this ->rozdeleniFields as $rozdeleniField) {
                $rozdeleniField ->ulozRozdeleni($this ->database, $rc, $recPP['OsobniCislo'], $recPP['DatumNastupu'], $this);
            }

            $this ->database ->commitTransaction();

            $this ->idPP = $records['PracovniPomer']['Id'];
        } catch (Exception $e) {
            $this ->database ->rollbackTransaction();
            throw $e;
        }

        RonPortalXMLForm::xmlEvents($this ->database, 'NOVY_ZAMESTNANEC_AFTER_POST', $records, 'insert', false, true);

        return true;
    }


    /**
     * Editace existujicich udaju, pripadne doplneni
     * @return bool
     * @throws Exception
     * @throws \Ron\DB\DBException
     */
    public function editRecord()
    {
        if (!$this ->pp) {
            throw new Exception('Chyba v metode NovyZamestnanecForm::editRecord - neni zadan idPP!');
        }

        if ($_POST['novyZamestnanecFormStatus'] === self::STATUS_RC) {
            // nemelo by nastat (bylo odeslano jen RC)
            return false;
        }

        if (!$this->checkRequiredFields(false)) {
            return false;
        }

        //$pp = PracovniPomer::nactiPP($this ->database, $this ->idPP);

        $records = $this ->createZamestnanecRecords($this ->pp);

        RonPortalXMLForm::xmlEvents($this ->database, 'NOVY_ZAMESTNANEC_BEFORE_POST', $records, 'update', false, true);

        $serioveCisloUctu = null;

        foreach ($records as $tableName => $record) {
            if (strtolower($tableName) === 'vyplacenimzdy' && isset($serioveCisloUctu)) {
                $record['SerioveCisloUctuVyplaty'] = $serioveCisloUctu;
            }

            if (strtolower($tableName) ===  'pracovnipomer') {
                // mohlo dojit k editaci osobniho cisla a datumu nastupu => zaznamena zmenu
                if ($record['OsobniCislo']) {
                    $this ->pp ->osobniCislo = $record['OsobniCislo'];
                }

                if ($record['DatumNastupu']) {
                    $this ->pp ->datumNastupu = $record['DatumNastupu'];
                }
            }

            if ($record->loadRecord()) {
                $record->updateRecord();
            } else {
                // jedna se o novy podrizeny zaznam
                $record['RC'] = $this ->pp ->rc;

                if ($f = $record ->findFieldByName('OsobniCislo')) {
                    // u tabulek napojenych na pracovni pomer doplni OsobniCislo a DatumNastupu
                    $f ->setNewValue($this ->pp ->osobniCislo);
                    $record['DatumNastupu'] = $this ->pp ->datumNastupu;
                }

                // TODO: nastavit jmeno bankovniho uctu
                $record->insertRecord();

                if (strtolower($tableName) === 'bankovniucet') {
                    // zaznamena si nove seriove cislo bankovniho uctu (pro pripadne vlozeni do zaznamu VyplaceniMzdy)
                    $serioveCisloUctu = $record['SerioveCislo'];
                }
            }
        }

        foreach ($this ->rozdeleniFields as $rozdeleniField) {
            $rozdeleniField ->ulozRozdeleni($this ->database, $this ->pp ->rc, $this ->pp ->osobniCislo, $this ->pp ->datumNastupu, $this);
        }
    }


}
